@@ -14,30 +14,30 @@ struct _Evas_Cache_Image_Func
    void         (*dealloc)(Image_Entry *im);
 
    /* The cache provide some helpers for surface manipulation. */
-   int          (*surface_alloc)(Image_Entry *im, int w, int h);
+   int          (*surface_alloc)(Image_Entry *im, unsigned int w, unsigned int h);
    void         (*surface_delete)(Image_Entry *im);
    DATA32      *(*surface_pixels)(Image_Entry *im);
 
    /* The cache is doing the allocation and deallocation, you must just do the rest. */
-   int          (*constructor)(Image_Entry *im);
+   int          (*constructor)(Image_Entry *im); /**< return is EVAS_LOAD_ERROR_* or EVAS_LOAD_ERROR_NONE! */
    void         (*destructor)(Image_Entry *im);
 
-   void         (*dirty_region)(Image_Entry *im, int x, int y, int w, int h);
+   void         (*dirty_region)(Image_Entry *im, unsigned int x, unsigned int y, unsigned int w, unsigned int h);
    /* Only called when references > 0. Need to provide a fresh copie of im. */
    /* The destination surface does have a surface, but no allocated pixel data. */
    int          (*dirty)(Image_Entry *dst, const Image_Entry *src);
    /* Only called when references == 1. We will call drop on `im'. */
    /* The destination surface does not have any surface. */
-   int          (*size_set)(Image_Entry *dst, const Image_Entry *src, int w, int h);
+   int          (*size_set)(Image_Entry *dst, const Image_Entry *src, unsigned int w, unsigned int h);
 
    /* The destination surface does not have any surface. */
-   int          (*copied_data)(Image_Entry *dst, int w, int h, DATA32 *image_data, int alpha, int cspace);
+   int          (*copied_data)(Image_Entry *dst, unsigned int w, unsigned int h, DATA32 *image_data, int alpha, int cspace);
    /* The destination surface does not have any surface. */
-   int          (*data)(Image_Entry *dst, int w, int h, DATA32 *image_data, int alpha, int cspace);
+   int          (*data)(Image_Entry *dst, unsigned int w, unsigned int h, DATA32 *image_data, int alpha, int cspace);
    int          (*color_space)(Image_Entry *dst, int cspace);
 
    /* This function need to update im->w and im->h. */
-   int          (*load)(Image_Entry *im);
+   int          (*load)(Image_Entry *im); /**< return is EVAS_LOAD_ERROR_* or EVAS_LOAD_ERROR_NONE! */
    int          (*mem_size_get)(Image_Entry *im);
    void         (*debug)(const char *context, Image_Entry *im);
 };
@@ -46,6 +46,9 @@ struct _Evas_Cache_Image
 {
    Evas_Cache_Image_Func         func;
 
+   Eina_List			*preload;
+   Eina_List			*pending;
+
    Eina_Inlist                  *dirty;
 
    Eina_Inlist                  *lru;
@@ -55,8 +58,11 @@ struct _Evas_Cache_Image
    void                         *data;
 
    int                           usage;
-   int                           limit;
+   unsigned int                  limit;
    int                           references;
+#ifdef EVAS_FRAME_QUEUING
+   LK(lock);
+#endif
 };
 
 struct _Evas_Cache_Engine_Image_Func
@@ -70,7 +76,7 @@ struct _Evas_Cache_Engine_Image_Func
    int                  (*constructor)(Engine_Image_Entry *eim, void* data);
    void                 (*destructor)(Engine_Image_Entry *eim);
 
-   void                 (*dirty_region)(Engine_Image_Entry *eim, int x, int y, int w, int h);
+   void                 (*dirty_region)(Engine_Image_Entry *eim, unsigned int x, unsigned int y, unsigned int w, unsigned int h);
    /* Only called when references > 0. Need to provide a fresh copie of im. */
    int                  (*dirty)(Engine_Image_Entry *dst, const Engine_Image_Entry *src);
    /* Only called when references == 1. We will call drop on `im'. */
@@ -111,7 +117,6 @@ extern "C" {
 EAPI Evas_Cache_Image*        evas_cache_image_init(const Evas_Cache_Image_Func *cb);
 EAPI void                     evas_cache_image_shutdown(Evas_Cache_Image *cache);
 EAPI Image_Entry*             evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *key, RGBA_Image_Loadopts *lo, int *error);
-EAPI void                     evas_cache_pending_process(void);
 EAPI void                     evas_cache_image_drop(Image_Entry *im);
 EAPI void                     evas_cache_image_data_not_needed(Image_Entry *im);
 EAPI int                      evas_cache_image_flush(Evas_Cache_Image *cache);
@@ -121,18 +126,21 @@ EAPI void*                    evas_cache_private_from_image_entry_get(Image_Entr
 
 EAPI int                      evas_cache_image_usage_get(Evas_Cache_Image *cache);
 EAPI int                      evas_cache_image_get(Evas_Cache_Image *cache);
-EAPI void                     evas_cache_image_set(Evas_Cache_Image *cache, int size);
+EAPI void                     evas_cache_image_set(Evas_Cache_Image *cache, unsigned int size);
 
 EAPI Image_Entry*             evas_cache_image_alone(Image_Entry *im);
-EAPI Image_Entry*             evas_cache_image_dirty(Image_Entry *im, int x, int y, int w, int h);
+EAPI Image_Entry*             evas_cache_image_dirty(Image_Entry *im, unsigned int x, unsigned int y, unsigned int w, unsigned int h);
 EAPI void                     evas_cache_image_load_data(Image_Entry *im);
-EAPI void                     evas_cache_image_surface_alloc(Image_Entry *im, int w, int h);
+EAPI void                     evas_cache_image_unload_data(Image_Entry *im);
+EAPI Eina_Bool                evas_cache_image_is_loaded(Image_Entry *im);
+EAPI void                     evas_cache_image_unload_all(Evas_Cache_Image *cache);
+EAPI void                     evas_cache_image_surface_alloc(Image_Entry *im, unsigned int w, unsigned int h);
 EAPI DATA32*                  evas_cache_image_pixels(Image_Entry *im);
-EAPI Image_Entry*             evas_cache_image_copied_data(Evas_Cache_Image *cache, int w, int h, DATA32 *image_data, int alpha, int cspace);
-EAPI Image_Entry*             evas_cache_image_data(Evas_Cache_Image *cache, int w, int h, DATA32 *image_data, int alpha, int cspace);
+EAPI Image_Entry*             evas_cache_image_copied_data(Evas_Cache_Image *cache, unsigned int w, unsigned int h, DATA32 *image_data, int alpha, int cspace);
+EAPI Image_Entry*             evas_cache_image_data(Evas_Cache_Image *cache, unsigned int w, unsigned int h, DATA32 *image_data, int alpha, int cspace);
 EAPI void                     evas_cache_image_colorspace(Image_Entry *im, int cspace);
 EAPI Image_Entry*             evas_cache_image_empty(Evas_Cache_Image *cache);
-EAPI Image_Entry*             evas_cache_image_size_set(Image_Entry *im, int w, int h);
+EAPI Image_Entry*             evas_cache_image_size_set(Image_Entry *im, unsigned int w, unsigned int h);
 
 EAPI Evas_Cache_Engine_Image* evas_cache_engine_image_init(const Evas_Cache_Engine_Image_Func *cb, Evas_Cache_Image *parent);
 EAPI void                     evas_cache_engine_image_shutdown(Evas_Cache_Engine_Image *cache);
@@ -146,17 +154,19 @@ EAPI void                     evas_cache_engine_parent_not_needed(Engine_Image_E
 EAPI Engine_Image_Entry*      evas_cache_engine_image_engine(Evas_Cache_Engine_Image *cache, void *engine_data);
 EAPI void                     evas_cache_engine_image_drop(Engine_Image_Entry *eim);
 EAPI Engine_Image_Entry*      evas_cache_engine_image_alone(Engine_Image_Entry *eim, void *data);
-EAPI Engine_Image_Entry*      evas_cache_engine_image_dirty(Engine_Image_Entry *eim, int x, int y, int w, int h);
-EAPI Engine_Image_Entry*      evas_cache_engine_image_copied_data(Evas_Cache_Engine_Image *cache, int w, int h, DATA32 *image_data, int alpha, int cspace, void *engine_data);
-EAPI Engine_Image_Entry*      evas_cache_engine_image_data(Evas_Cache_Engine_Image *cache, int w, int h, DATA32 *image_data, int alpha, int cspace, void *engine_data);
+EAPI Engine_Image_Entry*      evas_cache_engine_image_dirty(Engine_Image_Entry *eim, unsigned int x, unsigned int y, unsigned int w, unsigned int h);
+EAPI Engine_Image_Entry*      evas_cache_engine_image_copied_data(Evas_Cache_Engine_Image *cache, unsigned int w, unsigned int h, DATA32 *image_data, int alpha, int cspace, void *engine_data);
+EAPI Engine_Image_Entry*      evas_cache_engine_image_data(Evas_Cache_Engine_Image *cache, unsigned int w, unsigned int h, DATA32 *image_data, int alpha, int cspace, void *engine_data);
 EAPI void                     evas_cache_engine_image_colorspace(Engine_Image_Entry *eim, int cspace, void *engine_data);
-EAPI Engine_Image_Entry*      evas_cache_engine_image_size_set(Engine_Image_Entry *eim, int w, int h);
+EAPI Engine_Image_Entry*      evas_cache_engine_image_size_set(Engine_Image_Entry *eim, unsigned int w, unsigned int h);
 
 EAPI void                     evas_cache_engine_image_load_data(Engine_Image_Entry *eim);
 
 EAPI void                     evas_cache_image_preload_data(Image_Entry *im, const void *target);
 EAPI void                     evas_cache_image_preload_cancel(Image_Entry *im, const void *target);
 
+EAPI void                     evas_cache_image_wakeup(void);
+
 #ifdef __cplusplus
 }
 #endif
