@@ -1,7 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #include <assert.h>
 
 #include "evas_common.h"
@@ -201,7 +197,7 @@ evas_cache_engine_image_dup(const Evas_Cache_Engine_Image_Func *cb, Evas_Cache_E
 {
    Evas_Cache_Engine_Image     *new;
 
-   new = malloc(sizeof (Evas_Cache_Engine_Image));
+   new = calloc(1, sizeof (Evas_Cache_Engine_Image));
    if (!new)
      return NULL;
 
@@ -401,7 +397,7 @@ evas_cache_engine_image_drop(Engine_Image_Entry *eim)
 }
 
 EAPI Engine_Image_Entry *
-evas_cache_engine_image_dirty(Engine_Image_Entry *eim, int x, int y, int w, int h)
+evas_cache_engine_image_dirty(Engine_Image_Entry *eim, unsigned int x, unsigned int y, unsigned int w, unsigned int h)
 {
    Engine_Image_Entry           *eim_dirty = eim;
    Image_Entry                  *im_dirty = NULL;
@@ -539,7 +535,7 @@ _evas_cache_engine_image_push_dirty(Evas_Cache_Engine_Image *cache, Image_Entry
 }
 
 EAPI Engine_Image_Entry *
-evas_cache_engine_image_copied_data(Evas_Cache_Engine_Image *cache, int w, int h, DATA32 *image_data, int alpha, int cspace, void *engine_data)
+evas_cache_engine_image_copied_data(Evas_Cache_Engine_Image *cache, unsigned int w, unsigned int h, DATA32 *image_data, int alpha, int cspace, void *engine_data)
 {
    Image_Entry           *im;
 
@@ -551,7 +547,7 @@ evas_cache_engine_image_copied_data(Evas_Cache_Engine_Image *cache, int w, int h
 }
 
 EAPI Engine_Image_Entry *
-evas_cache_engine_image_data(Evas_Cache_Engine_Image *cache, int w, int h, DATA32 *image_data, int alpha, int cspace, void *engine_data)
+evas_cache_engine_image_data(Evas_Cache_Engine_Image *cache, unsigned int w, unsigned int h, DATA32 *image_data, int alpha, int cspace, void *engine_data)
 {
    Image_Entry           *im;
 
@@ -563,7 +559,7 @@ evas_cache_engine_image_data(Evas_Cache_Engine_Image *cache, int w, int h, DATA3
 }
 
 EAPI Engine_Image_Entry *
-evas_cache_engine_image_size_set(Engine_Image_Entry *eim, int w, int h)
+evas_cache_engine_image_size_set(Engine_Image_Entry *eim, unsigned int w, unsigned int h)
 {
    Evas_Cache_Engine_Image      *cache;
    Engine_Image_Entry           *new;
@@ -671,14 +667,10 @@ evas_cache_engine_image_engine(Evas_Cache_Engine_Image *cache, void *engine_data
 
   on_error:
    if (!eim)
-     {
-        if (ie)
-          evas_cache_image_drop(ie);
-     }
+     evas_cache_image_drop(ie);
    else
-     {
-        evas_cache_engine_image_drop(eim);
-     }
+     evas_cache_engine_image_drop(eim);
+
    return NULL;
 }
 
