@@ -1,7 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -47,12 +43,51 @@ static void _evas_cache_image_entry_preload_remove(Image_Entry *ie, const void *
        eina_stringshare_del(Var);  \
        Var = NULL;                 \
     }
+//#define CACHEDUMP 1
+
+#ifdef CACHEDUMP
+static void
+_dump_img(Image_Entry *im, const char *type)
+{
+  printf("%s: %4i: %4ikb, %4ix%4i alloc[%4ix%4i] [%s] [%s]\n",
+         type,
+         im->references,
+         (im->allocated.w * im->allocated.h * 4) / 1024,
+         im->w, im->h, im->allocated.w, im->allocated.h,
+         im->file, im->key);
+}
+
+static Eina_Bool
+_dump_cache_active(__UNUSED__ const Eina_Hash *hash, __UNUSED__ const void *key, void *data, void *fdata __UNUSED__)
+{
+  Image_Entry *im = data;
+ 
+  _dump_img(im, "ACTIVE");
+   return EINA_TRUE;
+}
+
+static void
+_dump_cache(Evas_Cache_Image *cache)
+{
+  Image_Entry *im;
+  
+  printf("--CACHE DUMP----------------------------------------------------\n");
+  printf("%cache: %ikb / %ikb\n",
+         cache->usage / 1024,
+         cache->limit / 1024);
+  printf("................................................................\n");
+  EINA_INLIST_FOREACH(cache->lru_nodata, im)
+    _dump_img(im, "NODATA");
+  EINA_INLIST_FOREACH(cache->lru, im)
+    _dump_img(im, "DATA  ");
+  eina_hash_foreach(cache->activ, _dump_cache_active, NULL);
+}
+#endif
 
 static void _evas_cache_image_entry_delete(Evas_Cache_Image *cache, Image_Entry *ie);
 
 static void
-_evas_cache_image_make_dirty(Evas_Cache_Image *cache,
-                             Image_Entry *im)
+_evas_cache_image_make_dirty(Evas_Cache_Image *cache, Image_Entry *im)
 {
    im->flags.cached = 1;
    im->flags.dirty = 1;
@@ -75,12 +110,11 @@ _evas_cache_image_make_dirty(Evas_Cache_Image *cache,
 
 static void
 _evas_cache_image_make_activ(Evas_Cache_Image *cache,
-                             Image_Entry *im,
-                             const char *key)
+                             Image_Entry *im, const char *key)
 {
    /* FIXME: Handle case when image is being processed anyway and don't do a double decode. */
    im->cache_key = key;
-   if (key != NULL)
+   if (key)
      {
         im->flags.cached = 1;
         im->flags.activ = 1;
@@ -102,8 +136,7 @@ _evas_cache_image_make_activ(Evas_Cache_Image *cache,
 
 static void
 _evas_cache_image_make_inactiv(Evas_Cache_Image *cache,
-                               Image_Entry *im,
-                               const char *key)
+                               Image_Entry *im, const char *key)
 {
    if (im->cache_key)
      {
@@ -127,8 +160,7 @@ _evas_cache_image_make_inactiv(Evas_Cache_Image *cache,
  }
 
 static void
-_evas_cache_image_remove_lru_nodata(Evas_Cache_Image *cache,
-                                    Image_Entry *im)
+_evas_cache_image_remove_lru_nodata(Evas_Cache_Image *cache, Image_Entry *im)
 {
    if (im->flags.lru_nodata)
      {
@@ -145,8 +177,7 @@ _evas_cache_image_remove_lru_nodata(Evas_Cache_Image *cache,
 }
 
 static void
-_evas_cache_image_activ_lru_nodata(Evas_Cache_Image *cache,
-                                   Image_Entry *im)
+_evas_cache_image_activ_lru_nodata(Evas_Cache_Image *cache, Image_Entry *im)
 {
    im->flags.need_data = 0;
    im->flags.lru_nodata = 1;
@@ -161,8 +192,7 @@ _evas_cache_image_activ_lru_nodata(Evas_Cache_Image *cache,
 }
 
 static void
-_evas_cache_image_remove_activ(Evas_Cache_Image *cache,
-                               Image_Entry *ie)
+_evas_cache_image_remove_activ(Evas_Cache_Image *cache, Image_Entry *ie)
 {
    if (ie->flags.cached)
      {
@@ -205,8 +235,7 @@ _evas_cache_image_remove_activ(Evas_Cache_Image *cache,
 static void
 _evas_cache_image_entry_delete(Evas_Cache_Image *cache, Image_Entry *ie)
 {
-   if (!ie)
-   	return ;
+   if (!ie) return;
 
    if (cache->func.debug)
      cache->func.debug("deleting", ie);
@@ -224,10 +253,10 @@ _evas_cache_image_entry_delete(Evas_Cache_Image *cache, Image_Entry *ie)
      }
 #endif
 
-   cache->func.destructor(ie);
-
    _evas_cache_image_remove_activ(cache, ie);
 
+   cache->func.destructor(ie);
+
    if (ie->cache_key)
      {
         eina_stringshare_del(ie->cache_key);
@@ -243,6 +272,7 @@ _evas_cache_image_entry_delete(Evas_Cache_Image *cache, Image_Entry *ie)
 
 #ifdef BUILD_ASYNC_PRELOAD
    LKD(ie->lock);
+   LKD(ie->lock_cancel);
 #endif
 #ifdef EVAS_FRAME_QUEUING
    LKD(ie->lock_references);
@@ -307,6 +337,7 @@ _evas_cache_image_entry_new(Evas_Cache_Image *cache,
 
 #ifdef BUILD_ASYNC_PRELOAD
    LKI(ie->lock);
+   LKI(ie->lock_cancel); 
    ie->targets = NULL;
    ie->preload = NULL;
    ie->flags.delete_me = 0;
@@ -333,8 +364,8 @@ _evas_cache_image_entry_new(Evas_Cache_Image *cache,
 static void
 _evas_cache_image_entry_surface_alloc__locked(Evas_Cache_Image *cache,
 					      Image_Entry *ie,
-					      int wmin,
-					      int hmin)
+					      unsigned int wmin,
+                                              unsigned int hmin)
 {
    if (ie->allocated.w == wmin && ie->allocated.h == hmin)
      return ;
@@ -353,9 +384,7 @@ _evas_cache_image_entry_surface_alloc__locked(Evas_Cache_Image *cache,
 
 static void
 _evas_cache_image_entry_surface_alloc(Evas_Cache_Image *cache,
-                                      Image_Entry *ie,
-                                      int w,
-                                      int h)
+                                      Image_Entry *ie, int w, int h)
 {
    int  wmin;
    int  hmin;
@@ -404,6 +433,18 @@ _evas_cache_image_async_heavy(void *data)
      }
 
    current->channel = pchannel;
+   
+   // check the unload cancel flag
+   LKL(current->lock_cancel);
+   if (current->unload_cancel)
+     {
+        current->unload_cancel = EINA_FALSE;
+        cache->func.surface_delete(current);
+        current->flags.loaded = 0;
+        current->flags.preload_done = 0;
+     }
+   LKU(current->lock_cancel);
+   
    LKU(current->lock);
 }
 
@@ -466,8 +507,7 @@ _evas_cache_image_async_cancel(void *data)
 }
 
 static int
-_evas_cache_image_entry_preload_add(Image_Entry *ie,
-				    const void *target)
+_evas_cache_image_entry_preload_add(Image_Entry *ie, const void *target)
 {
    Evas_Cache_Target *tg;
 
@@ -525,7 +565,7 @@ _evas_cache_image_entry_preload_remove(Image_Entry *ie, const void *target)
 	  }
      }
 
-   if (ie->targets == NULL && ie->preload && !ie->flags.pending)
+   if (!ie->targets && ie->preload && !ie->flags.pending)
      {
 	ie->cache->preload = eina_list_remove(ie->cache->preload, ie);
 	ie->cache->pending = eina_list_append(ie->cache->pending, ie);
@@ -554,7 +594,7 @@ evas_cache_image_get(Evas_Cache_Image *cache)
 }
 
 EAPI void
-evas_cache_image_set(Evas_Cache_Image *cache, int limit)
+evas_cache_image_set(Evas_Cache_Image *cache, unsigned int limit)
 {
    assert(cache != NULL);
 #ifdef EVAS_FRAME_QUEUING
@@ -577,11 +617,10 @@ evas_cache_image_set(Evas_Cache_Image *cache, int limit)
 EAPI Evas_Cache_Image *
 evas_cache_image_init(const Evas_Cache_Image_Func *cb)
 {
-   Evas_Cache_Image    *new;
+   Evas_Cache_Image *new;
 
    new = malloc(sizeof (Evas_Cache_Image));
-   if (!new)
-     return NULL;
+   if (!new) return NULL;
 
    new->func = *cb;
 
@@ -743,7 +782,7 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
         (lo->scale_down_by == 0) &&
         (lo->dpi == 0.0) &&
         ((lo->w == 0) || (lo->h == 0)) &&
-        ((lo->region.w == 0) || (lo->region.w == 0))
+        ((lo->region.w == 0) || (lo->region.h == 0))
         ))
      {
         lo = &prevent;
@@ -806,8 +845,8 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
 #endif
    if (im)
      {
-        time_t  t;
-        int     ok;
+        time_t t;
+        int ok;
 
         ok = 1;
         t = time(NULL);
@@ -915,8 +954,8 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
 EAPI void
 evas_cache_image_drop(Image_Entry *im)
 {
-   Evas_Cache_Image    *cache;
-   int                              references;
+   Evas_Cache_Image *cache;
+   int references;
 
    assert(im);
    assert(im->cache);
@@ -935,16 +974,16 @@ evas_cache_image_drop(Image_Entry *im)
    if (references == 0)
      {
 #ifdef EVAS_FRAME_QUEUING
-        LKL(((RGBA_Image *)im)->ref_fq_add);
-        LKL(((RGBA_Image *)im)->ref_fq_del);
-        if (((RGBA_Image *)im)->ref_fq[0] != ((RGBA_Image *)im)->ref_fq[1])
+        LKL(im->ref_fq_add);
+        LKL(im->ref_fq_del);
+        if (im->ref_fq[0] != im->ref_fq[1])
           {
-             LKU(((RGBA_Image *)im)->ref_fq_add);
-             LKU(((RGBA_Image *)im)->ref_fq_del);
+             LKU(im->ref_fq_add);
+             LKU(im->ref_fq_del);
              return;
           }
-	LKU(((RGBA_Image *)im)->ref_fq_add);
-	LKU(((RGBA_Image *)im)->ref_fq_del);
+	LKU(im->ref_fq_add);
+	LKU(im->ref_fq_del);
 #endif
 
 #ifdef BUILD_ASYNC_PRELOAD
@@ -970,9 +1009,9 @@ evas_cache_image_drop(Image_Entry *im)
 EAPI void
 evas_cache_image_data_not_needed(Image_Entry *im)
 {
-   Evas_Cache_Image    *cache;
-   int   references;
-   
+   Evas_Cache_Image *cache;
+   int references;
+
    assert(im);
    assert(im->cache);
 
@@ -993,11 +1032,11 @@ evas_cache_image_data_not_needed(Image_Entry *im)
 }
 
 EAPI Image_Entry *
-evas_cache_image_dirty(Image_Entry *im, int x, int y, int w, int h)
+evas_cache_image_dirty(Image_Entry *im, unsigned int x, unsigned int y, unsigned int w, unsigned int h)
 {
-   Image_Entry          *im_dirty = im;
-   Evas_Cache_Image     *cache;
-   int                           references;
+   Image_Entry *im_dirty = im;
+   Evas_Cache_Image *cache;
+   int references;
 
    assert(im);
    assert(im->cache);
@@ -1019,13 +1058,12 @@ evas_cache_image_dirty(Image_Entry *im, int x, int y, int w, int h)
         else
 #endif
           {
-             int        error;
+             int error;
 
-             im_dirty = evas_cache_image_copied_data
-               (cache, im->w, im->h, 
-                evas_cache_image_pixels(im), 
-                im->flags.alpha, 
-                im->space);
+             im_dirty = 
+               evas_cache_image_copied_data(cache, im->w, im->h, 
+                                            evas_cache_image_pixels(im), 
+                                            im->flags.alpha, im->space);
              if (!im_dirty) goto on_error;
              if (cache->func.debug)
                cache->func.debug("dirty-src", im);
@@ -1075,9 +1113,9 @@ evas_cache_image_dirty(Image_Entry *im, int x, int y, int w, int h)
 EAPI Image_Entry *
 evas_cache_image_alone(Image_Entry *im)
 {
-   Evas_Cache_Image     *cache;
-   Image_Entry          *im_dirty = im;
-   int                           references;
+   Evas_Cache_Image *cache;
+   Image_Entry *im_dirty = im;
+   int references;
 
    assert(im);
    assert(im->cache);
@@ -1101,7 +1139,7 @@ evas_cache_image_alone(Image_Entry *im)
      }
    else
      {
-        int     error;
+        int error;
 
         im_dirty = evas_cache_image_copied_data
           (cache, im->w, im->h, 
@@ -1146,9 +1184,9 @@ evas_cache_image_alone(Image_Entry *im)
 }
 
 EAPI Image_Entry *
-evas_cache_image_copied_data(Evas_Cache_Image *cache, int w, int h, DATA32 *image_data, int alpha, int cspace)
+evas_cache_image_copied_data(Evas_Cache_Image *cache, unsigned int w, unsigned int h, DATA32 *image_data, int alpha, int cspace)
 {
-   Image_Entry  *im;
+   Image_Entry *im;
 
    assert(cache);
 
@@ -1183,9 +1221,9 @@ evas_cache_image_copied_data(Evas_Cache_Image *cache, int w, int h, DATA32 *imag
 }
 
 EAPI Image_Entry *
-evas_cache_image_data(Evas_Cache_Image *cache, int w, int h, DATA32 *image_data, int alpha, int cspace)
+evas_cache_image_data(Evas_Cache_Image *cache, unsigned int w, unsigned int h, DATA32 *image_data, int alpha, int cspace)
 {
-   Image_Entry  *im;
+   Image_Entry *im;
 
    assert(cache);
 
@@ -1217,9 +1255,9 @@ evas_cache_image_data(Evas_Cache_Image *cache, int w, int h, DATA32 *image_data,
 }
 
 EAPI void
-evas_cache_image_surface_alloc(Image_Entry *im, int w, int h)
+evas_cache_image_surface_alloc(Image_Entry *im, unsigned int w, unsigned int h)
 {
-   Evas_Cache_Image     *cache;
+   Evas_Cache_Image *cache;
 
    assert(im);
    assert(im->cache);
@@ -1237,11 +1275,11 @@ evas_cache_image_surface_alloc(Image_Entry *im, int w, int h)
 }
 
 EAPI Image_Entry *
-evas_cache_image_size_set(Image_Entry *im, int w, int h)
+evas_cache_image_size_set(Image_Entry *im, unsigned int w, unsigned int h)
 {
-   Evas_Cache_Image    *cache;
-   Image_Entry         *new;
-   int                  error;
+   Evas_Cache_Image *cache;
+   Image_Entry *new;
+   int error;
 
    assert(im);
    assert(im->cache);
@@ -1308,10 +1346,7 @@ evas_cache_image_load_data(Image_Entry *im)
    assert(im->cache);
    cache = im->cache;
 
-   if (im->flags.loaded)
-     {
-	return;
-     }
+   if (im->flags.loaded) return;
 
 #ifdef BUILD_ASYNC_PRELOAD
    if (im->preload)
@@ -1380,12 +1415,20 @@ evas_cache_image_unload_data(Image_Entry *im)
    assert(im->cache);
    cache = im->cache;
 
-   if (im->flags.in_progress) return ;
+   if (im->flags.in_progress) return;
 
    evas_cache_image_preload_cancel(im, NULL);
 
 #ifdef BUILD_ASYNC_PRELOAD
-   LKL(im->lock);
+   LKL(im->lock_cancel);
+   if (LKT(im->lock) != 0) /* can't get image lock - busy async load */
+     {
+        im->unload_cancel = EINA_TRUE;
+        LKU(im->lock_cancel);
+        return;
+     }
+   LKU(im->lock_cancel);
+//   LKL(im->lock);
 #endif
    if ((!im->flags.loaded) || (!im->file) ||
        (!im->info.module) || (im->flags.dirty))
@@ -1395,12 +1438,12 @@ evas_cache_image_unload_data(Image_Entry *im)
 #endif
 	return;
      }
-   evas_common_rgba_image_scalecache_dirty(im);
    cache->func.destructor(im);
 
 #ifdef BUILD_ASYNC_PRELOAD
    LKU(im->lock);
 #endif
+   //FIXME: imagedataunload - inform owners
 }
 
 static Eina_Bool
@@ -1433,21 +1476,22 @@ EAPI void
 evas_cache_image_preload_data(Image_Entry *im, const void *target)
 {
 #ifdef BUILD_ASYNC_PRELOAD
-   Evas_Cache_Image    *cache;
-
+   RGBA_Image *img = (RGBA_Image *)im;
+   
    assert(im);
    assert(im->cache);
 
-   if (im->flags.loaded)
+   if ((im->flags.loaded) && (img->image.data))
      {
 	evas_object_inform_call_image_preloaded((Evas_Object *)target);
 	return;
      }
-
-   cache = im->cache;
+   im->flags.loaded = 0;
 
    if (!_evas_cache_image_entry_preload_add(im, target))
-     evas_object_inform_call_image_preloaded((Evas_Object *)target);
+     {
+        evas_object_inform_call_image_preloaded((Evas_Object *)target);
+     }
 #else
    evas_cache_image_load_data(im);
    evas_object_inform_call_image_preloaded((Evas_Object *)target);
@@ -1458,14 +1502,10 @@ EAPI void
 evas_cache_image_preload_cancel(Image_Entry *im, const void *target)
 {
 #ifdef BUILD_ASYNC_PRELOAD
-   Evas_Cache_Image *cache;
-
    assert(im);
    assert(im->cache);
-   cache = im->cache;
-
-   if (target == NULL) return ;
 
+   if (!target) return;
    _evas_cache_image_entry_preload_remove(im, target);
 #else
    (void)im;
@@ -1476,10 +1516,14 @@ EAPI int
 evas_cache_image_flush(Evas_Cache_Image *cache)
 {
    assert(cache);
+   assert(cache->usage >= 0);
 
-   if (cache->limit == -1) return -1;
+#ifdef CACHEDUMP
+  _dump_cache(cache);
+#endif  
+   if (cache->limit == (unsigned int)-1) return -1;
 
-   while ((cache->lru) && (cache->limit < cache->usage))
+   while ((cache->lru) && (cache->limit < (unsigned int)cache->usage))
      {
         Image_Entry *im;
 
@@ -1487,7 +1531,7 @@ evas_cache_image_flush(Evas_Cache_Image *cache)
         _evas_cache_image_entry_delete(cache, im);
      }
 
-   while ((cache->lru_nodata) && (cache->limit < cache->usage))
+   while ((cache->lru_nodata) && (cache->limit < (unsigned int)cache->usage))
      {
         Image_Entry *im;
 
@@ -1523,7 +1567,7 @@ evas_cache_image_empty(Evas_Cache_Image *cache)
 EAPI void
 evas_cache_image_colorspace(Image_Entry *im, int cspace)
 {
-   Evas_Cache_Image    *cache;
+   Evas_Cache_Image *cache;
 
    assert(im);
    assert(im->cache);
@@ -1539,7 +1583,7 @@ evas_cache_image_colorspace(Image_Entry *im, int cspace)
 EAPI void *
 evas_cache_private_from_image_entry_get(Image_Entry *im)
 {
-   Evas_Cache_Image	*cache;
+   Evas_Cache_Image *cache;
 
    assert(im);
    assert(im->cache);
@@ -1568,7 +1612,7 @@ evas_cache_private_set(Evas_Cache_Image *cache, const void *data)
 EAPI DATA32 *
 evas_cache_image_pixels(Image_Entry *im)
 {
-   Evas_Cache_Image    *cache;
+   Evas_Cache_Image *cache;
 
    assert(im);
    assert(im->cache);
