@@ -3,7 +3,9 @@
 
 #ifdef BUILD_ASYNC_EVENTS
 
-#include <unistd.h>
+#ifndef _MSC_VER
+# include <unistd.h>
+#endif
 #include <fcntl.h>
 #include <errno.h>
 
@@ -16,10 +18,10 @@ typedef struct _Evas_Event_Async	Evas_Event_Async;
 
 struct _Evas_Event_Async
 {
-   const void		 *target;
-   void			 *event_info;
-   void			(*func)(void *target, Evas_Callback_Type type, void *event_info);
-   Evas_Callback_Type	  type;
+   const void		    *target;
+   void			    *event_info;
+   Evas_Async_Events_Put_Cb  func;
+   Evas_Callback_Type	     type;
 };
 
 int
@@ -73,7 +75,7 @@ evas_async_events_shutdown(void)
  * that.
  *
  * An example of actual use of this API is for image asynchronous
- * preload inside evas. If the canvas was instantiated throught
+ * preload inside evas. If the canvas was instantiated through
  * ecore-evas usage, ecore itself will take care of calling those
  * events' processing.
  *
@@ -161,7 +163,7 @@ evas_async_events_process(void)
  * @ingroup Evas_Group
  */
 EAPI Eina_Bool
-evas_async_events_put(const void *target, Evas_Callback_Type type, void *event_info, void (*func)(void *target, Evas_Callback_Type type, void *event_info))
+evas_async_events_put(const void *target, Evas_Callback_Type type, void *event_info, Evas_Async_Events_Put_Cb func)
 {
 #ifdef BUILD_ASYNC_EVENTS
    Evas_Event_Async *ev;
