@@ -2,40 +2,31 @@
 #include "evas_private.h"
 
 static void
-_evas_event_havemap_adjust(Evas_Object *obj, Evas_Coord *x, Evas_Coord *y)
+_evas_event_havemap_adjust(Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Eina_Bool mouse_grabbed)
 {
-   Evas_Object *pmap;
-   
-   if (!obj->havemap_parent) return;
-   pmap = obj->smart.parent;
-   if (!pmap) return;
-   while (pmap)
+   if (obj->smart.parent)
+      _evas_event_havemap_adjust(obj->smart.parent, x, y, mouse_grabbed);
+
+   if ((!obj->cur.map) || (!obj->cur.map->count == 4) || (!obj->cur.usemap))
+      return;
+
+   if (obj->cur.map)
      {
-        if ((pmap->cur.map) && (pmap->cur.map->count == 4) && (pmap->cur.usemap))
-          break;
-        pmap = pmap->smart.parent;
-     }
-   if (!pmap) return;
-   evas_map_coords_get(pmap->cur.map, *x, *y, x, y, obj->mouse_grabbed);
-   if (pmap->cur.map)
-     {
-        *x += pmap->cur.map->normal_geometry.x;
-        *y += pmap->cur.map->normal_geometry.y;
+        evas_map_coords_get(obj->cur.map, *x, *y, x, y, mouse_grabbed);
+        *x += obj->cur.geometry.x;
+        *y += obj->cur.geometry.y;
      }
 }
 
 static Eina_List *
 _evas_event_object_list_in_get(Evas *e, Eina_List *in,
-                               const Eina_Inlist *list, Evas_Object *stop, 
-                               int x, int y, int *no_rep, int parmap)
+                               const Eina_Inlist *list, Evas_Object *stop,
+                               int x, int y, int *no_rep)
 {
    Evas_Object *obj;
    if (!list) return in;
    EINA_INLIST_REVERSE_FOREACH(list, obj)
      {
-        if (parmap) obj->havemap_parent = 1;
-        else obj->havemap_parent = 0;
-        
 	if (obj == stop)
 	  {
 	     *no_rep = 1;
@@ -50,7 +41,7 @@ _evas_event_object_list_in_get(Evas *e, Eina_List *in,
                {
                   int norep;
                   int inside;
-                  
+
                   norep = 0;
                   if (((obj->cur.map) && (obj->cur.map->count == 4) && (obj->cur.usemap)))
                     {
@@ -65,27 +56,20 @@ _evas_event_object_list_in_get(Evas *e, Eina_List *in,
                               }
                             else
                               {
-                                 parmap = 1;
+                                 in = _evas_event_object_list_in_get
+                                    (e, in,
+                                     evas_object_smart_members_get_direct(obj),
+                                     stop,
+                                     obj->cur.geometry.x + obj->cur.map->mx,
+                                     obj->cur.geometry.y + obj->cur.map->my, &norep);
                               }
                          }
-                       else
-                         inside = 0;
-                       if (inside)
-                         {
-                            in = _evas_event_object_list_in_get
-                              (e, in,
-                               evas_object_smart_members_get_direct(obj),
-                               stop, 
-                               obj->cur.geometry.x + obj->cur.map->mx, 
-                               obj->cur.geometry.y + obj->cur.map->my, 
-                               &norep, parmap);
-                         }
                     }
                   else
                     {
                        in = _evas_event_object_list_in_get
-                         (e, in, evas_object_smart_members_get_direct(obj),
-                          stop, x, y, &norep, parmap);
+                          (e, in, evas_object_smart_members_get_direct(obj),
+                           stop, x, y, &norep);
                     }
                   if (norep)
                     {
@@ -100,18 +84,18 @@ _evas_event_object_list_in_get(Evas *e, Eina_List *in,
                   if (((obj->cur.map) && (obj->cur.map->count == 4) && (obj->cur.usemap)))
                     {
                        inside = evas_object_is_in_output_rect(obj, x, y, 1, 1);
-                       if (inside)
+                       if ((inside) && (!evas_map_coords_get(obj->cur.map, x, y,
+                                                             &(obj->cur.map->mx),
+                                                             &(obj->cur.map->my), 0)))
                          {
-                            if (!evas_map_coords_get(obj->cur.map, x, y,
-                                                     &(obj->cur.map->mx),
-                                                     &(obj->cur.map->my), 0))
-                              inside = 0;
+                            inside = 0;
                          }
-                       else
-                         inside = 0;
                     }
                   else
-                    inside = evas_object_is_in_output_rect(obj, x, y, 1, 1);
+                    {
+                       inside = evas_object_is_in_output_rect(obj, x, y, 1, 1);
+                    }
+
                   if (inside && ((!obj->precise_is_inside) ||
                                  (evas_object_is_inside(obj, x, y))))
                     {
@@ -143,7 +127,7 @@ evas_event_objects_event_list(Evas *e, Evas_Object *stop, int x, int y)
 	norep = 0;
 	in = _evas_event_object_list_in_get(e, in, 
                                             EINA_INLIST_GET(lay->objects), 
-                                            stop, x, y, &norep, 0);
+                                            stop, x, y, &norep);
 	if (norep) return in;
      }
    return in;
@@ -328,9 +312,11 @@ evas_event_feed_mouse_down(Evas *e, int b, Evas_Button_Flags flags, unsigned int
    copy = evas_event_list_copy(e->pointer.object.in);
    EINA_LIST_FOREACH(copy, l, obj)
      {
+        if (obj->delete_me) continue;
+        
         ev.canvas.x = e->pointer.x;
         ev.canvas.y = e->pointer.y;
-        _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y);
+        _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y, obj->mouse_grabbed);
 	if (obj->pointer_mode != EVAS_OBJECT_POINTER_MODE_NOGRAB)
 	  {
 	     obj->mouse_grabbed++;
@@ -401,15 +387,18 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
 	  {
              ev.canvas.x = e->pointer.x;
              ev.canvas.y = e->pointer.y;
-             _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y);
+             _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y, obj->mouse_grabbed);
 	     if ((obj->pointer_mode != EVAS_OBJECT_POINTER_MODE_NOGRAB) &&
 		 (obj->mouse_in) && (obj->mouse_grabbed > 0))
 	       {
 		  obj->mouse_grabbed--;
 		  e->pointer.mouse_grabbed--;
 	       }
-	     if (e->events_frozen <= 0)
-	       evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_UP, &ev);
+             if (!obj->delete_me)
+               {
+                  if (e->events_frozen <= 0)
+                     evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_UP, &ev);
+               }
 	     if (e->delete_me) break;
 	  }
 	if (copy) copy = eina_list_free(copy);
@@ -446,14 +435,16 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
 	       {
                   ev.canvas.x = e->pointer.x;
                   ev.canvas.y = e->pointer.y;
-                  _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y);
+                  _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y, obj->mouse_grabbed);
 		  if ((!eina_list_data_find(ins, obj)) ||
 		      (!e->pointer.inside))
 		    {
-
-		       obj->mouse_in = 0;
-		       if (e->events_frozen <= 0)
-			 evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev);
+                       if (obj->mouse_in)
+                         {
+                            obj->mouse_in = 0;
+                            if (e->events_frozen <= 0)
+                               evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev);
+                         }
 		    }
 		  if (e->delete_me) break;
 	       }
@@ -482,13 +473,15 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
 	       {
                   ev.canvas.x = e->pointer.x;
                   ev.canvas.y = e->pointer.y;
-                  _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y);
+                  _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y, obj->mouse_grabbed);
 		  if (!eina_list_data_find(e->pointer.object.in, obj))
 		    {
-
-		       obj->mouse_in = 1;
-		       if (e->events_frozen <= 0)
-			 evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_IN, &ev);
+                       if (!obj->mouse_in)
+                         {
+                            obj->mouse_in = 1;
+                            if (e->events_frozen <= 0)
+                               evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_IN, &ev);
+                         }
 		    }
 		  if (e->delete_me) break;
 	       }
@@ -601,7 +594,7 @@ evas_event_feed_mouse_wheel(Evas *e, int direction, int z, unsigned int timestam
      {
         ev.canvas.x = e->pointer.x;
         ev.canvas.y = e->pointer.y;
-        _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y);
+        _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y, obj->mouse_grabbed);
 	if (e->events_frozen <= 0)
 	  evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_WHEEL, &ev);
 	if (e->delete_me) break;
@@ -684,7 +677,7 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 	       {
                   ev.cur.canvas.x = e->pointer.x;
                   ev.cur.canvas.y = e->pointer.y;
-                  _evas_event_havemap_adjust(obj, &ev.cur.canvas.x, &ev.cur.canvas.y);
+                  _evas_event_havemap_adjust(obj, &ev.cur.canvas.x, &ev.cur.canvas.y, obj->mouse_grabbed);
 		  if ((obj->cur.visible) &&
 		      (evas_object_clippers_is_visible(obj)) &&
 		      (!evas_event_passes_through(obj)) &&
@@ -718,7 +711,7 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 	     ev.timestamp = timestamp;
 	     ev.event_flags = EVAS_EVENT_FLAG_NONE;
 
-	     if (copy) copy = eina_list_free(copy);
+	     if (copy) eina_list_free(copy);
 	     while (outs)
 	       {
 		  Evas_Object *obj;
@@ -729,11 +722,17 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 		    {
                        ev.canvas.x = e->pointer.x;
                        ev.canvas.y = e->pointer.y;
-                       _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y);
+                       _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y, obj->mouse_grabbed);
 		       e->pointer.object.in = eina_list_remove(e->pointer.object.in, obj);
-                       obj->mouse_in = 0;
-                       if (e->events_frozen <= 0)
-                         evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev);
+                       if (obj->mouse_in)
+                         {
+                            obj->mouse_in = 0;
+                            if (!obj->delete_me)
+                              {
+                                 if (e->events_frozen <= 0)
+                                    evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev);
+                              }
+                         }
 		    }
 	       }
              _evas_post_event_callback_call(e);
@@ -810,7 +809,7 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 		    {
                        ev.cur.canvas.x = e->pointer.x;
                        ev.cur.canvas.y = e->pointer.y;
-                       _evas_event_havemap_adjust(obj, &ev.cur.canvas.x, &ev.cur.canvas.y);
+                       _evas_event_havemap_adjust(obj, &ev.cur.canvas.x, &ev.cur.canvas.y, obj->mouse_grabbed);
 		       if (e->events_frozen <= 0)
 			 evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_MOVE, &ev);
 		    }
@@ -818,12 +817,15 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 	     /* otherwise it has left the object */
 	     else
 	       {
-		  obj->mouse_in = 0;
-                  ev2.canvas.x = e->pointer.x;
-                  ev2.canvas.y = e->pointer.y;
-                  _evas_event_havemap_adjust(obj, &ev2.canvas.x, &ev2.canvas.y);
-		  if (e->events_frozen <= 0)
-		    evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev2);
+                  if (obj->mouse_in)
+                    {
+                       obj->mouse_in = 0;
+                       ev2.canvas.x = e->pointer.x;
+                       ev2.canvas.y = e->pointer.y;
+                       _evas_event_havemap_adjust(obj, &ev2.canvas.x, &ev2.canvas.y, obj->mouse_grabbed);
+                       if (e->events_frozen <= 0)
+                          evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev2);
+                    }
 	       }
 	     if (e->delete_me) break;
 	  }
@@ -837,14 +839,16 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 	  {
              ev3.canvas.x = e->pointer.x;
              ev3.canvas.y = e->pointer.y;
-             _evas_event_havemap_adjust(obj, &ev3.canvas.x, &ev3.canvas.y);
+             _evas_event_havemap_adjust(obj, &ev3.canvas.x, &ev3.canvas.y, obj->mouse_grabbed);
 	     /* if its not in the old list of ins send an enter event */
 	     if (!eina_list_data_find(e->pointer.object.in, obj))
 	       {
-		  obj->mouse_in = 1;
-
-		  if (e->events_frozen <= 0)
-		    evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_IN, &ev3);
+                  if (!obj->mouse_in)
+                    {
+                       obj->mouse_in = 1;
+                       if (e->events_frozen <= 0)
+                          evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_IN, &ev3);
+                    }
 	       }
 	     if (e->delete_me) break;
 	  }
@@ -907,13 +911,15 @@ evas_event_feed_mouse_in(Evas *e, unsigned int timestamp, const void *data)
      {
         ev.canvas.x = e->pointer.x;
         ev.canvas.y = e->pointer.y;
-        _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y);
+        _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y, obj->mouse_grabbed);
 	if (!eina_list_data_find(e->pointer.object.in, obj))
 	  {
-
-	     obj->mouse_in = 1;
-	     if (e->events_frozen <= 0)
-	       evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_IN, &ev);
+             if (!obj->mouse_in)
+               {
+                  obj->mouse_in = 1;
+                  if (e->events_frozen <= 0)
+                     evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_IN, &ev);
+               }
 	  }
 	if (e->delete_me) break;
      }
@@ -977,11 +983,16 @@ evas_event_feed_mouse_out(Evas *e, unsigned int timestamp, const void *data)
 	  {
              ev.canvas.x = e->pointer.x;
              ev.canvas.y = e->pointer.y;
-             _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y);
-             obj->mouse_in = 0;
-	    if (e->events_frozen <= 0)
-	      evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev);
-
+             _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y, obj->mouse_grabbed);
+             if (obj->mouse_in)
+               {
+                  obj->mouse_in = 0;
+                  if (!obj->delete_me)
+                    {
+                       if (e->events_frozen <= 0)
+                          evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev);
+                    }
+               }
 	    if (e->delete_me) break;
 	  }
 	if (copy) copy = eina_list_free(copy);
@@ -1041,7 +1052,7 @@ evas_event_feed_multi_down(Evas *e,
         ev.canvas.y = y;
         ev.canvas.xsub = fx;
         ev.canvas.ysub = fy;
-        _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y);
+        _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y, obj->mouse_grabbed);
         if (x != ev.canvas.x) 
           ev.canvas.xsub = ev.canvas.x; // fixme - lost precision
         if (y != ev.canvas.y)
@@ -1104,7 +1115,7 @@ evas_event_feed_multi_up(Evas *e,
         ev.canvas.y = y;
         ev.canvas.xsub = fx;
         ev.canvas.ysub = fy;
-        _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y);
+        _evas_event_havemap_adjust(obj, &ev.canvas.x, &ev.canvas.y, obj->mouse_grabbed);
         if (x != ev.canvas.x) 
           ev.canvas.xsub = ev.canvas.x; // fixme - lost precision
         if (y != ev.canvas.y)
@@ -1176,7 +1187,7 @@ evas_event_feed_multi_move(Evas *e,
                   ev.cur.canvas.y = y;
                   ev.cur.canvas.xsub = fx;
                   ev.cur.canvas.ysub = fy;
-                  _evas_event_havemap_adjust(obj, &ev.cur.canvas.x, &ev.cur.canvas.y);
+                  _evas_event_havemap_adjust(obj, &ev.cur.canvas.x, &ev.cur.canvas.y, obj->mouse_grabbed);
                   if (x != ev.cur.canvas.x) 
                     ev.cur.canvas.xsub = ev.cur.canvas.x; // fixme - lost precision
                   if (y != ev.cur.canvas.y)
@@ -1238,7 +1249,7 @@ evas_event_feed_multi_move(Evas *e,
                   ev.cur.canvas.y = y;
                   ev.cur.canvas.xsub = fx;
                   ev.cur.canvas.ysub = fy;
-                  _evas_event_havemap_adjust(obj, &ev.cur.canvas.x, &ev.cur.canvas.y);
+                  _evas_event_havemap_adjust(obj, &ev.cur.canvas.x, &ev.cur.canvas.y, obj->mouse_grabbed);
                   if (x != ev.cur.canvas.x) 
                     ev.cur.canvas.xsub = ev.cur.canvas.x; // fixme - lost precision
                   if (y != ev.cur.canvas.y)
@@ -1520,6 +1531,8 @@ evas_object_pass_events_set(Evas_Object *obj, Eina_Bool pass)
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
+   pass = !!pass;
+   if (obj->pass_events == pass) return;
    obj->pass_events = pass;
    evas_object_smart_member_cache_invalidate(obj);
    if (evas_object_is_in_output_rect(obj,
@@ -1559,7 +1572,7 @@ evas_object_pass_events_get(const Evas_Object *obj)
  * callbacks, but also to be repeated on the next lower object in the
  * stack.
  *
- * If @p repeat is false, events occuring on @p obj will be processed
+ * If @p repeat is false, events occurring on @p obj will be processed
  * normally.
  */
 EAPI void
@@ -1568,6 +1581,8 @@ evas_object_repeat_events_set(Evas_Object *obj, Eina_Bool repeat)
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
+   repeat = !!repeat;
+   if (obj->repeat_events == repeat) return;
    obj->repeat_events = repeat;
    if (evas_object_is_in_output_rect(obj,
 				     obj->layer->evas->pointer.x,
@@ -1607,7 +1622,7 @@ evas_object_repeat_events_get(const Evas_Object *obj)
  * This function has no effect if @p obj is not a member of a smart
  * object.
  *
- * If @p prop is true, events occuring on this object will propagate on
+ * If @p prop is true, events occurring on this object will propagate on
  * to the smart object of which @p obj is a member.
  *
  * If @p prop is false, events for which callbacks are set on the member
@@ -1627,7 +1642,7 @@ evas_object_propagate_events_set(Evas_Object *obj, Eina_Bool prop)
 /**
  * Determine whether an object is set to propagate events.
  * @param obj
- * @return propogate events state
+ * @return propagate events state
  */
 EAPI Eina_Bool
 evas_object_propagate_events_get(const Evas_Object *obj)
