@@ -31,6 +31,11 @@ struct _Fndat
    int         size;
    void       *font;
    int         ref;
+
+#ifdef HAVE_FONTCONFIG
+   FcFontSet *set;
+   FcPattern *p_nm;
+#endif
 };
 
 /* private methods for font dir cache */
@@ -51,15 +56,24 @@ static int fc_init = 0;
 void
 evas_font_dir_cache_free(void)
 {
-#ifdef HAVE_FONTCONFIG
-   fc_init--;
-   if (fc_init == 0) FcFini();
-#endif
    if (!font_dirs) return;
 
    eina_hash_foreach(font_dirs, font_cache_dir_free, NULL);
    eina_hash_free(font_dirs);
    font_dirs = NULL;
+
+#ifdef HAVE_FONTCONFIG
+/* this is bad i got a:
+ * fccache.c:512: FcCacheFini: Assertion fcCacheChains[i] == ((void *)0)' failed.   
+ * 
+ * all i can do for now is shut this puppy down. butthat breaks, so disable
+ * it as in reality - there is little reason to care about the memory not
+ * being freed etc.
+ * 
+ *   fc_init--;
+ *   if (fc_init == 0) FcFini();
+ */
+#endif
 }
 
 const char *
@@ -124,6 +138,10 @@ evas_fonts_zero_free(Evas *evas)
 	if (fd->name) eina_stringshare_del(fd->name);
 	if (fd->source) eina_stringshare_del(fd->source);
 	evas->engine.func->font_free(evas->engine.data.output, fd->font);
+#ifdef HAVE_FONTCONFIG
+	if (fd->set) FcFontSetDestroy(fd->set);
+	if (fd->p_nm) FcPatternDestroy(fd->p_nm);
+#endif
 	free(fd);
      }
 }
@@ -144,6 +162,10 @@ evas_fonts_zero_presure(Evas *evas)
 	if (fd->name) eina_stringshare_del(fd->name);
 	if (fd->source) eina_stringshare_del(fd->source);
 	evas->engine.func->font_free(evas->engine.data.output, fd->font);
+#ifdef HAVE_FONTCONFIG
+	if (fd->set) FcFontSetDestroy(fd->set);
+	if (fd->p_nm) FcPatternDestroy(fd->p_nm);
+#endif
 	free(fd);
 
 	if (eina_list_count(fonts_zero) < 5) break;
@@ -180,6 +202,10 @@ evas_font_free(Evas *evas, void *font)
 	if (fd->name) eina_stringshare_del(fd->name);
 	if (fd->source) eina_stringshare_del(fd->source);
 	evas->engine.func->font_free(evas->engine.data.output, fd->font);
+#ifdef HAVE_FONTCONFIG
+	if (fd->set) FcFontSetDestroy(fd->set);
+	if (fd->p_nm) FcPatternDestroy(fd->p_nm);
+#endif
 	free(fd);
 
 	if (eina_list_count(fonts_zero) < 43) break;
@@ -202,9 +228,45 @@ evas_font_init(void)
 #endif
 }
 
+#ifdef HAVE_FONTCONFIG
+static void *
+evas_load_fontconfig(Evas *evas, FcFontSet *set, int size)
+{
+   void *font = NULL;
+   int i;
+
+   /* Do loading for all in family */
+   for (i = 0; i < set->nfont; i++)
+     {
+	FcValue filename;
+
+	FcPatternGet(set->fonts[i], FC_FILE, 0, &filename);
+
+	if (font)
+	  evas->engine.func->font_add(evas->engine.data.output, font, (char *)filename.u.s, size);
+	else
+	  font = evas->engine.func->font_load(evas->engine.data.output, (char *)filename.u.s, size);
+     }
+
+   return font;
+}
+#endif
+
+struct _FcPattern {   
+   int             num;
+   int             size;
+   intptr_t        elts_offset;
+   int             ref;
+};
+
 void *
 evas_font_load(Evas *evas, const char *name, const char *source, int size)
 {
+#ifdef HAVE_FONTCONFIG
+   FcPattern *p_nm = NULL;
+   FcFontSet *set = NULL;
+#endif
+
    void *font = NULL;
    Eina_List *fonts, *l;
    Fndat *fd;
@@ -228,6 +290,13 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 		       fd->ref++;
 		       return fd->font;
 		    }
+#ifdef HAVE_FONTCONFIG
+		  else if (fd->set && fd->p_nm)
+		    {
+		       font = evas_load_fontconfig(evas, fd->set, size);
+		       goto on_find;
+		    }
+#endif
 	       }
 	  }
      }
@@ -246,6 +315,13 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 		       fd->ref++;
 		       return fd->font;
 		    }
+#ifdef HAVE_FONTCONFIG
+		  else if (fd->set && fd->p_nm)
+		    {
+		       font = evas_load_fontconfig(evas, fd->set, size);
+		       goto on_find;
+		    }
+#endif
 	       }
 	  }
      }
@@ -379,12 +455,10 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
    fonts = eina_list_free(fonts);
 
 #ifdef HAVE_FONTCONFIG
+   
    if (!font) /* Search using fontconfig */
      {
-	FcPattern *p_nm = NULL;
-	FcFontSet *set;
 	FcResult res;
-	int i;
 
 	p_nm = FcNameParse((FcChar8 *)name);
 	FcConfigSubstitute(NULL, p_nm, FcMatchPattern);
@@ -395,28 +469,22 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 	if (!set)
 	  {
 	     ERR("No fontconfig font matches '%s'. It was the last resource, no font found!", name);
-	     goto fc_end;
+	     FcPatternDestroy(p_nm);
+	     p_nm = NULL;
 	  }
-
-	/* Do loading for all in family */
-	for (i = 0; i < set->nfont; i++)
-	  {
-	     FcValue filename;
-
-	     FcPatternGet(set->fonts[i], FC_FILE, 0, &filename);
-
-	     if (font)
-	       evas->engine.func->font_add(evas->engine.data.output, font, (char *)filename.u.s, size);
-	     else
-	       font = evas->engine.func->font_load(evas->engine.data.output, (char *)filename.u.s, size);
-	  }
-
-	FcFontSetDestroy(set);
-     fc_end:
-	FcPatternDestroy(p_nm);
+	else
+          {
+             // FIXME: this i think is a bugfix for a rare bug... but i'm
+             // not sure 100%. it seems that way from fc. if trim is set
+             // to FcTrue...
+             //  ok - not a bugfix... but there is something going on somewhere that's wierd?
+//             FcPatternReference(p_nm); /* we have to reference count the pat */
+             font = evas_load_fontconfig(evas, set, size);
+          }
      }
 #endif
 
+ on_find:
    fd = calloc(1, sizeof(Fndat));
    if (fd)
      {
@@ -426,6 +494,10 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 	fd->font = font;
 	fd->ref = 1;
 	fonts_cache = eina_list_prepend(fonts_cache, fd);
+#ifdef HAVE_FONTCONFIG
+	fd->set = set;
+	fd->p_nm = p_nm;
+#endif
      }
 
    if (font)
