@@ -2,6 +2,11 @@
 #include "evas_private.h"
 #include "evas_cs.h"
 
+#ifdef LKDEBUG
+EAPI Eina_Bool lockdebug = EINA_FALSE;
+EAPI int lockmax = 0;
+#endif
+
 static int _evas_init_count = 0;
 int _evas_log_dom_global = -1;
 /**
@@ -22,6 +27,14 @@ evas_init(void)
    if (++_evas_init_count != 1)
      return _evas_init_count;
 
+#ifdef LKDEBUG
+   if (getenv("EVAS_LOCK_DEBUG"))
+      {
+         lockdebug = EINA_TRUE;
+         lockmax = atoi(getenv("EVAS_LOCK_DEBUG"));
+      }
+#endif
+   
 #ifdef HAVE_EVIL
    if (!evil_init())
      return --_evas_init_count;
@@ -30,11 +43,12 @@ evas_init(void)
    if (!eina_init())
      goto shutdown_evil;
 
-   _evas_log_dom_global = eina_log_domain_register("evas_main",EVAS_DEFAULT_LOG_COLOR);
+   _evas_log_dom_global = eina_log_domain_register
+     ("evas_main", EVAS_DEFAULT_LOG_COLOR);
    if (_evas_log_dom_global < 0)
      {
-	EINA_LOG_ERR("Evas could not create a default log domain\n");
-	goto shutdown_eina;
+        EINA_LOG_ERR("Can not create a module log domain.");
+        goto shutdown_eina;
      }
 
    evas_module_init();
@@ -155,15 +169,20 @@ evas_new(void)
    e->hinting = EVAS_FONT_HINTING_BYTECODE;
    e->name_hash = eina_hash_string_superfast_new(NULL);
 
-   eina_array_step_set(&e->delete_objects, 256);
-   eina_array_step_set(&e->active_objects, 256);
-   eina_array_step_set(&e->restack_objects, 256);
-   eina_array_step_set(&e->render_objects, 256);
-   eina_array_step_set(&e->pending_objects, 256);
-   eina_array_step_set(&e->obscuring_objects, 256);
-   eina_array_step_set(&e->temporary_objects, 256);
-   eina_array_step_set(&e->calculate_objects, 256);
-   eina_array_step_set(&e->clip_changes, 256);
+#define EVAS_ARRAY_SET(E, Array)		\
+   eina_array_step_set(&E->Array, sizeof (E->Array), 256);
+
+   EVAS_ARRAY_SET(e, delete_objects);
+   EVAS_ARRAY_SET(e, active_objects);
+   EVAS_ARRAY_SET(e, restack_objects);
+   EVAS_ARRAY_SET(e, render_objects);
+   EVAS_ARRAY_SET(e, pending_objects);
+   EVAS_ARRAY_SET(e, obscuring_objects);
+   EVAS_ARRAY_SET(e, temporary_objects);
+   EVAS_ARRAY_SET(e, calculate_objects);
+   EVAS_ARRAY_SET(e, clip_changes);
+
+#undef EVAS_ARRAY_SET
 
    return e;
 }
@@ -197,19 +216,22 @@ evas_free(Evas *e)
    if (e->walking_list == 0) evas_render_idle_flush(e);
    
    if (e->walking_list > 0) return;
-   if (!e->callbacks) return;
-   if (e->callbacks->deletions_waiting) return;
-   
-   e->callbacks->deletions_waiting = 0;
-   evas_event_callback_list_post_free(&e->callbacks->callbacks);
-   if (!e->callbacks->callbacks)
+
+   if (e->callbacks)
      {
-        free(e->callbacks);
-	e->callbacks = NULL;
+	if (e->callbacks->deletions_waiting) return;
+
+	e->callbacks->deletions_waiting = 0;
+	evas_event_callback_list_post_free(&e->callbacks->callbacks);
+	if (!e->callbacks->callbacks)
+	  {
+	     free(e->callbacks);
+	     e->callbacks = NULL;
+	  }
+
+	_evas_post_event_callback_free(e);
      }
    
-   _evas_post_event_callback_free(e);
-   
    del = 1;
    e->walking_list++;
    e->cleanup = 1;
@@ -398,19 +420,19 @@ evas_engine_info_get(const Evas *e)
  *
  * @param   e    The pointer to the Evas Canvas
  * @param   info The pointer to the Engine Info to use
- * @return  1 if no error occured, 0 otherwise
+ * @return  1 if no error occurred, 0 otherwise
  * @ingroup Evas_Output_Method
  */
-EAPI int
+EAPI Eina_Bool
 evas_engine_info_set(Evas *e, Evas_Engine_Info *info)
 {
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
-   return 0;
+   return EINA_FALSE;
    MAGIC_CHECK_END();
-   if (!info) return 0;
-   if (info != e->engine.info) return 0;
-   if (info->magic != e->engine.info_magic) return 0;
-   return e->engine.func->setup(e, info);
+   if (!info) return EINA_FALSE;
+   if (info != e->engine.info) return EINA_FALSE;
+   if (info->magic != e->engine.info_magic) return EINA_FALSE;
+   return (Eina_Bool)e->engine.func->setup(e, info);
 }
 
 /**
@@ -1109,6 +1131,36 @@ evas_focus_state_get(const Evas *e)
    return e->focus;
 }
 
+/**
+ * Push the nochange flag up 1
+ *
+ * This tells evas, that while the nochange flag is greater than 0, do not
+ * mark objects as "changed" when making changes.
+ * 
+ * @param e The evas to change information.
+ * @ingroup Evas_Canvas
+ */
+EAPI void
+evas_nochange_push(Evas *e)
+{
+   e->nochange++;
+}
+
+/**
+ * Pop the nochange flag down 1
+ *
+ * This tells evas, that while the nochange flag is greater than 0, do not
+ * mark objects as "changed" when making changes.
+ * 
+ * @param e The evas to change information.
+ * @ingroup Evas_Canvas
+ */
+EAPI void
+evas_nochange_pop(Evas *e)
+{
+   e->nochange--;
+}
+
 void
 _evas_walk(Evas *e)
 {
@@ -1130,14 +1182,14 @@ _evas_unwalk(Evas *e)
  * @ingroup Evas_Utils
  */
 EAPI const char *
-evas_load_error_str(int error)
+evas_load_error_str(Evas_Load_Error error)
 {
    switch (error)
      {
       case EVAS_LOAD_ERROR_NONE:
 	 return "No error on load";
       case EVAS_LOAD_ERROR_GENERIC:
-	 return "A non-specific error occured";
+	 return "A non-specific error occurred";
       case EVAS_LOAD_ERROR_DOES_NOT_EXIST:
 	 return "File (or file path) does not exist";
       case EVAS_LOAD_ERROR_PERMISSION_DENIED:
@@ -1152,3 +1204,121 @@ evas_load_error_str(int error)
 	 return "Unknown error";
      }
 }
+
+/**
+ * Convert a given color from HSV to RGB format.
+ *
+ * @param h The Hue component of the color.
+ * @param s The Saturation component of the color.
+ * @param v The Value component of the color.
+ * @param r The Red component of the color.
+ * @param g The Green component of the color.
+ * @param b The Blue component of the color.
+ *
+ * This function converts a given color in HSV color format to RGB
+ * color format.
+ *
+ * @ingroup Evas_Utils
+ **/
+EAPI void
+evas_color_hsv_to_rgb(float h, float s, float v, int *r, int *g, int *b)
+{
+   evas_common_convert_color_hsv_to_rgb(h, s, v, r, g, b);
+}
+
+/**
+ * Convert a given color from RGB to HSV format.
+ *
+ * @param r The Red component of the color.
+ * @param g The Green component of the color.
+ * @param b The Blue component of the color.
+ * @param h The Hue component of the color.
+ * @param s The Saturation component of the color.
+ * @param v The Value component of the color.
+ *
+ * This function converts a given color in RGB color format to HSV
+ * color format.
+ *
+ * @ingroup Evas_Utils
+ **/
+EAPI void
+evas_color_rgb_to_hsv(int r, int g, int b, float *h, float *s, float *v)
+{
+   evas_common_convert_color_rgb_to_hsv(r, g, b, h, s, v);
+}
+
+/**
+ * Pre-multiplies a rgb triplet by an alpha factor.
+ *
+ * @param a The alpha factor.
+ * @param r The Red component of the color.
+ * @param g The Green component of the color.
+ * @param b The Blue component of the color.
+ *
+ * This function pre-multiplies a given rbg triplet by an alpha
+ * factor. Alpha factor is used to define transparency.
+ *
+ * @ingroup Evas_Utils
+ **/
+EAPI void
+evas_color_argb_premul(int a, int *r, int *g, int *b)
+{
+   evas_common_convert_color_argb_premul(a, r, g, b);
+}
+
+/**
+ * Undo pre-multiplication of a rgb triplet by an alpha factor.
+ *
+ * @param a The alpha factor.
+ * @param r The Red component of the color.
+ * @param g The Green component of the color.
+ * @param b The Blue component of the color.
+ *
+ * This function undoes pre-multiplication a given rbg triplet by an
+ * alpha factor. Alpha factor is used to define transparency.
+ *
+ * @see evas_color_argb_premul().
+ *
+ * @ingroup Evas_Utils
+ **/
+EAPI void
+evas_color_argb_unpremul(int a, int *r, int *g, int *b)
+{
+   evas_common_convert_color_argb_unpremul(a, r, g, b);
+}
+
+/**
+ * Pre-multiplies data by an alpha factor.
+ *
+ * @param data The data value.
+ * @param len  The length value.
+ *
+ * This function pre-multiplies a given data by an alpha
+ * factor. Alpha factor is used to define transparency.
+ *
+ * @ingroup Evas_Utils
+ **/
+EAPI void
+evas_data_argb_premul(unsigned int *data, unsigned int len)
+{
+   if (!data || (len < 1)) return;
+   evas_common_convert_argb_premul(data, len);
+}
+
+/**
+ * Undo pre-multiplication data by an alpha factor.
+ *
+ * @param data The data value.
+ * @param len  The length value.
+ *
+ * This function undoes pre-multiplication of a given data by an alpha
+ * factor. Alpha factor is used to define transparency.
+ *
+ * @ingroup Evas_Utils
+ **/
+EAPI void
+evas_data_argb_unpremul(unsigned int *data, unsigned int len)
+{
+   if (!data || (len < 1)) return;
+   evas_common_convert_argb_unpremul(data, len);
+}
