@@ -37,9 +37,36 @@ _evas_map_calc_map_geometry(Evas_Object *obj)
 {
    Evas_Coord x1, x2, y1, y2;
    const Evas_Map_Point *p, *p_end;
-   int ch;
+   Eina_Bool ch = EINA_FALSE;
 
    if (!obj->cur.map) return;
+   if (obj->prev.map)
+     {
+        // FIXME: this causes an infinite loop somewhere... hard to debug
+        if (obj->prev.map->count == obj->cur.map->count)
+          {
+             const Evas_Map_Point *p2;
+             
+             p = obj->cur.map->points;
+             p_end = p + obj->cur.map->count;
+             p2 = obj->prev.map->points;
+             
+             for (; p < p_end; p++, p2++)
+               {
+                  if ((p->a != p2->a) ||
+                      (p->r != p2->r) ||
+                      (p->g != p2->g) ||
+                      (p->b != p2->b))
+                    {
+                       ch = 1;
+                       break;
+                    }
+               }
+          }
+        else
+           ch = 1;
+     }
+
    p = obj->cur.map->points;
    p_end = p + obj->cur.map->count;
    x1 = p->x;
@@ -54,7 +81,6 @@ _evas_map_calc_map_geometry(Evas_Object *obj)
         if (p->y < y1) y1 = p->y;
         if (p->y > y2) y2 = p->y;
      }
-   ch = 0;
    if (obj->cur.map->normal_geometry.x != x1) ch = 1;
    if (obj->cur.map->normal_geometry.y != y1) ch = 1;
    if (obj->cur.map->normal_geometry.w != (x2 - x1)) ch = 1;
@@ -70,9 +96,10 @@ static inline Evas_Map *
 _evas_map_new(int count)
 {
    int i;
-   Evas_Map *m = calloc(1, sizeof(Evas_Map) + count * sizeof(Evas_Map_Point));
+   Evas_Map *m = calloc(1, sizeof(Evas_Map) + (count * sizeof(Evas_Map_Point)));
    if (!m) return NULL;
    m->count = count;
+   m->persp.foc = 0;
    m->alpha = 1;
    m->smooth = 1;
    for (i = 0; i < count; i++)
@@ -96,6 +123,7 @@ _evas_map_copy(Evas_Map *dst, const Evas_Map *src)
    memcpy(dst->points, src->points, src->count * sizeof(Evas_Map_Point));
    dst->smooth = src->smooth;
    dst->alpha = src->alpha;
+   dst->persp = src->persp;
    return EINA_TRUE;
 }
 
@@ -107,6 +135,7 @@ _evas_map_dup(const Evas_Map *orig)
    memcpy(copy->points, orig->points, orig->count * sizeof(Evas_Map_Point));
    copy->smooth = orig->smooth;
    copy->alpha = orig->alpha;
+   copy->persp = orig->persp;
    return copy;
 }
 
@@ -288,12 +317,14 @@ evas_object_map_enable_set(Evas_Object *obj, Eina_Bool enabled)
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
-   if (obj->cur.usemap == !!enabled) return;
-   obj->cur.usemap = !!enabled;
+   enabled = !!enabled;
+   if (obj->cur.usemap == enabled) return;
+   obj->cur.usemap = enabled;
    if (enabled)
      {
         if (!obj->cur.map)
           obj->cur.map = _evas_map_new(4);
+        evas_object_mapped_clip_across_mark(obj);
 //        obj->cur.map->normal_geometry = obj->cur.geometry;
      }
    else
@@ -301,6 +332,7 @@ evas_object_map_enable_set(Evas_Object *obj, Eina_Bool enabled)
         if (obj->cur.map)
           {
              _evas_map_calc_geom_change(obj);
+             evas_object_mapped_clip_across_mark(obj);
           }
      }
    _evas_map_calc_map_geometry(obj);
@@ -330,7 +362,7 @@ evas_object_map_enable_get(const Evas_Object *obj)
 
 
 /**
- * Set the map sourc eobject
+ * Set the map source object
  * 
  * This sets the object from which the map is taken - can be any object that
  * has map enabled on it.
@@ -346,10 +378,11 @@ evas_object_map_source_set(Evas_Object *obj, Evas_Object *src)
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
+   (void)src; /* method still needs to be implemented. */
 }
 
 /**
- * get the map source object
+ * Get the map source object
  * 
  * See evas_object_map_source_set()
  * 
@@ -378,7 +411,7 @@ evas_object_map_source_get(const Evas_Object *obj)
  * image that is mapped to that map vertex/point. The u corresponds to the x
  * coordinate of this mapped point and v, the y coordinate. Note that these
  * coordinates describe a bounding region to sample. If you have a 200x100
- * source image and wannt to display it at 200x100 with proper pixel
+ * source image and want to display it at 200x100 with proper pixel
  * precision, then do:
  * 
  * @code
@@ -396,7 +429,7 @@ evas_object_map_source_get(const Evas_Object *obj)
  * @endcode
  * 
  * Note that the map points a uv coordinates match the image geometry. If
- * the @p map parameter is NULL, the sotred map will be freed and geometry
+ * the @p map parameter is NULL, the stored map will be freed and geometry
  * prior to enabling/setting a map will be restored.
  *
  * @param obj object to change transformation map
@@ -426,24 +459,32 @@ evas_object_map_set(Evas_Object *obj, const Evas_Map *map)
                {
 		  _evas_map_free(obj, obj->cur.map);
                   obj->cur.map = NULL;
+                  evas_object_mapped_clip_across_mark(obj);
                   return;
                }
              _evas_map_free(obj, obj->cur.map);
              obj->cur.map = NULL;
              if (!obj->cur.usemap) _evas_map_calc_geom_change(obj);
              else _evas_map_calc_map_geometry(obj);
+             if (obj->cur.usemap)
+                evas_object_mapped_clip_across_mark(obj);
           }
         return;
      }
    if (!obj->cur.map)
      {
         obj->cur.map = _evas_map_dup(map);
-        obj->prev.map = NULL;
+        if (obj->cur.usemap)
+           evas_object_mapped_clip_across_mark(obj);
      }
    else
      {
-	_evas_map_copy(obj->cur.map, map);
-        obj->prev.map = NULL;
+        Evas_Map *omap = obj->cur.map;
+        obj->cur.map = _evas_map_new(4);
+        memcpy(obj->cur.map, omap, sizeof(Evas_Map) + (4 * sizeof(Evas_Map_Point)));
+        _evas_map_copy(obj->cur.map, map);
+        if (obj->prev.map == omap) obj->prev.map = NULL;
+        free(omap);
      }
    _evas_map_calc_map_geometry(obj);
 }
@@ -639,8 +680,8 @@ evas_map_point_coord_set(Evas_Map *m, int idx, Evas_Coord x, Evas_Coord y, Evas_
    if (!m) return;
    if (idx >= m->count) return;
    p = m->points + idx;
-   p->x = x;
-   p->y = y;
+   p->x = p->px = x;
+   p->y = p->py = y;
    p->z = z;
 }
 
@@ -799,7 +840,8 @@ static inline void
 _evas_map_util_points_populate(Evas_Map *m, const Evas_Coord x, const Evas_Coord y, const Evas_Coord w, const Evas_Coord h, const Evas_Coord z)
 {
    Evas_Map_Point *p = m->points;
-
+   int i;
+   
    p[0].x = x;
    p[0].y = y;
    p[0].z = z;
@@ -823,6 +865,12 @@ _evas_map_util_points_populate(Evas_Map *m, const Evas_Coord x, const Evas_Coord
    p[3].z = z;
    p[3].u = 0.0;
    p[3].v = h;
+   
+   for (i = 0; i < 4; i++)
+     {
+        p[i].px = p[i].x;
+        p[i].py = p[i].y;
+     }
 }
 
 /**
@@ -984,8 +1032,8 @@ evas_map_util_points_color_set(Evas_Map *m, int r, int g, int b, int a)
  *
  * @param m map to change.
  * @param degrees amount of degrees from 0.0 to 360.0 to rotate.
- * @param cx rotation's center horizontal positon.
- * @param cy rotation's center vertical positon.
+ * @param cx rotation's center horizontal position.
+ * @param cy rotation's center vertical position.
  *
  * @see evas_map_point_coord_set()
  * @see evas_map_util_zoom()
@@ -1002,18 +1050,18 @@ evas_map_util_rotate(Evas_Map *m, double degrees, Evas_Coord cx, Evas_Coord cy)
 
    for (; p < p_end; p++)
      {
-        Evas_Coord x, y, xx, yy;
+        double x, y, xx, yy;
 
-        xx = x = p->x - cx;
-        yy = y = p->y - cy;
+        x = p->x - cx;
+        y = p->y - cy;
 
         xx = (x * cos(r));
         yy = (x * sin(r));
         x = xx + (y * cos(r + (M_PI / 2.0)));
         y = yy + (y * sin(r + (M_PI / 2.0)));
 
-        p->x = x + cx;
-        p->y = y + cy;
+        p->px = p->x = x + cx;
+        p->py = p->y = y + cy;
      }
 }
 
@@ -1029,8 +1077,8 @@ evas_map_util_rotate(Evas_Map *m, double degrees, Evas_Coord cx, Evas_Coord cy)
  * @param m map to change.
  * @param zoomx horizontal zoom to use.
  * @param zoomy vertical zoom to use.
- * @param cx zooming center horizontal positon.
- * @param cy zooming center vertical positon.
+ * @param cx zooming center horizontal position.
+ * @param cy zooming center vertical position.
  *
  * @see evas_map_point_coord_set()
  * @see evas_map_util_rotate()
@@ -1046,7 +1094,7 @@ evas_map_util_zoom(Evas_Map *m, double zoomx, double zoomy, Evas_Coord cx, Evas_
 
    for (; p < p_end; p++)
      {
-        Evas_Coord x, y;
+        double x, y;
 
         x = p->x - cx;
         y = p->y - cy;
@@ -1054,8 +1102,8 @@ evas_map_util_zoom(Evas_Map *m, double zoomx, double zoomy, Evas_Coord cx, Evas_
         x = (((double)x) * zoomx);
         y = (((double)y) * zoomy);
 
-        p->x = x + cx;
-        p->y = y + cy;
+        p->px = p->x = x + cx;
+        p->py = p->y = y + cy;
      }
 }
 
@@ -1073,9 +1121,9 @@ evas_map_util_zoom(Evas_Map *m, double zoomx, double zoomy, Evas_Coord cx, Evas_
  * @param dx amount of degrees from 0.0 to 360.0 to rotate arount X axis.
  * @param dy amount of degrees from 0.0 to 360.0 to rotate arount Y axis.
  * @param dz amount of degrees from 0.0 to 360.0 to rotate arount Z axis.
- * @param cx rotation's center horizontal positon.
- * @param cy rotation's center vertical positon.
- * @param cz rotation's center vertical positon.
+ * @param cx rotation's center horizontal position.
+ * @param cy rotation's center vertical position.
+ * @param cz rotation's center vertical position.
  */
 EAPI void
 evas_map_util_3d_rotate(Evas_Map *m, double dx, double dy, double dz, 
@@ -1122,8 +1170,8 @@ evas_map_util_3d_rotate(Evas_Map *m, double dx, double dy, double dz,
              y = yy + (y * sin(rx + M_PI_2));
           }
         
-        p->x = x + cx;
-        p->y = y + cy;
+        p->px = p->x = x + cx;
+        p->py = p->y = y + cy;
         p->z = z + cz;
      }
 }
@@ -1228,7 +1276,7 @@ evas_map_util_3d_lighting(Evas_Map *m,
 
 /**
  * Apply a perspective transform to the map
- * 
+* 
  * This applies a given perspective (3D) to the map coordinates. X, Y and Z
  * values are used. The px and py points specify the "infinite distance" point
  * in the 3D conversion (where all lines converge to like when artists draw
@@ -1259,9 +1307,13 @@ evas_map_util_3d_perspective(Evas_Map *m,
    p = m->points;
    p_end = p + m->count;
 
+   m->persp.px = px;
+   m->persp.py = py;
+   m->persp.z0 = z0;
+   m->persp.foc = foc;
    for (; p < p_end; p++)
      {
-        Evas_Coord x, y, zz;
+        double x, y, zz;
 
         if (foc > 0)
           {
