@@ -72,6 +72,8 @@ static const Evas_Object_Func object_func =
  * @{
  */
 
+EVAS_MEMPOOL(_mp_obj);
+
 /**
  * Adds a rectangle to the given evas.
  * @param   e The given evas.
@@ -125,7 +127,10 @@ evas_object_rectangle_new(void)
    Evas_Object_Rectangle *o;
 
    /* alloc obj private data */
-   o = calloc(1, sizeof(Evas_Object_Rectangle));
+   EVAS_MEMPOOL_INIT(_mp_obj, "evas_object_rectangle", Evas_Object_Rectangle, 256, NULL);
+   o = EVAS_MEMPOOL_ALLOC(_mp_obj, Evas_Object_Rectangle);
+   if (!o) return NULL;
+   EVAS_MEMPOOL_PREP(_mp_obj, o, Evas_Object_Rectangle);
    o->magic = MAGIC_OBJ_RECTANGLE;
    return o;
 }
@@ -142,7 +147,7 @@ evas_object_rectangle_free(Evas_Object *obj)
    MAGIC_CHECK_END();
    /* free obj */
    o->magic = 0;
-   free(o);
+   EVAS_MEMPOOL_FREE(_mp_obj, o);
 }
 
 static void
@@ -196,6 +201,8 @@ evas_object_rectangle_render_pre(Evas_Object *obj)
    /* if it just became visible or invisible */
    is_v = evas_object_is_visible(obj);
    was_v = evas_object_was_visible(obj);
+/* POST 1.0 -> enable */   
+/*   if (!(is_v | was_v)) goto done;*/
    if (is_v != was_v)
      {
 	evas_object_render_pre_visible_change(&obj->layer->evas->clip_changes, obj, is_v, was_v);
@@ -233,7 +240,7 @@ evas_object_rectangle_render_pre(Evas_Object *obj)
 	goto done;
      }
    /* if it changed geometry - and obviously not visibility or color */
-   /* caluclate differences since we have a constant color fill */
+   /* calculate differences since we have a constant color fill */
    /* we really only need to update the differences */
    if ((obj->cur.geometry.x != obj->prev.geometry.x) ||
        (obj->cur.geometry.y != obj->prev.geometry.y) ||
