@@ -64,6 +64,9 @@ static const Evas_Object_Func object_func =
      NULL
 };
 
+EVAS_MEMPOOL(_mp_obj);
+EVAS_MEMPOOL(_mp_cb);
+
 /* public funcs */
 /**
  * Store a pointer to user data for a smart object.
@@ -201,6 +204,7 @@ evas_object_smart_member_add(Evas_Object *obj, Evas_Object *smart_obj)
    evas_object_smart_member_cache_invalidate(obj);
    obj->restack = 1;
    evas_object_change(obj);
+   evas_object_mapped_clip_across_mark(obj);
    if (smart_obj->smart.smart->smart_class->member_add)
      smart_obj->smart.smart->smart_class->member_add(smart_obj, obj);
 }
@@ -241,6 +245,7 @@ evas_object_smart_member_del(Evas_Object *obj)
    evas_object_inject(obj, obj->layer->evas);
    obj->restack = 1;
    evas_object_change(obj);
+   evas_object_mapped_clip_across_mark(obj);
 }
 
 /**
@@ -360,6 +365,14 @@ evas_object_smart_members_get_direct(const Evas_Object *obj)
    return o->contained;
 }
 
+void
+_evas_object_smart_members_all_del(Evas_Object *obj)
+{
+   Evas_Object_Smart *o = (Evas_Object_Smart *)(obj->object_data);
+   while (o->contained)
+      evas_object_del((Evas_Object *)(o->contained));
+}
+
 /**
  * Instantiates a new smart object described by @p s.
  *
@@ -418,7 +431,10 @@ evas_object_smart_callback_add(Evas_Object *obj, const char *event, void (*func)
    MAGIC_CHECK_END();
    if (!event) return;
    if (!func) return;
-   cb = calloc(1, sizeof(Evas_Smart_Callback));
+   EVAS_MEMPOOL_INIT(_mp_cb, "evas_smart_callback", Evas_Smart_Callback, 512, );
+   cb = EVAS_MEMPOOL_ALLOC(_mp_cb, Evas_Smart_Callback);
+   if (!cb) return;
+   EVAS_MEMPOOL_PREP(_mp_cb, cb, Evas_Smart_Callback);
    cb->event = eina_stringshare_add(event);
    cb->func = func;
    cb->func_data = (void *)data;
@@ -475,7 +491,7 @@ evas_object_smart_callback_del(Evas_Object *obj, const char *event, void (*func)
  * @param event_info an event specific struct of info to pass to the callback
  *
  * This should be called internally in the smart object when some specific
- * event has occured. The documentation for the smart object should include
+ * event has occurred. The documentation for the smart object should include
  * a list of possible events and what type of @p event_info to expect.
  *
  * @ingroup Evas_Smart_Object_Group
@@ -559,7 +575,7 @@ evas_object_smart_callbacks_descriptions_set(Evas_Object *obj, const Evas_Smart_
 	return 1;
      }
 
-   for (count = 0, d = descriptions; d->name != NULL; d++)
+   for (count = 0, d = descriptions; d->name; d++)
      count++;
 
    evas_smart_cb_descriptions_resize(&o->callbacks_descriptions, count);
@@ -877,7 +893,7 @@ evas_object_smart_callbacks_clear(Evas_Object *obj)
 	  {
 	     o->callbacks = eina_list_remove(o->callbacks, cb);
 	     if (cb->event) eina_stringshare_del(cb->event);
-	     free(cb);
+             EVAS_MEMPOOL_FREE(_mp_cb, cb);
 	  }
      }
 }
@@ -915,7 +931,7 @@ evas_object_smart_cleanup(Evas_Object *obj)
 	     cb = o->callbacks->data;
 	     o->callbacks = eina_list_remove(o->callbacks, cb);
 	     if (cb->event) eina_stringshare_del(cb->event);
-	     free(cb);
+             EVAS_MEMPOOL_FREE(_mp_cb, cb);
 	  }
 
 	evas_smart_cb_descriptions_resize(&o->callbacks_descriptions, 0);
@@ -1012,7 +1028,10 @@ evas_object_smart_new(void)
    Evas_Object_Smart *o;
 
    /* alloc obj private data */
-   o = calloc(1, sizeof(Evas_Object_Smart));
+   EVAS_MEMPOOL_INIT(_mp_obj, "evas_object_smart", Evas_Object_Smart, 256, NULL);
+   o = EVAS_MEMPOOL_ALLOC(_mp_obj, Evas_Object_Smart);
+   if (!o) return NULL;
+   EVAS_MEMPOOL_PREP(_mp_obj, o, Evas_Object_Smart);
    o->magic = MAGIC_OBJ_SMART;
    return o;
 }
@@ -1029,7 +1048,7 @@ evas_object_smart_free(Evas_Object *obj)
    MAGIC_CHECK_END();
    /* free obj */
    o->magic = 0;
-   free(o);
+   EVAS_MEMPOOL_FREE(_mp_obj, o);
 }
 
 static void
