@@ -1,4 +1,4 @@
-#include "evas_common.h"
+#include "evas_common.h" /* Includes evas_bidi_utils stuff. */
 #include "evas_private.h"
 
 /* save typing */
@@ -9,22 +9,24 @@
 static const char o_type[] = "text";
 
 /* private struct for text object internal data */
-typedef struct _Evas_Object_Text      Evas_Object_Text;
+typedef struct _Evas_Object_Text Evas_Object_Text;
 
 struct _Evas_Object_Text
 {
    DATA32               magic;
 
    struct {
-      const char       *text;
-      const char       *font;
-      const char       *source;
-      Evas_Font_Size    size;
+      const Eina_Unicode  *text; /*The shaped text */
+      const char          *utf8_text; /* The text exposed to the API */
+      const char          *font;
+      const char          *source;
+      Evas_Font_Size       size;
       struct {
 	 unsigned char  r, g, b, a;
       } outline, shadow, glow, glow2;
 
-      unsigned char     style;
+      unsigned char               style;
+      Evas_BiDi_Props             intl_props;
    } cur, prev;
 
    float                ascent, descent;
@@ -74,6 +76,7 @@ static const Evas_Object_Func object_func =
      NULL,
      evas_object_text_scale_update,
      NULL,
+     NULL,
      NULL
 };
 
@@ -85,6 +88,8 @@ static const Evas_Object_Func object_func =
  * @{
  */
 
+EVAS_MEMPOOL(_mp_obj);
+
 /**
  * Creates a new text @c Evas_Object on the provided @c Evas canvas.
  *
@@ -132,9 +137,12 @@ evas_object_text_font_source_set(Evas_Object *obj, const char *font_source)
    if ((o->cur.source) && (font_source) &&
        (!strcmp(o->cur.source, font_source)))
      return;
+   /*
    if (o->cur.source) eina_stringshare_del(o->cur.source);
    if (font_source) o->cur.source = eina_stringshare_add(font_source);
    else o->cur.source = NULL;
+    */
+   eina_stringshare_replace(&o->cur.source, font_source);
 }
 
 /**
@@ -172,8 +180,7 @@ evas_object_text_font_set(Evas_Object *obj, const char *font, Evas_Font_Size siz
    int is, was = 0, pass = 0;
    int same_font = 0;
 
-   if (!font) return;
-   if (size <= 0) return;
+   if ((!font) || (size <= 0)) return;
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -209,9 +216,12 @@ evas_object_text_font_set(Evas_Object *obj, const char *font, Evas_Font_Size siz
      }
    if (!same_font)
      {
+	/*
 	if (o->cur.font) eina_stringshare_del(o->cur.font);
 	if (font) o->cur.font = eina_stringshare_add(font);
 	else o->cur.font = NULL;
+	 */
+	eina_stringshare_replace(&o->cur.font, font);
 	o->prev.font = NULL;
      }
    o->cur.size = size;
@@ -222,9 +232,10 @@ evas_object_text_font_set(Evas_Object *obj, const char *font, Evas_Font_Size siz
      {
 	int w, h;
 
+        o->ascent =  ENFN->font_ascent_get(ENDT, o->engine_data);
 	ENFN->font_string_size_get(ENDT,
 				   o->engine_data,
-				   o->cur.text,
+				   o->cur.text, &o->cur.intl_props,
 				   &w, &h);
 	o->ascent = ENFN->font_ascent_get(ENDT, o->engine_data);
 	o->descent = ENFN->font_descent_get(ENDT, o->engine_data);
@@ -314,12 +325,12 @@ evas_object_text_font_get(const Evas_Object *obj, const char **font, Evas_Font_S
  * @param text Text to display.
  */
 EAPI void
-evas_object_text_text_set(Evas_Object *obj, const char *text)
+evas_object_text_text_set(Evas_Object *obj, const char *_text)
 {
    Evas_Object_Text *o;
-   int is, was;
-
-   if (!text) text = "";
+   int is, was, len;
+   Eina_Unicode *text;
+   
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -327,15 +338,41 @@ evas_object_text_text_set(Evas_Object *obj, const char *text)
    MAGIC_CHECK(o, Evas_Object_Text, MAGIC_OBJ_TEXT);
    return;
    MAGIC_CHECK_END();
-   if ((o->cur.text) && (text) && (!strcmp(o->cur.text, text))) return;
+
+   text = evas_common_encoding_utf8_to_unicode(_text, &len);
+
+   if (!text) text = eina_unicode_strdup(EINA_UNICODE_EMPTY_STRING);
+   if ((o->cur.text) && (text) && (!eina_unicode_strcmp(o->cur.text, text)))
+     {
+        free(text);
+        return;
+     }
    was = evas_object_is_in_output_rect(obj,
 				       obj->layer->evas->pointer.x,
 				       obj->layer->evas->pointer.y, 1, 1);
    /* DO II */
-   if (o->cur.text) eina_stringshare_del(o->cur.text);
-   if ((text) && (*text)) o->cur.text = eina_stringshare_add(text);
-   else o->cur.text = NULL;
+   /*Update intl_props*/
+#ifdef BIDI_SUPPORT
+   evas_bidi_paragraph_props_unref(o->cur.intl_props.props);
+   o->cur.intl_props.props = evas_bidi_paragraph_props_get(text);
+   evas_bidi_shape_string(text, &o->cur.intl_props, len);
+#endif
+   if (o->cur.text) eina_ustringshare_del(o->cur.text);
+   if (o->cur.utf8_text) eina_stringshare_del(o->cur.utf8_text);
+
+   if ((text) && (*text)) 
+     {
+        o->cur.text = eina_ustringshare_add(text);
+        o->cur.utf8_text = eina_stringshare_add(_text);
+     }
+   else 
+     {
+        o->cur.text = NULL;
+        o->cur.utf8_text = NULL;
+     }
+
    o->prev.text = NULL;
+
    if ((o->engine_data) && (o->cur.text))
      {
 	int w, h;
@@ -343,7 +380,7 @@ evas_object_text_text_set(Evas_Object *obj, const char *text)
 
 	ENFN->font_string_size_get(ENDT,
 				   o->engine_data,
-				   o->cur.text,
+				   o->cur.text, &o->cur.intl_props,
 				   &w, &h);
 	evas_text_style_pad_get(o->cur.style, &l, &r, &t, &b);
 	obj->cur.geometry.w = w + l + r;
@@ -373,6 +410,7 @@ evas_object_text_text_set(Evas_Object *obj, const char *text)
 				obj->layer->evas->last_timestamp,
 				NULL);
    evas_object_inform_call_resize(obj);
+   if (text) free(text);
 }
 
 /**
@@ -392,7 +430,7 @@ evas_object_text_text_get(const Evas_Object *obj)
    MAGIC_CHECK(o, Evas_Object_Text, MAGIC_OBJ_TEXT);
    return NULL;
    MAGIC_CHECK_END();
-   return o->cur.text;
+   return o->cur.utf8_text;
 }
 
 /**
@@ -522,7 +560,7 @@ evas_object_text_horiz_advance_get(const Evas_Object *obj)
    MAGIC_CHECK_END();
    if (!o->engine_data) return 0;
    if (!o->cur.text) return 0;
-   return ENFN->font_h_advance_get(ENDT, o->engine_data, o->cur.text);
+   return ENFN->font_h_advance_get(ENDT, o->engine_data, o->cur.text, &o->cur.intl_props);
 }
 
 /**
@@ -545,7 +583,7 @@ evas_object_text_vert_advance_get(const Evas_Object *obj)
    MAGIC_CHECK_END();
    if (!o->engine_data) return 0;
    if (!o->cur.text) return o->ascent + o->descent;
-   return ENFN->font_v_advance_get(ENDT, o->engine_data, o->cur.text);
+   return ENFN->font_v_advance_get(ENDT, o->engine_data, o->cur.text, &o->cur.intl_props);
 }
 
 /**
@@ -572,7 +610,6 @@ evas_object_text_char_pos_get(const Evas_Object *obj, int pos, Evas_Coord *cx, E
    Evas_Object_Text *o;
    int l = 0, r = 0, t = 0, b = 0;
    int ret, x = 0, y = 0, w = 0, h = 0;
-   int inset;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return EINA_FALSE;
@@ -583,28 +620,24 @@ evas_object_text_char_pos_get(const Evas_Object *obj, int pos, Evas_Coord *cx, E
    MAGIC_CHECK_END();
    if (!o->engine_data) return EINA_FALSE;
    if (!o->cur.text) return EINA_FALSE;
-   inset =
-     ENFN->font_inset_get(ENDT, o->engine_data, o->cur.text);
-   ret = ENFN->font_char_coords_get(ENDT, o->engine_data, o->cur.text,
-							     pos,
-							     &x, &y,
-							     &w, &h);
+   ret = ENFN->font_char_coords_get(ENDT, o->engine_data, o->cur.text, &o->cur.intl_props,
+				    pos, &x, &y, &w, &h);
    evas_text_style_pad_get(o->cur.style, &l, &r, &t, &b);
    y += o->max_ascent - t;
-   x -= inset + l;
+   x -= l;
    if (x < 0)
      {
 	w += x;
 	x = 0;
      }
-   if (x + w > obj->cur.geometry.w) w = obj->cur.geometry.w - x;
+   if ((x + w) > obj->cur.geometry.w) w = obj->cur.geometry.w - x;
    if (w < 0) w = 0;
    if (y < 0)
      {
 	h += y;
 	y = 0;
      }
-   if (y + h > obj->cur.geometry.h) h = obj->cur.geometry.h - y;
+   if ((y + h) > obj->cur.geometry.h) h = obj->cur.geometry.h - y;
    if (h < 0) h = 0;
    if (cx) *cx = x;
    if (cy) *cy = y;
@@ -617,7 +650,7 @@ evas_object_text_char_pos_get(const Evas_Object *obj, int pos, Evas_Coord *cx, E
 /**
  * Returns the logical position of the last char in the text
  * up to the pos given. this is NOT the position of the last char
- * because of the possibilty of RTL in the text.
+ * because of the possibility of RTL in the text.
  * To be documented.
  *
  * FIXME: To be fixed.
@@ -627,7 +660,6 @@ EAPI int
 evas_object_text_last_up_to_pos(const Evas_Object *obj, Evas_Coord x, Evas_Coord y)
 {
    Evas_Object_Text *o;
-   int inset;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return -1;
@@ -638,12 +670,10 @@ evas_object_text_last_up_to_pos(const Evas_Object *obj, Evas_Coord x, Evas_Coord
    MAGIC_CHECK_END();
    if (!o->engine_data) return -1;
    if (!o->cur.text) return -1;
-   inset =
-     ENFN->font_inset_get(ENDT, o->engine_data, o->cur.text);
    return ENFN->font_last_up_to_pos(ENDT,
 				       o->engine_data,
-				       o->cur.text,
-				       x + inset,
+				       o->cur.text, &o->cur.intl_props,
+				       x,
 				       y - o->max_ascent);	
 }
 
@@ -659,7 +689,6 @@ evas_object_text_char_coords_get(const Evas_Object *obj, Evas_Coord x, Evas_Coor
    Evas_Object_Text *o;
    int l = 0, r = 0, t = 0, b = 0;
    int ret, rx = 0, ry = 0, rw = 0, rh = 0;
-   int inset;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return -1;
@@ -670,31 +699,29 @@ evas_object_text_char_coords_get(const Evas_Object *obj, Evas_Coord x, Evas_Coor
    MAGIC_CHECK_END();
    if (!o->engine_data) return -1;
    if (!o->cur.text) return -1;
-   inset =
-     ENFN->font_inset_get(ENDT, o->engine_data, o->cur.text);
    ret = ENFN->font_char_at_coords_get(ENDT,
 				       o->engine_data,
-				       o->cur.text,
-				       x + inset,
+				       o->cur.text, &o->cur.intl_props,
+				       x,
 				       y - o->max_ascent,
 				       &rx, &ry,
 				       &rw, &rh);
    evas_text_style_pad_get(o->cur.style, &l, &r, &t, &b);
    ry += o->max_ascent - t;
-   rx -= inset + l;
+   rx -= l;
    if (rx < 0)
      {
 	rw += rx;
 	rx = 0;
      }
-   if (rx + rw > obj->cur.geometry.w) rw = obj->cur.geometry.w - rx;
+   if ((rx + rw) > obj->cur.geometry.w) rw = obj->cur.geometry.w - rx;
    if (rw < 0) rw = 0;
    if (ry < 0)
      {
 	rh += ry;
 	ry = 0;
      }
-   if (ry + rh > obj->cur.geometry.h) rh = obj->cur.geometry.h - ry;
+   if ((ry + rh) > obj->cur.geometry.h) rh = obj->cur.geometry.h - ry;
    if (rh < 0) rh = 0;
    if (cx) *cx = rx;
    if (cy) *cy = ry;
@@ -722,8 +749,7 @@ evas_object_text_style_set(Evas_Object *obj, Evas_Text_Style_Type style)
    MAGIC_CHECK(o, Evas_Object_Text, MAGIC_OBJ_TEXT);
    return;
    MAGIC_CHECK_END();
-   if (o->cur.style == style)
-     return;
+   if (o->cur.style == style) return;
    evas_text_style_pad_get(o->cur.style, &pl, &pr, &pt, &pb);
    o->cur.style = style;
    evas_text_style_pad_get(o->cur.style, &l, &r, &t, &b);
@@ -798,10 +824,18 @@ evas_object_text_shadow_color_get(const Evas_Object *obj, int *r, int *g, int *b
    Evas_Object_Text *o;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   if (r) *r = 0;
+   if (g) *g = 0;
+   if (b) *b = 0;
+   if (a) *a = 0;
    return;
    MAGIC_CHECK_END();
    o = (Evas_Object_Text *)(obj->object_data);
    MAGIC_CHECK(o, Evas_Object_Text, MAGIC_OBJ_TEXT);
+   if (r) *r = 0;
+   if (g) *g = 0;
+   if (b) *b = 0;
+   if (a) *a = 0;
    return;
    MAGIC_CHECK_END();
    if (r) *r = o->cur.shadow.r;
@@ -851,10 +885,18 @@ evas_object_text_glow_color_get(const Evas_Object *obj, int *r, int *g, int *b,
    Evas_Object_Text *o;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   if (r) *r = 0;
+   if (g) *g = 0;
+   if (b) *b = 0;
+   if (a) *a = 0;
    return;
    MAGIC_CHECK_END();
    o = (Evas_Object_Text *)(obj->object_data);
    MAGIC_CHECK(o, Evas_Object_Text, MAGIC_OBJ_TEXT);
+   if (r) *r = 0;
+   if (g) *g = 0;
+   if (b) *b = 0;
+   if (a) *a = 0;
    return;
    MAGIC_CHECK_END();
    if (r) *r = o->cur.glow.r;
@@ -904,10 +946,18 @@ evas_object_text_glow2_color_get(const Evas_Object *obj, int *r, int *g, int *b,
    Evas_Object_Text *o;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   if (r) *r = 0;
+   if (g) *g = 0;
+   if (b) *b = 0;
+   if (a) *a = 0;
    return;
    MAGIC_CHECK_END();
    o = (Evas_Object_Text *)(obj->object_data);
    MAGIC_CHECK(o, Evas_Object_Text, MAGIC_OBJ_TEXT);
+   if (r) *r = 0;
+   if (g) *g = 0;
+   if (b) *b = 0;
+   if (a) *a = 0;
    return;
    MAGIC_CHECK_END();
    if (r) *r = o->cur.glow2.r;
@@ -957,10 +1007,18 @@ evas_object_text_outline_color_get(const Evas_Object *obj, int *r, int *g, int *
    Evas_Object_Text *o;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   if (r) *r = 0;
+   if (g) *g = 0;
+   if (b) *b = 0;
+   if (a) *a = 0;
    return;
    MAGIC_CHECK_END();
    o = (Evas_Object_Text *)(obj->object_data);
    MAGIC_CHECK(o, Evas_Object_Text, MAGIC_OBJ_TEXT);
+   if (r) *r = 0;
+   if (g) *g = 0;
+   if (b) *b = 0;
+   if (a) *a = 0;
    return;
    MAGIC_CHECK_END();
    if (r) *r = o->cur.outline.r;
@@ -986,10 +1044,18 @@ evas_object_text_style_pad_get(const Evas_Object *obj, int *l, int *r, int *t, i
    Evas_Object_Text *o;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   if (l) *l = 0;
+   if (r) *r = 0;
+   if (t) *t = 0;
+   if (b) *b = 0;
    return;
    MAGIC_CHECK_END();
    o = (Evas_Object_Text *)(obj->object_data);
    MAGIC_CHECK(o, Evas_Object_Text, MAGIC_OBJ_TEXT);
+   if (l) *l = 0;
+   if (r) *r = 0;
+   if (t) *t = 0;
+   if (b) *b = 0;
    return;
    MAGIC_CHECK_END();
    /* use temps to be certain we have initialized values */
@@ -1036,8 +1102,8 @@ evas_font_path_append(Evas *e, const char *path)
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
    MAGIC_CHECK_END();
-   if (!path) return;
 
+   if (!path) return;
    e->font_path = eina_list_append(e->font_path, eina_stringshare_add(path));
 }
 
@@ -1252,9 +1318,21 @@ evas_font_available_list_free(Evas *e, Eina_List *available)
 }
 
 /**
- * To be documented.
+ * Gets the next character in the string
  *
- * FIXME: To be fixed.
+ * Given the UTF-8 string in @p str, and starting byte position in @p pos,
+ * this function will place in @p decoded the decoded code point at @p pos
+ * and return the byte index for the next character in the string.
+ *
+ * The only boundary check done is that @p pos must be >= 0. Other than that,
+ * no checks are performed, so passing an index value that's not within the
+ * length of the string will result in undefined behavior.
+ *
+ * @param str The UTF-8 string
+ * @param pos The byte index where to start
+ * @param decoded Address where to store the decoded code point. Optional.
+ *
+ * @return The byte index of the next character
  *
  * @ingroup Evas_Utils
  */
@@ -1264,18 +1342,29 @@ evas_string_char_next_get(const char *str, int pos, int *decoded)
    int p, d;
 
    if (decoded) *decoded = 0;
-   if (!str) return 0;
-   if (pos < 0) return 0;
+   if ((!str) || (pos < 0)) return 0;
    p = pos;
-   d = evas_common_font_utf8_get_next((unsigned char *)str, &p);
+   d = evas_common_encoding_utf8_get_next(str, &p);
    if (decoded) *decoded = d;
    return p;
 }
 
 /**
- * To be documented.
+ * Gets the previous character in the string
  *
- * FIXME: To be fixed.
+ * Given the UTF-8 string in @p str, and starting byte position in @p pos,
+ * this function will place in @p decoded the decoded code point at @p pos
+ * and return the byte index for the previous character in the string.
+ *
+ * The only boundary check done is that @p pos must be >= 1. Other than that,
+ * no checks are performed, so passing an index value that's not within the
+ * length of the string will result in undefined behavior.
+ *
+ * @param str The UTF-8 string
+ * @param pos The byte index where to start
+ * @param decoded Address where to store the decoded code point. Optional.
+ *
+ * @return The byte index of the previous character
  *
  * @ingroup Evas_Utils
  */
@@ -1285,10 +1374,9 @@ evas_string_char_prev_get(const char *str, int pos, int *decoded)
    int p, d;
 
    if (decoded) *decoded = 0;
-   if (!str) return 0;
-   if (pos < 1) return 0;
+   if ((!str) || (pos < 1)) return 0;
    p = pos;
-   d = evas_common_font_utf8_get_prev((unsigned char *)str, &p);
+   d = evas_common_encoding_utf8_get_prev(str, &p);
    if (decoded) *decoded = d;
    return p;
 }
@@ -1302,8 +1390,8 @@ evas_string_char_prev_get(const char *str, int pos, int *decoded)
 EAPI int
 evas_string_char_len_get(const char *str)
 {
-	if (!str) return 0;
-	return evas_common_font_utf8_get_len((const unsigned char *) str);
+   if (!str) return 0;
+   return evas_common_encoding_utf8_get_len(str);
 }
 
 /**
@@ -1424,9 +1512,15 @@ evas_object_text_new(void)
    Evas_Object_Text *o;
 
    /* alloc obj private data */
-   o = calloc(1, sizeof(Evas_Object_Text));
+   EVAS_MEMPOOL_INIT(_mp_obj, "evas_object_text", Evas_Object_Text, 128, NULL);
+   o = EVAS_MEMPOOL_ALLOC(_mp_obj, Evas_Object_Text);
+   if (!o) return NULL;
+   EVAS_MEMPOOL_PREP(_mp_obj, o, Evas_Object_Text);
    o->magic = MAGIC_OBJ_TEXT;
    o->prev = o->cur;
+#ifdef BIDI_SUPPORT
+   o->cur.intl_props.props = evas_bidi_paragraph_props_new();
+#endif
    return o;
 }
 
@@ -1441,12 +1535,16 @@ evas_object_text_free(Evas_Object *obj)
    return;
    MAGIC_CHECK_END();
    /* free obj */
-   if (o->cur.text) eina_stringshare_del(o->cur.text);
+   if (o->cur.text) eina_ustringshare_del(o->cur.text);
+   if (o->cur.utf8_text) eina_stringshare_del(o->cur.utf8_text);
    if (o->cur.font) eina_stringshare_del(o->cur.font);
    if (o->cur.source) eina_stringshare_del(o->cur.source);
    if (o->engine_data) evas_font_free(obj->layer->evas, o->engine_data);
+#ifdef BIDI_SUPPORT
+   evas_bidi_props_clean(&o->cur.intl_props);
+#endif
    o->magic = 0;
-   free(o);
+   EVAS_MEMPOOL_FREE(_mp_obj, o);
 }
 
 static void
@@ -1456,11 +1554,11 @@ evas_object_text_render(Evas_Object *obj, void *output, void *context, void *sur
    Evas_Object_Text *o;
    const char vals[5][5] =
      {
-	  {0, 1, 2, 1, 0},
-	  {1, 3, 4, 3, 1},
-	  {2, 4, 5, 4, 2},
-	  {1, 3, 4, 3, 1},
-	  {0, 1, 2, 1, 0}
+	{0, 1, 2, 1, 0},
+	{1, 3, 4, 3, 1},
+	{2, 4, 5, 4, 2},
+	{1, 3, 4, 3, 1},
+	{0, 1, 2, 1, 0}
      };
    int sl = 0, st = 0;
 
@@ -1530,7 +1628,7 @@ evas_object_text_render(Evas_Object *obj, void *output, void *context, void *sur
 		     obj->cur.geometry.h, \
 		     obj->cur.geometry.w, \
 		     obj->cur.geometry.h, \
-		     o->cur.text);
+		     o->cur.text, &o->cur.intl_props);
 #if 0
 #define DRAW_TEXT(ox, oy) \
    if ((o->engine_data) && (o->cur.text)) \
@@ -1538,8 +1636,7 @@ evas_object_text_render(Evas_Object *obj, void *output, void *context, void *sur
 		     context, \
 		     surface, \
 		     o->engine_data, \
-		     obj->cur.cache.geometry.x + x + sl + ox - \
-		     ENFN->font_inset_get(ENDT, o->engine_data, o->cur.text), \
+		     obj->cur.cache.geometry.x + x + sl + ox, \
 		     obj->cur.cache.geometry.y + y + st + oy + \
 		     (int) \
 		     (((o->max_ascent * obj->cur.cache.geometry.h) / obj->cur.geometry.h) - 0.5), \
@@ -1547,7 +1644,7 @@ evas_object_text_render(Evas_Object *obj, void *output, void *context, void *sur
 		     obj->cur.cache.geometry.h, \
 		     obj->cur.geometry.w, \
 		     obj->cur.geometry.h, \
-		     o->cur.text);
+		     o->cur.text, &o->cur.intl_props);
 #endif
    /* shadows */
    if (o->cur.style == EVAS_TEXT_STYLE_SHADOW)
@@ -1652,11 +1749,11 @@ evas_object_text_render_pre(Evas_Object *obj)
    /* dont pre-render the obj twice! */
    if (obj->pre_render_done) return;
    obj->pre_render_done = 1;
-   /* pre-render phase. this does anything an object needs to do just before */
-   /* rendering. this could mean loading the image data, retrieving it from */
-   /* elsewhere, decoding video etc. */
-   /* then when this is done the object needs to figure if it changed and */
-   /* if so what and where and add thr appropriate redraw rectangles */
+   /* pre-render phase. this does anything an object needs to do just before
+    rendering. This could mean loading the image data, retrieving it from 
+    elsewhere, decoding video etc.
+    Then when this is done the object needs to figure if it changed and 
+    if so what and where and add the appropriate redraw rectangles */
    o = (Evas_Object_Text *)(obj->object_data);
    /* if someone is clipping this obj - go calculate the clipper */
    if (obj->cur.clipper)
@@ -1665,19 +1762,21 @@ evas_object_text_render_pre(Evas_Object *obj)
 	  evas_object_clip_recalc(obj->cur.clipper);
 	obj->cur.clipper->func->render_pre(obj->cur.clipper);
      }
-   /* now figure what changed and add draw rects */
-   /* if it just became visible or invisible */
+   /* now figure what changed and add draw rects
+    if it just became visible or invisible */
    is_v = evas_object_is_visible(obj);
    was_v = evas_object_was_visible(obj);
    if (is_v != was_v)
      {
-	evas_object_render_pre_visible_change(&obj->layer->evas->clip_changes, obj, is_v, was_v);
+	evas_object_render_pre_visible_change(&obj->layer->evas->clip_changes, 
+					      obj, is_v, was_v);
 	goto done;
      }
    if ((obj->cur.map != obj->prev.map) ||
        (obj->cur.usemap != obj->prev.usemap))
      {
-	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, 
+					    obj);
         goto done;
      }
    /* its not visible - we accounted for it appearing or not so just abort */
@@ -1687,7 +1786,8 @@ evas_object_text_render_pre(Evas_Object *obj)
    /* if we restacked (layer or just within a layer) and dont clip anyone */
    if (obj->restack)
      {
-	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, 
+					    obj);
 	goto done;
      }
    /* if it changed color */
@@ -1696,37 +1796,43 @@ evas_object_text_render_pre(Evas_Object *obj)
        (obj->cur.color.b != obj->prev.color.b) ||
        (obj->cur.color.a != obj->prev.color.a))
      {
-	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, 
+					    obj);
 	goto done;
      }
-   /* if it changed geometry - and obviously not visibility or color */
-   /* caluclate differences since we have a constant color fill */
-   /* we really only need to update the differences */
+   /* if it changed geometry - and obviously not visibility or color
+    calculate differences since we have a constant color fill
+    we really only need to update the differences */
    if ((obj->cur.geometry.x != obj->prev.geometry.x) ||
        (obj->cur.geometry.y != obj->prev.geometry.y) ||
        (obj->cur.geometry.w != obj->prev.geometry.w) ||
        (obj->cur.geometry.h != obj->prev.geometry.h))
      {
-	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, 
+					    obj);
 	goto done;
      }
    if (obj->cur.render_op != obj->prev.render_op)
      {
-	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, 
+					    obj);
 	goto done;
      }
    if (obj->cur.scale != obj->prev.scale)
      {
-	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, 
+					    obj);
 	goto done;
      }
    if (o->changed)
      {
 	if ((o->cur.size != o->prev.size) ||
-	    ((o->cur.font) && (o->prev.font) && (strcmp(o->cur.font, o->prev.font))) ||
+	    ((o->cur.font) && (o->prev.font) && 
+		(strcmp(o->cur.font, o->prev.font))) ||
 	    ((o->cur.font) && (!o->prev.font)) ||
 	    ((!o->cur.font) && (o->prev.font)) ||
-	    ((o->cur.text) && (o->prev.text) && (strcmp(o->cur.text, o->prev.text))) ||
+	    ((o->cur.text) && (o->prev.text) &&
+                (eina_unicode_strcmp(o->cur.text, o->prev.text))) ||
 	    ((o->cur.text) && (!o->prev.text)) ||
 	    ((!o->cur.text) && (o->prev.text)) ||
 	    ((o->cur.style != o->prev.style)) ||
@@ -1747,12 +1853,14 @@ evas_object_text_render_pre(Evas_Object *obj)
 	    ((o->cur.glow2.b != o->prev.glow2.b)) ||
 	    ((o->cur.glow2.a != o->prev.glow2.a)))
 	  {
-	     evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
+	     evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, 
+						 obj);
 	     goto done;
 	  }
      }
    done:
-   evas_object_render_pre_effect_updates(&obj->layer->evas->clip_changes, obj, is_v, was_v);
+   evas_object_render_pre_effect_updates(&obj->layer->evas->clip_changes, 
+					 obj, is_v, was_v);
 }
 
 static void
@@ -1760,9 +1868,9 @@ evas_object_text_render_post(Evas_Object *obj)
 {
    Evas_Object_Text *o;
 
-   /* this moves the current data to the previous state parts of the object */
-   /* in whatever way is safest for the object. also if we don't need object */
-   /* data anymore we can free it if the object deems this is a good idea */
+   /* this moves the current data to the previous state parts of the object
+    in whatever way is safest for the object. also if we don't need object
+    data anymore we can free it if the object deems this is a good idea */
    o = (Evas_Object_Text *)(obj->object_data);
    /* remove those pesky changes */
    evas_object_clip_changes_clean(obj);
@@ -1772,7 +1880,8 @@ evas_object_text_render_post(Evas_Object *obj)
    o->changed = 0;
 }
 
-static unsigned int evas_object_text_id_get(Evas_Object *obj)
+static unsigned int 
+evas_object_text_id_get(Evas_Object *obj)
 {
    Evas_Object_Text *o;
 
@@ -1781,7 +1890,8 @@ static unsigned int evas_object_text_id_get(Evas_Object *obj)
    return MAGIC_OBJ_TEXT;
 }
 
-static unsigned int evas_object_text_visual_id_get(Evas_Object *obj)
+static unsigned int 
+evas_object_text_visual_id_get(Evas_Object *obj)
 {
    Evas_Object_Text *o;
 
@@ -1790,7 +1900,8 @@ static unsigned int evas_object_text_visual_id_get(Evas_Object *obj)
    return MAGIC_OBJ_SHAPE;
 }
 
-static void *evas_object_text_engine_data_get(Evas_Object *obj)
+static void *
+evas_object_text_engine_data_get(Evas_Object *obj)
 {
    Evas_Object_Text *o;
 
@@ -1800,18 +1911,18 @@ static void *evas_object_text_engine_data_get(Evas_Object *obj)
 }
 
 static int
-evas_object_text_is_opaque(Evas_Object *obj)
+evas_object_text_is_opaque(Evas_Object *obj __UNUSED__)
 {
-   /* this returns 1 if the internal object data implies that the object is */
-   /* currently fulyl opque over the entire gradient it occupies */
+   /* this returns 1 if the internal object data implies that the object is 
+    currently fully opaque over the entire gradient it occupies */
    return 0;
 }
 
 static int
-evas_object_text_was_opaque(Evas_Object *obj)
+evas_object_text_was_opaque(Evas_Object *obj __UNUSED__)
 {
-   /* this returns 1 if the internal object data implies that the object was */
-   /* currently fulyl opque over the entire gradient it occupies */
+   /* this returns 1 if the internal object data implies that the object was
+    currently fully opaque over the entire gradient it occupies */
    return 0;
 }
 
@@ -1858,7 +1969,7 @@ _evas_object_text_rehint(Evas_Object *obj)
 
 	ENFN->font_string_size_get(ENDT,
 				   o->engine_data,
-				   o->cur.text,
+				   o->cur.text, &o->cur.intl_props,
 				   &w, &h);
 	evas_text_style_pad_get(o->cur.style, &l, &r, &t, &b);
 	obj->cur.geometry.w = w + l + r;
