@@ -11,90 +11,6 @@ static void _evas_smart_class_callbacks_create(Evas_Smart *s);
  * @{
  */
 
-/**
- * Create an Evas_Smart, which can be used to instantiate new smart objects.
- *
- * This function internally creates an Evas_Smart_Class and sets the
- * provided callbacks. Callbacks that are unneeded (or marked DEPRECATED
- * below) should be set to NULL.
- *
- * Alternatively you can create an Evas_Smart_Class yourself and use
- * evas_smart_class_new().
- *
- * @param name a unique name for the smart
- * @param func_add callback called when smart object is added
- * @param func_del callback called when smart object is deleted
- * @param func_layer_set DEPRECATED
- * @param func_raise DEPRECATED
- * @param func_lower DEPRECATED
- * @param func_stack_above DEPRECATED
- * @param func_stack_below DEPRECATED
- * @param func_move callback called when smart object is moved
- * @param func_resize callback called when smart object is resized
- * @param func_show callback called when smart object is shown
- * @param func_hide callback called when smart object is hidden
- * @param func_color_set callback called when smart object has its color set
- * @param func_clip_set callback called when smart object has its clip set
- * @param func_clip_unset callback called when smart object has its clip unset
- * @param data a pointer to user data for the smart
- * @return an Evas_Smart
- *
- */
-EAPI Evas_Smart *
-evas_smart_new(const char *name,
-	       void      (*func_add) (Evas_Object *o),
-	       void      (*func_del) (Evas_Object *o),
-	       void      (*func_layer_set) (Evas_Object *o, int l) __UNUSED__,
-	       void      (*func_raise) (Evas_Object *o) __UNUSED__,
-	       void      (*func_lower) (Evas_Object *o) __UNUSED__,
-	       void      (*func_stack_above) (Evas_Object *o, Evas_Object *above) __UNUSED__,
-	       void      (*func_stack_below) (Evas_Object *o, Evas_Object *below) __UNUSED__,
-	       void      (*func_move) (Evas_Object *o, Evas_Coord x, Evas_Coord y),
-	       void      (*func_resize) (Evas_Object *o, Evas_Coord w, Evas_Coord h),
-	       void      (*func_show) (Evas_Object *o),
-	       void      (*func_hide) (Evas_Object *o),
-	       void      (*func_color_set) (Evas_Object *o, int r, int g, int b, int a),
-	       void      (*func_clip_set) (Evas_Object *o, Evas_Object *clip),
-	       void      (*func_clip_unset) (Evas_Object *o),
-	       const void *data)
-{
-   Evas_Smart *s;
-   Evas_Smart_Class *sc;
-
-   WRN("----- WARNING. evas_smart_new() will be deprecated and removed soon"
-	  "----- Please use evas_smart_class_new() instead");
-
-   if (!name) return NULL;
-
-   s = evas_mem_calloc(sizeof(Evas_Smart));
-   if (!s) return NULL;
-
-   s->magic = MAGIC_SMART;
-
-   s->class_allocated = 1;
-
-   sc = evas_mem_calloc(sizeof(Evas_Smart_Class));
-   if (!sc)
-     {
-	free(s);
-	return NULL;
-     }
-   sc->name = name;
-   sc->add = func_add;
-   sc->del = func_del;
-   sc->move = func_move;
-   sc->resize = func_resize;
-   sc->show = func_show;
-   sc->hide = func_hide;
-   sc->color_set = func_color_set;
-   sc->clip_set = func_clip_set;
-   sc->clip_unset = func_clip_unset;
-   sc->data = (void *)data;
-   s->smart_class = sc;
-
-   return s;
-}
-
 /**
  * Free an Evas_Smart
  *
@@ -333,7 +249,7 @@ evas_smart_cb_descriptions_resize(Evas_Smart_Cb_Description_Array *a, unsigned i
      }
    else
      {
-	fprintf(stderr, "ERROR: realloc failed!\n");
+	ERR("realloc failed!");
 	return 0;
      }
 }
@@ -391,7 +307,7 @@ evas_smart_cb_descriptions_fix(Evas_Smart_Cb_Description_Array *a)
 	  }
      }
 
-   evas_smart_cb_descriptions_resize(a, a->size - (j - i));
+   evas_smart_cb_descriptions_resize(a, i + 1);
 }
 
 static void
@@ -400,19 +316,19 @@ _evas_smart_class_callbacks_create(Evas_Smart *s)
    const Evas_Smart_Class *sc;
    unsigned int n = 0;
 
-   for (sc = s->smart_class; sc != NULL; sc = sc->parent)
+   for (sc = s->smart_class; sc; sc = sc->parent)
      {
 	const Evas_Smart_Cb_Description *d;
-	for (d = sc->callbacks; d && d->name != NULL; d++)
+	for (d = sc->callbacks; d && d->name; d++)
 	  n++;
      }
 
    if (n == 0) return;
    if (!evas_smart_cb_descriptions_resize(&s->callbacks, n)) return;
-   for (n = 0, sc = s->smart_class; sc != NULL; sc = sc->parent)
+   for (n = 0, sc = s->smart_class; sc; sc = sc->parent)
      {
 	const Evas_Smart_Cb_Description *d;
-	for (d = sc->callbacks; d->name != NULL; d++)
+	for (d = sc->callbacks; d && d->name; d++)
 	  s->callbacks.array[n++] = d;
      }
    evas_smart_cb_descriptions_fix(&s->callbacks);
