@@ -2,6 +2,11 @@
 #include "evas_private.h"
 #include "evas_cs.h"
 
+/**
+ * Retrieves if the system wants to share bitmaps using the server.
+ * @return @c EINA_TRUE if wants, @c EINA_FALSE otherwise.
+ * @ingroup Evas_Cserve
+ */
 EAPI Eina_Bool
 evas_cserve_want_get(void)
 {
@@ -11,6 +16,11 @@ evas_cserve_want_get(void)
    return 0;
 }
 
+/**
+ * Retrieves if the system is connected to the server used to shae bitmaps.
+ * @return @c EINA_TRUE if connected, @c EINA_FALSE otherwise.
+ * @ingroup Evas_Cserve
+ */
 EAPI Eina_Bool
 evas_cserve_connected_get(void)
 {
@@ -20,6 +30,14 @@ evas_cserve_connected_get(void)
    return 0;
 }
 
+/**
+ * Retrieves if the system wants to share bitmaps using the server.
+ * @param stats pointer to structure to fill with statistics about
+ *        cache server.
+ * @return @c EINA_TRUE if @p stats were filled with data,
+ *         @c EINA_FALSE otherwise and @p stats is untouched.
+ * @ingroup Evas_Cserve
+ */
 EAPI Eina_Bool
 evas_cserve_stats_get(Evas_Cserve_Stats *stats)
 {
@@ -46,7 +64,6 @@ evas_cserve_image_cache_contents_get(Evas_Cserve_Image_Cache *cache)
 {
 #ifdef EVAS_CSERVE
    Op_Getinfo_Reply *info;
-   Op_Getinfo_Item *itt;
    unsigned char *p;
    int i, j;
 
@@ -70,9 +87,8 @@ evas_cserve_image_cache_contents_get(Evas_Cserve_Image_Cache *cache)
         Op_Getinfo_Item it;
         char *file, *key;
 
-        itt = (Op_Getinfo_Item *)p;
-        memcpy(&it, itt, sizeof(Op_Getinfo_Item));
-        file = p + sizeof(Op_Getinfo_Item);
+        memcpy(&it, p, sizeof(Op_Getinfo_Item));
+        file = (char*) (p + sizeof(Op_Getinfo_Item));
         key = file + strlen(file) + 1;
         im = calloc(1, sizeof(Evas_Cserve_Image));
         if (!im) continue;
@@ -139,6 +155,13 @@ evas_cserve_image_cache_contents_clean(Evas_Cserve_Image_Cache *cache)
 #endif
 }
 
+/**
+ * Retrieves the current configuration of the server.
+ * @param config where to store current server configuration.
+ * @return @c EINA_TRUE if @p config were filled with data,
+ *         @c EINA_FALSE otherwise and @p config is untouched.
+ * @ingroup Evas_Cserve
+ */
 EAPI Eina_Bool
 evas_cserve_config_get(Evas_Cserve_Config *config)
 {
@@ -156,8 +179,15 @@ evas_cserve_config_get(Evas_Cserve_Config *config)
 #endif
 }
 
+/**
+ * Changes the configuration of the server.
+ * @param config where to store current server configuration.
+ * @return @c EINA_TRUE if @p config were successfully applied,
+ *         @c EINA_FALSE otherwise.
+ * @ingroup Evas_Cserve
+ */
 EAPI Eina_Bool
-evas_cserve_config_set(Evas_Cserve_Config *config)
+evas_cserve_config_set(const Evas_Cserve_Config *config)
 {
 #ifdef EVAS_CSERVE
    Op_Setconfig conf;
@@ -172,6 +202,10 @@ evas_cserve_config_set(Evas_Cserve_Config *config)
 #endif
 }
 
+/**
+ * Force system to disconnect from cache server.
+ * @ingroup Evas_Cserve
+ */
 EAPI void
 evas_cserve_disconnect(void)
 {
