@@ -1,6 +1,3 @@
-/*
-  * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
-  */
 //#undef SCALE_USING_MMX
 {
    if (smooth)
@@ -11,7 +8,10 @@
              FPc u, v, ud, vd, dv, ue, ve;
              DATA32 *d, *s;
 #ifdef COLMUL             
-             FPc cv, cd, cc; // col
+             FPc cv, cd; // col
+# ifdef SCALE_USING_MMX
+             FPc cc;
+#endif             
              DATA32 c1, c2; // col
 #endif             
              Line *line;
@@ -126,7 +126,7 @@
              FPc u, v, ud, vd, ue, ve;
              DATA32 *d, *s;
 #ifdef COLMUL
-             FPc cv, cd, cc; // col
+             FPc cv, cd; // col
              DATA32 c1, c2; // col
 #endif             
              Line *line;
