@@ -1,7 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #ifndef EVAS_COMMON_H
 #define EVAS_COMMON_H
 
@@ -53,7 +49,9 @@ extern EAPI int _evas_log_dom_global;
 #endif
 #define CRIT(...) EINA_LOG_DOM_CRIT(_EVAS_DEFAULT_LOG_DOM, __VA_ARGS__)
 
-/*****************************************************************************/
+/************************ Unicode stuff **************************/
+#include "../engines/common/evas_encoding.h"
+/*****************************************************************/
 
 #include "evas_options.h"
 
@@ -115,10 +113,6 @@ extern EAPI int _evas_log_dom_global;
 # define __ARM_ARCH__ 73
 #endif
 
-#ifndef _GNU_SOURCE
-# define _GNU_SOURCE
-#endif
-
 #ifndef BUILD_PTHREAD
 # undef BUILD_PIPE_RENDER
 #endif
@@ -135,19 +129,56 @@ extern EAPI int _evas_log_dom_global;
 
 # include <pthread.h>
 # include <sched.h>
-# define LK(x)  pthread_mutex_t x
-#ifndef EVAS_FRAME_QUEUING
-# define LKI(x) pthread_mutex_init(&(x), NULL);
-#else
-# define LKI(x) {pthread_mutexattr_t    attr;\
-         pthread_mutexattr_init(&attr); \
-         pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);	\
-         pthread_mutex_init(&(x), &attr);}
+#ifdef __linux__
+# include <sys/time.h>
+# include <sys/resource.h>
+# include <errno.h>
 #endif
-# define LKD(x) pthread_mutex_destroy(&(x));
-# define LKL(x) pthread_mutex_lock(&(x));
-# define LKT(x) pthread_mutex_trylock(&(x));
-# define LKU(x) pthread_mutex_unlock(&(x));
+
+//#define LKDEBUG 1
+
+#ifdef LKDEBUG
+EAPI Eina_Bool lockdebug;
+EAPI int lockmax;
+#endif
+
+#define LK(x)  pthread_mutex_t x
+#ifndef EVAS_FRAME_QUEUING
+# define LKI(x) pthread_mutex_init(&(x), NULL)
+#else
+# define LKI(x) do {pthread_mutexattr_t    __attr;\
+         pthread_mutexattr_init(&__attr); \
+         pthread_mutexattr_settype(&__attr, PTHREAD_MUTEX_RECURSIVE);	\
+         pthread_mutex_init(&(x), &__attr);} while (0)
+#endif
+# define LKD(x) pthread_mutex_destroy(&(x))
+# ifdef LKDEBUG
+#  define LKL(x) \
+   do { \
+      if (lockdebug) { \
+         struct timeval t0, t1; \
+         int dt; \
+         gettimeofday(&t0, NULL); \
+         pthread_mutex_lock(&(x)); \
+         gettimeofday(&t1, NULL); \
+         dt = (t1.tv_sec - t0.tv_sec) * 1000000; \
+         if (t1.tv_usec > t0.tv_usec) dt += (t1.tv_usec - t0.tv_usec); \
+         else dt -= t0.tv_usec - t1.tv_usec; \
+         dt /= 1000; \
+         if (dt > lockmax) { \
+            fprintf(stderr, "HANG %ims - %s:%i - %s()\n", \
+                    dt, __FILE__, __LINE__, __FUNCTION__); \
+         } \
+      } \
+      else { \
+         pthread_mutex_lock(&(x)); \
+      } \
+   } while (0)
+# else
+#  define LKL(x) pthread_mutex_lock(&(x))
+# endif
+# define LKT(x) pthread_mutex_trylock(&(x))
+# define LKU(x) pthread_mutex_unlock(&(x))
 # define TH(x)  pthread_t x
 # define THI(x) int x
 # define TH_MAX 8
@@ -210,9 +241,9 @@ void *alloca (size_t);
 #include <sys/stat.h>
 #include <time.h>
 #include <ctype.h>
+
 #ifndef _MSC_VER
 # include <stdint.h>
-# include <unistd.h>
 #endif
 
 #include <ft2build.h>
@@ -250,18 +281,20 @@ void *alloca (size_t);
 # define DM_BITS      4
 # define DM_DIV       16
 # define USE_DITHER_44 1
-# define DM_MSK       (DM_SIZE - 1)
-# define DM_SHF(_b)   (DM_BITS - (8 - _b))
 #else
 # define DM_TABLE     _evas_dither_128128
 # define DM_SIZE      128
 # define DM_BITS      6
 # define DM_DIV       64
 # define USE_DITHER_128128 1
-# define DM_MSK       (DM_SIZE - 1)
-# define DM_SHF(_b)   (DM_BITS - (8 - _b))
 #endif
 
+#define DM_MSK       (DM_SIZE - 1)
+#define DM_SHF(_b)   (DM_BITS - (8 - _b))
+/* Supports negative right shifts */
+#define DM_SHR(x, _b)   ((DM_SHF(_b) >= 0) ? \
+      ((x) >> DM_SHF(_b)) : ((x) << -DM_SHF(_b)))
+
 /* if more than 1/ALPHA_SPARSE_INV_FRACTION is "alpha" (1-254) then sparse
  * alpha flag gets set */
 #define ALPHA_SPARSE_INV_FRACTION 3
@@ -379,13 +412,6 @@ typedef struct _RGBA_Pipe_Thread_Info RGBA_Pipe_Thread_Info;
 typedef struct _RGBA_Image            RGBA_Image;
 typedef struct _RGBA_Image_Span       RGBA_Image_Span;
 typedef struct _RGBA_Draw_Context     RGBA_Draw_Context;
-typedef struct _RGBA_Gradient         RGBA_Gradient;
-typedef struct _RGBA_Gradient_Color_Stop   RGBA_Gradient_Color_Stop;
-typedef struct _RGBA_Gradient_Alpha_Stop   RGBA_Gradient_Alpha_Stop;
-typedef struct _RGBA_Gradient_Type    RGBA_Gradient_Type;
-typedef struct _RGBA_Gradient2         RGBA_Gradient2;
-typedef struct _RGBA_Gradient2_Type    RGBA_Gradient2_Type;
-typedef struct _RGBA_Gradient2_Color_Np_Stop   RGBA_Gradient2_Color_Np_Stop;
 typedef struct _RGBA_Polygon_Point    RGBA_Polygon_Point;
 typedef struct _RGBA_Map_Point        RGBA_Map_Point;
 typedef struct _RGBA_Font             RGBA_Font;
@@ -427,16 +453,6 @@ typedef void (*Gfx_Func_Copy)    (DATA32 *src, DATA32 *dst, int len);
 
 typedef void (*Gfx_Func_Convert) (DATA32 *src, DATA8 *dst, int src_jump, int dst_jump, int w, int h, int dith_x, int dith_y, DATA8 *pal);
 
-typedef void (*Gfx_Func_Gradient_Fill)(DATA32 *src, int src_len,
-                                         DATA32 *dst, DATA8 *mask, int len,
-                                         int x, int y, int axx, int axy, int ayx, int ayy,
-                                         void *geom_data);
-
-typedef void (*Gfx_Func_Gradient2_Fill)(DATA32 *src, int src_len,
-                                         DATA32 *dst, DATA8 *mask, int len,
-                                         int x, int y,
-                                         void *geom_data);
-
 #include "../cache/evas_cache.h"
 
 /*****************************************************************************/
@@ -498,9 +514,9 @@ struct _RGBA_Image_Loadopts
 {
    int                  scale_down_by; // if > 1 then use this
    double               dpi; // if > 0.0 use this
-   int                  w, h; // if > 0 use this
+   unsigned int         w, h; // if > 0 use this
    struct {
-      int               x, y, w, h;
+      unsigned int      x, y, w, h;
    } region;
 };
 
@@ -551,17 +567,27 @@ struct _Image_Entry
    LK(lock_references);   // needed for accessing references
 #endif
 
+#ifdef BUILD_PIPE_RENDER
+   RGBA_Pipe           *pipe;
+#ifdef EVAS_FRAME_QUEUING
+   LK(ref_fq_add);
+   LK(ref_fq_del);
+   pthread_cond_t cond_fq_del;
+   int ref_fq[2];		// ref_fq[0] is for addition, ref_fq[1] is for deletion
+#endif
+#endif
+
    unsigned char          scale;
 
    RGBA_Image_Loadopts    load_opts;
    int                    space;
-   int                    w;
-   int                    h;
+   unsigned int           w;
+   unsigned int           h;
 
    struct
      {
-        int		 w;
-        int		 h;
+        unsigned int w;
+        unsigned int h;
      } allocated;
 
    struct
@@ -572,6 +598,8 @@ struct _Image_Entry
 
 #ifdef BUILD_ASYNC_PRELOAD
    LK(lock);
+   LK(lock_cancel);
+   Eina_Bool unload_cancel : 1;
 #endif
 
    Image_Entry_Flags      flags;
@@ -660,6 +688,7 @@ struct _RGBA_Draw_Context
 
 #ifdef BUILD_PIPE_RENDER
 #include "../engines/common/evas_map_image.h"
+#include "../engines/common/evas_bidi_utils.h"
 
 struct _RGBA_Pipe_Op
 {
@@ -677,18 +706,11 @@ struct _RGBA_Pipe_Op
       struct {
 	 RGBA_Polygon_Point *points;
       } poly;
-      struct {
-	 RGBA_Gradient      *grad;
-	 int                 x, y, w, h;
-      } grad;
-      struct {
-	 RGBA_Gradient2      *grad;
-	 int                 x, y, w, h;
-      } grad2;
       struct {
 	 RGBA_Font          *font;
 	 int                 x, y;
-	 char               *text;
+	 Eina_Unicode       *text;
+         Evas_BiDi_Props     intl_props;
       } text;
       struct {
 	 RGBA_Image         *src;
@@ -736,15 +758,6 @@ struct _RGBA_Image
      } info;
 
    void                *extended_info;
-#ifdef BUILD_PIPE_RENDER
-   RGBA_Pipe           *pipe;
-#ifdef EVAS_FRAME_QUEUING
-   LK(ref_fq_add);
-   LK(ref_fq_del);
-   pthread_cond_t cond_fq_del;
-   int ref_fq[2];		// ref_fq[0] is for addition, ref_fq[1] is for deletion
-#endif
-#endif
    int                  ref;
 
 /*    unsigned char        scale; */
@@ -773,150 +786,6 @@ struct _RGBA_Image
    } cache;
 };
 
-struct _RGBA_Gradient_Color_Stop
-{
-   EINA_INLIST;
-   int               r, g, b, a;
-   int               dist;
-};
-
-struct _RGBA_Gradient_Alpha_Stop
-{
-   EINA_INLIST;
-   int               a;
-   int               dist;
-};
-
-struct _RGBA_Gradient
-{
-   struct
-     {
-	DATA32        *data;
-	int            len;
-	float          angle;
-	int            direction;
-	float          offset;
-	Eina_Bool      has_alpha : 1;
-     } map;
-
-   struct {
-	Eina_Inlist *stops;
-	DATA32           *data;
-	int               nstops;
-	int               len;
-   }  color;
-   struct {
-	Eina_Inlist *stops;
-	DATA8            *data;
-	int               nstops;
-	int               len;
-   }  alpha;
-
-   struct
-     {
-	int            x, y, w, h;
-	int            spread;
-	float          angle;
-     } fill;
-   struct
-     {
-	char          *name;
-	char          *params;
-	RGBA_Gradient_Type *geometer;
-	void          *gdata;
-     } type;
-
-   int references;
-#ifdef EVAS_FRAME_QUEUING
-   LK(ref_fq_add);
-   LK(ref_fq_del);
-   pthread_cond_t cond_fq_del;
-   int ref_fq[2];	//ref_fq[0] is for addition,
-                         //ref_fq[1] is for deletion
-#endif
-
-   Eina_Bool imported_data : 1;
-   Eina_Bool has_alpha : 1;
-};
-
-struct _RGBA_Gradient_Type
-{
-   const char              *name;
-   void                    (*init)(void);
-   void                    (*shutdown)(void);
-   void                    (*geom_init)(RGBA_Gradient *gr);
-   void                    (*geom_set)(RGBA_Gradient *gr);
-   void                    (*geom_free)(void *gdata);
-   int                     (*has_alpha)(RGBA_Gradient *gr, int render_op);
-   int                     (*has_mask)(RGBA_Gradient *gr, int render_op);
-   int                     (*get_map_len)(RGBA_Gradient *gr);
-   Gfx_Func_Gradient_Fill  (*get_fill_func)(RGBA_Gradient *gr, int render_op, unsigned char aa);
-};
-
-struct _RGBA_Gradient2_Color_Np_Stop
-{
-   EINA_INLIST;
-   int               r, g, b, a;
-   float             pos;
-   int               dist;
-};
-
-struct _RGBA_Gradient2
-{
-   struct
-     {
-	DATA32        *data;
-	int            len;
-	Eina_Bool      has_alpha : 1;
-     } map;
-
-   struct {
-	Eina_Inlist *stops;
-	int               nstops;
-	DATA32           *cdata;
-	DATA8            *adata;
-	int               len;
-   }  stops;
-
-   struct
-     {
-	Evas_Common_Transform  transform;
-	int                    spread;
-     } fill;
-   struct
-     {
-        int                 id;
-	RGBA_Gradient2_Type *geometer;
-	void                *gdata;
-     } type;
-
-   int references;
-#ifdef EVAS_FRAME_QUEUING
-   LK(ref_fq_add);
-   LK(ref_fq_del);
-   pthread_cond_t cond_fq_del;
-   int ref_fq[2];	//ref_fq[0] is for addition,
-                         //ref_fq[1] is for deletion
-#endif
-
-   Eina_Bool has_alpha : 1;
-};
-
-struct _RGBA_Gradient2_Type
-{
-   const char              *name;
-   void                    (*init)(void);
-   void                    (*shutdown)(void);
-   void                    (*geom_init)(RGBA_Gradient2 *gr);
-   void                    (*geom_update)(RGBA_Gradient2 *gr);
-   void                    (*geom_free)(void *gdata);
-   int                     (*has_alpha)(RGBA_Gradient2 *gr, int render_op);
-   int                     (*has_mask)(RGBA_Gradient2 *gr, int render_op);
-   int                     (*get_map_len)(RGBA_Gradient2 *gr);
-   Gfx_Func_Gradient2_Fill  (*get_fill_func)(RGBA_Gradient2 *gr, int render_op);
-};
-
-
 struct _RGBA_Polygon_Point
 {
    EINA_INLIST;
@@ -926,16 +795,21 @@ struct _RGBA_Polygon_Point
 struct _RGBA_Map_Point
 {
    FPc x, y; // x, y screenspace
+   float fx, fy, fz; // x, y, z in floats
+//   FPc x3, y3; // x, y 3d space
    FPc z; // z in world space. optional
    FPc u, v; // u, v in tex coords
    DATA32 col; // color at this point
+   // for perspective correctness - only point 0 has relevant info
+   FPc px, py, z0, foc;
 };
 
 // for fonts...
 /////
 typedef struct _Fash_Item_Index_Map Fash_Item_Index_Map;
-typedef struct _Fash_Int_Map Fash_Int_Map;
-typedef struct _Fash_Int Fash_Int;
+typedef struct _Fash_Int_Map        Fash_Int_Map;
+typedef struct _Fash_Int_Map2       Fash_Int_Map2;
+typedef struct _Fash_Int            Fash_Int;
 struct _Fash_Item_Index_Map
 {
    RGBA_Font_Int *fint;
@@ -945,85 +819,83 @@ struct _Fash_Int_Map
 {
   Fash_Item_Index_Map item[256];
 };
-struct _Fash_Int
+struct _Fash_Int_Map2
 {
    Fash_Int_Map *bucket[256];
+};
+struct _Fash_Int
+{
+   Fash_Int_Map2 *bucket[256];
    void (*freeme) (Fash_Int *fash);
 };
 
 /////
-typedef struct _Fash_Glyph_Map Fash_Glyph_Map;
-typedef struct _Fash_Glyph Fash_Glyph;
+typedef struct _Fash_Glyph_Map  Fash_Glyph_Map;
+typedef struct _Fash_Glyph_Map2 Fash_Glyph_Map2;
+typedef struct _Fash_Glyph      Fash_Glyph;
 struct _Fash_Glyph_Map
 {
    RGBA_Font_Glyph *item[256];
 };
-struct _Fash_Glyph
+struct _Fash_Glyph_Map2
 {
    Fash_Glyph_Map *bucket[256];
+};
+struct _Fash_Glyph
+{
+   Fash_Glyph_Map2 *bucket[256];
    void (*freeme) (Fash_Glyph *fash);
 };
 /////
 
 struct _RGBA_Font
 {
-   Eina_List *fonts;
-   Font_Hint_Flags hinting;
-   int references;
-   Fash_Int *fash;
-   unsigned char sizeok : 1;
-   LK(lock);
+   Eina_List       *fonts;
+   Fash_Int        *fash;
+   Font_Hint_Flags  hinting;
+   int              references;
 #ifdef EVAS_FRAME_QUEUING
+   int              ref_fq[2]; //ref_fq[0] is for addition, ref_fq[1] is for deletion
+   pthread_cond_t   cond_fq_del;
    LK(ref_fq_add);
    LK(ref_fq_del);
-   pthread_cond_t cond_fq_del;
-   int ref_fq[2];		//ref_fq[0] is for addition, ref_fq[1] is for deletion
 #endif
+   LK(lock);
+   unsigned char    sizeok : 1;
 };
 
 struct _RGBA_Font_Int
 {
+   EINA_INLIST;
    RGBA_Font_Source *src;
-
+   Eina_Hash        *kerning;
+   Fash_Glyph       *fash;
    unsigned int      size;
    int               real_size;
    int               max_h;
-
+   int               references;
+   int               usage;
    struct {
       FT_Size       size;
    } ft;
-
-//   Eina_Hash       *glyphs;
-
    LK(ft_mutex);
-
-   Eina_Hash       *kerning;
-//   Eina_Hash       *indexes;
-
-   int              usage;
-   Font_Hint_Flags hinting;
-
-   int              references;
-
-   Fash_Glyph *fash;
-   unsigned char sizeok : 1;
+   Font_Hint_Flags  hinting;
+   unsigned char    sizeok : 1;
+   unsigned char    inuse : 1;
 };
 
 struct _RGBA_Font_Source
 {
    const char       *name;
    const char       *file;
-
    void             *data;
+   unsigned int      current_size;
    int               data_size;
-   int               current_size;
-   
+   int               references;
    struct {
-      int           orig_upem;
-      FT_Face       face;
+      int            orig_upem;
+      FT_Face        face;
    } ft;
-
-   int              references;
 };
 
 struct _RGBA_Font_Glyph
@@ -1280,7 +1152,6 @@ EAPI void     evas_common_blit_init               (void);
 EAPI void     evas_common_blit_rectangle          (const RGBA_Image *src, RGBA_Image *dst, int src_x, int src_y, int w, int h, int dst_x, int dst_y);
 
 /****/
-#include "../engines/common/evas_gradient.h"
 #include "../engines/common/evas_font.h"
 
 /****/
@@ -1322,13 +1193,8 @@ void              evas_font_dir_cache_free(void);
 
 /*****************************************************************************/
 
-//#if defined(__ARM_ARCH__) && (__ARM_ARCH__ >= 70)
-#ifdef BUILD_NEON
-# include <arm_neon.h>
-#endif
-//#endif
-
 #ifdef __cplusplus
 }
 #endif
+
 #endif
