@@ -32,12 +32,10 @@ extern "C" {
    ((((g) >> 2) & 0x3f) << 5) |                                         \
    (((b) >> 3) & 0x1f))
 
-static inline int
-_calc_stride(int w)
+static inline unsigned int
+_calc_stride(unsigned int w)
 {
-   int pad;
-
-   pad = w % 4;
+   unsigned int pad = w % 4;
    if (!pad)  return w;
    else return w + 4 - pad;
 }
@@ -71,37 +69,37 @@ EAPI void                evas_common_soft16_image_init(void);
 EAPI void                evas_common_soft16_image_shutdown(void);
 EAPI Evas_Cache_Image   *evas_common_soft16_image_cache_get(void);
 
-EAPI void                soft16_image_draw(Soft16_Image *src, Soft16_Image *dst, RGBA_Draw_Context *dc, int src_region_x, int src_region_y, int src_region_w, int src_region_h, int dst_region_x, int dst_region_y, int dst_region_w, int dst_region_h, int smooth);
-EAPI Soft16_Image       *soft16_image_alpha_set(Soft16_Image *im, int have_alpha);
+EAPI void                evas_common_soft16_image_draw(Soft16_Image *src, Soft16_Image *dst, RGBA_Draw_Context *dc, int src_region_x, int src_region_y, int src_region_w, int src_region_h, int dst_region_x, int dst_region_y, int dst_region_w, int dst_region_h, int smooth);
+EAPI Soft16_Image       *evas_common_soft16_image_alpha_set(Soft16_Image *im, int have_alpha);
 
-void                     soft16_image_draw_unscaled(Soft16_Image *src, Soft16_Image *dst, RGBA_Draw_Context *dc, const Eina_Rectangle sr, const Eina_Rectangle dr, const Eina_Rectangle cr);
-void                     soft16_image_draw_scaled_sampled(Soft16_Image *src, Soft16_Image *dst, RGBA_Draw_Context *dc, const Eina_Rectangle sr, const Eina_Rectangle dr, const Eina_Rectangle cr);
+void                     evas_common_soft16_image_draw_unscaled(Soft16_Image *src, Soft16_Image *dst, RGBA_Draw_Context *dc, const Eina_Rectangle sr, const Eina_Rectangle dr, const Eina_Rectangle cr);
+void                     evas_common_soft16_image_draw_scaled_sampled(Soft16_Image *src, Soft16_Image *dst, RGBA_Draw_Context *dc, const Eina_Rectangle sr, const Eina_Rectangle dr, const Eina_Rectangle cr);
 
 /* convert/dither functions */
-void                     soft16_image_convert_from_rgb(Soft16_Image *im, const DATA32 *src);
-void                     soft16_image_convert_from_rgba(Soft16_Image *im, const DATA32 *src);
+void                     evas_common_soft16_image_convert_from_rgb(Soft16_Image *im, const DATA32 *src);
+void                     evas_common_soft16_image_convert_from_rgba(Soft16_Image *im, const DATA32 *src);
 
 /**
  * Rectangle (evas_soft16_rectangle.c)
  */
-EAPI void                soft16_rectangle_draw(Soft16_Image *dst, RGBA_Draw_Context *dc, int x, int y, int w, int h);
+EAPI void                evas_common_soft16_rectangle_draw(Soft16_Image *dst, RGBA_Draw_Context *dc, int x, int y, int w, int h);
 
 /**
  * Polygon (evas_soft16_polygon.c)
  */
-  EAPI void                soft16_polygon_draw(Soft16_Image *dst, RGBA_Draw_Context *dc, RGBA_Polygon_Point *points, int x, int y);
+  EAPI void                evas_common_soft16_polygon_draw(Soft16_Image *dst, RGBA_Draw_Context *dc, RGBA_Polygon_Point *points, int x, int y);
 
 /**
  * Line (evas_soft16_line.c)
  */
-EAPI void                soft16_line_draw(Soft16_Image *dst, RGBA_Draw_Context *dc, int x0, int y0, int x1, int y1);
+EAPI void                evas_common_soft16_line_draw(Soft16_Image *dst, RGBA_Draw_Context *dc, int x0, int y0, int x1, int y1);
 
 /**
  * Font (evas_soft16_font.c)
  */
-EAPI void               *soft16_font_glyph_new(void *data, RGBA_Font_Glyph *fg);
-EAPI void                soft16_font_glyph_free(void *ext_dat);
-EAPI void                soft16_font_glyph_draw(void *data, void *dest, void *context, RGBA_Font_Glyph *fg, int x, int y);
+EAPI void               *evas_common_soft16_font_glyph_new(void *data, RGBA_Font_Glyph *fg);
+EAPI void                evas_common_soft16_font_glyph_free(void *ext_dat);
+EAPI void                evas_common_soft16_font_glyph_draw(void *data, void *dest, void *context, RGBA_Font_Glyph *fg, int x, int y);
 
 #ifdef __cplusplus
 }
