@@ -1,10 +1,6 @@
 #ifndef EVAS_PRIVATE_H
 #define EVAS_PRIVATE_H
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -15,6 +11,7 @@
 
 #include "../file/evas_module.h"
 #include "../file/evas_path.h"
+#include "../engines/common/evas_bidi_utils.h"
 
 #ifdef EVAS_MAGIC_DEBUG
 /* complain when peole pass in wrong object types etc. */
@@ -40,29 +37,23 @@ typedef struct _Evas_Image_Load_Func        Evas_Image_Load_Func;
 typedef struct _Evas_Image_Save_Func        Evas_Image_Save_Func;
 typedef struct _Evas_Object_Func            Evas_Object_Func;
 typedef struct _Evas_Intercept_Func         Evas_Intercept_Func;
-typedef struct _Evas_Intercept_Func_Basic   Evas_Intercept_Func_Basic;
-typedef struct _Evas_Intercept_Func_SizePos Evas_Intercept_Func_SizePos;
-typedef struct _Evas_Intercept_Func_Obj     Evas_Intercept_Func_Obj;
-typedef struct _Evas_Intercept_Func_Int     Evas_Intercept_Func_Int;
-typedef struct _Evas_Intercept_Func_Color   Evas_Intercept_Func_Color;
 typedef struct _Evas_Key_Grab               Evas_Key_Grab;
 typedef struct _Evas_Callbacks              Evas_Callbacks;
 typedef struct _Evas_Format                 Evas_Format;
 typedef struct _Evas_Map_Point              Evas_Map_Point;
+typedef struct _Evas_Smart_Cb_Description_Array Evas_Smart_Cb_Description_Array;
+typedef struct _Evas_Post_Callback          Evas_Post_Callback;
 
 #define MAGIC_EVAS                 0x70777770
 #define MAGIC_OBJ                  0x71777770
 #define MAGIC_OBJ_RECTANGLE        0x71777771
 #define MAGIC_OBJ_LINE             0x71777772
-#define MAGIC_OBJ_GRADIENT         0x71777773
 #define MAGIC_OBJ_POLYGON          0x71777774
 #define MAGIC_OBJ_IMAGE            0x71777775
 #define MAGIC_OBJ_TEXT             0x71777776
 #define MAGIC_OBJ_SMART            0x71777777
 #define MAGIC_OBJ_TEXTBLOCK        0x71777778
 #define MAGIC_SMART                0x72777770
-#define MAGIC_OBJ_GRADIENT_LINEAR  0x72777771
-#define MAGIC_OBJ_GRADIENT_RADIAL  0x72777772
 #define MAGIC_OBJ_SHAPE            0x72777773
 #define MAGIC_OBJ_CONTAINER        0x72777774
 #define MAGIC_OBJ_CUSTOM           0x72777775
@@ -116,36 +107,6 @@ MAGIC_CHECK_FAILED(o, t, m)
 	 (o)->prev.key = NULL;                                              \
      }
 
-struct _Evas_Intercept_Func_Basic
-{
-   void (*func) (void *data, Evas_Object *obj);
-   void *data;
-};
-
-struct _Evas_Intercept_Func_SizePos
-{
-   void (*func) (void *data, Evas_Object *obj, Evas_Coord x, Evas_Coord y);
-   void *data;
-};
-
-struct _Evas_Intercept_Func_Obj
-{
-   void (*func) (void *data, Evas_Object *obj, Evas_Object *obj2);
-   void *data;
-};
-
-struct _Evas_Intercept_Func_Int
-{
-   void (*func) (void *data, Evas_Object *obj, int n);
-   void *data;
-};
-
-struct _Evas_Intercept_Func_Color
-{
-   void (*func) (void *data, Evas_Object *obj, int r, int g, int b, int a);
-   void *data;
-};
-
 struct _Evas_Key_Grab
 {
    char               *keyname;
@@ -159,18 +120,60 @@ struct _Evas_Key_Grab
 
 struct _Evas_Intercept_Func
 {
-   Evas_Intercept_Func_Basic   show;
-   Evas_Intercept_Func_Basic   hide;
-   Evas_Intercept_Func_SizePos move;
-   Evas_Intercept_Func_SizePos resize;
-   Evas_Intercept_Func_Basic   raise;
-   Evas_Intercept_Func_Basic   lower;
-   Evas_Intercept_Func_Obj     stack_above;
-   Evas_Intercept_Func_Obj     stack_below;
-   Evas_Intercept_Func_Int     layer_set;
-   Evas_Intercept_Func_Color   color_set;
-   Evas_Intercept_Func_Obj     clip_set;
-   Evas_Intercept_Func_Basic   clip_unset;
+   struct {
+      Evas_Object_Intercept_Show_Cb func;
+      void *data;
+   } show;
+   struct {
+      Evas_Object_Intercept_Hide_Cb func;
+      void *data;
+   } hide;
+   struct {
+      Evas_Object_Intercept_Move_Cb func;
+      void *data;
+   } move;
+   struct {
+      Evas_Object_Intercept_Resize_Cb func;
+      void *data;
+   } resize;
+   struct {
+      Evas_Object_Intercept_Raise_Cb func;
+      void *data;
+   } raise;
+   struct {
+      Evas_Object_Intercept_Lower_Cb func;
+      void *data;
+   } lower;
+   struct {
+      Evas_Object_Intercept_Stack_Above_Cb func;
+      void *data;
+   } stack_above;
+   struct {
+      Evas_Object_Intercept_Stack_Below_Cb func;
+      void *data;
+   } stack_below;
+   struct {
+      Evas_Object_Intercept_Layer_Set_Cb func;
+      void *data;
+   } layer_set;
+   struct {
+      Evas_Object_Intercept_Color_Set_Cb func;
+      void *data;
+   } color_set;
+   struct {
+      Evas_Object_Intercept_Clip_Set_Cb func;
+      void *data;
+   } clip_set;
+   struct {
+      Evas_Object_Intercept_Clip_Unset_Cb func;
+      void *data;
+   } clip_unset;
+};
+
+struct _Evas_Smart_Cb_Description_Array
+{
+   unsigned int                      size;
+   const Evas_Smart_Cb_Description **array;
 };
 
 struct _Evas_Smart
@@ -181,6 +184,8 @@ struct _Evas_Smart
 
    const Evas_Smart_Class *smart_class;
 
+   Evas_Smart_Cb_Description_Array callbacks;
+
    unsigned char     delete_me : 1;
    unsigned char     class_allocated : 1;
 
@@ -204,29 +209,19 @@ struct _Evas_Lock
    Evas_Modifier_Mask mask; /* we have a max of 64 locks */
 };
 
+struct _Evas_Post_Callback
+{
+   Evas_Object               *obj;
+   Evas_Object_Event_Post_Cb  func;
+   const void                *data;
+   unsigned char              delete_me : 1;
+};
+
 struct _Evas_Callbacks
 {
    Eina_Inlist *callbacks;
    int               walking_list;
    unsigned char     deletions_waiting : 1;
-/*
-   Eina_Inlist *down;
-   Eina_Inlist *up;
-   Eina_Inlist *move;
-   Eina_Inlist *in;
-   Eina_Inlist *out;
-   Eina_Inlist *wheel;
-   Eina_Inlist *key_down;
-   Eina_Inlist *key_up;
-   Eina_Inlist *free;
-   Eina_Inlist *obj_focus_in;
-   Eina_Inlist *obj_focus_out;
-   Eina_Inlist *obj_show;
-   Eina_Inlist *obj_hide;
-   Eina_Inlist *obj_move;
-   Eina_Inlist *obj_resize;
-   Eina_Inlist *obj_restack;
- */
 };
 
 struct _Evas
@@ -240,9 +235,6 @@ struct _Evas
       int            mouse_grabbed;
       DATA32         button;
       Evas_Coord     x, y;
-
-////      Evas_Coord         canvas_x, canvas_y;
-
       struct {
 	 Eina_List *in;
       } object;
@@ -295,6 +287,10 @@ struct _Evas
    Eina_Array     calculate_objects;
    Eina_Array     clip_changes;
 
+   Eina_List     *post_events; // free me on evas_free
+
+   Evas_Callbacks *callbacks;
+
    int            delete_grabs;
    int            walking_grabs;
    Eina_List     *grabs;
@@ -308,11 +304,13 @@ struct _Evas
    unsigned int   last_timestamp;
    int            last_mouse_down_counter;
    int            last_mouse_up_counter;
+   int            nochange;
    Evas_Font_Hinting_Flags hinting;
    unsigned char  changed : 1;
    unsigned char  delete_me : 1;
    unsigned char  invalidate : 1;
    unsigned char  cleanup : 1;
+   unsigned char  focus : 1;
 };
 
 struct _Evas_Layer
@@ -360,87 +358,84 @@ struct _Evas_Size_Hints
 
 struct _Evas_Map_Point
 {
-   Evas_Coord x, y, z;
+   double x, y, z, px, py;
    double u, v;
    unsigned char r, g, b, a;
 };
 
 struct _Evas_Map
 {
-   int count; // num of points
-   Evas_Coord_Rectangle normal_geometry; // bounding box of map geom actually
-   void *surface; // surface holding map if needed
-   int surface_w, surface_h; // current surface w & h alloc
-   Evas_Coord mx, my; // mouse x, y after conversion to map space
-   Eina_Bool alpha : 1;
-   Eina_Bool smooth : 1;
-   Evas_Map_Point points[]; // actual points
+   int                   count; // num of points
+   Evas_Coord_Rectangle  normal_geometry; // bounding box of map geom actually
+   void                 *surface; // surface holding map if needed
+   int                   surface_w, surface_h; // current surface w & h alloc
+   Evas_Coord            mx, my; // mouse x, y after conversion to map space
+   struct {
+      Evas_Coord         px, py, z0, foc;
+   } persp;
+   Eina_Bool             alpha : 1;
+   Eina_Bool             smooth : 1;
+   Evas_Map_Point        points[]; // actual points
 };
 
 struct _Evas_Object
 {
    EINA_INLIST;
 
-   DATA32            magic;
+   DATA32                   magic;
 
-   const char       *type;
-   Evas_Layer       *layer;
+   const char              *type;
+   Evas_Layer              *layer;
 
    struct {
+      Evas_Map             *map;
+      Evas_Object          *clipper;
+      Evas_Object          *map_parent;
+      double                scale;
+      Evas_Coord_Rectangle  geometry;
       struct {
-/*
 	 struct {
-	    int            x, y, w, h;
-	    int            validity;
-	 } geometry;
- */
-	 struct {
-	    int            x, y, w, h;
-	    unsigned char  r, g, b, a;
-	    Eina_Bool      visible : 1;
-	    Eina_Bool      dirty : 1;
+	    Evas_Coord      x, y, w, h;
+	    unsigned char   r, g, b, a;
+	    Eina_Bool       visible : 1;
+	    Eina_Bool       dirty : 1;
 	 } clip;
       } cache;
-      double scale;
-      Evas_Map *map;
-      Evas_Coord_Rectangle geometry;
+      short                 layer;
       struct {
-	 unsigned char  r, g, b, a;
+	 unsigned char      r, g, b, a;
       } color;
-      Evas_Object      *clipper;
-      short             layer;
-      Eina_Bool         usemap : 1;
-      Eina_Bool         visible : 1;
-      Eina_Bool         have_clipees : 1;
-      Eina_Bool         anti_alias : 1;
-      unsigned char     interpolation_color_space : 1;
-      Evas_Render_Op    render_op : 4;
+      Eina_Bool             usemap : 1;
+      Eina_Bool             visible : 1;
+      Eina_Bool             have_clipees : 1;
+      Eina_Bool             anti_alias : 1;
+      Evas_Render_Op        render_op : 4;
    } cur, prev;
 
    char                       *name;
 
-   Evas_Intercept_Func *interceptors;
+   Evas_Intercept_Func        *interceptors;
 
    struct {
-      Eina_List *elements;
+      Eina_List               *elements;
    } data;
 
-   Eina_List *grabs;
+   Eina_List                  *grabs;
 
-   Evas_Callbacks *callbacks;
+   Evas_Callbacks             *callbacks;
 
    struct {
-      Eina_List   *clipees;
-      Eina_List   *changes;
+      Eina_List               *clipees;
+      Eina_List               *changes;
    } clip;
 
-   const Evas_Object_Func *func;
+   const Evas_Object_Func     *func;
 
-   void             *object_data;
+   void                       *object_data;
 
    struct {
-      Evas_Smart       *smart;
-      Evas_Object      *parent;
+      Evas_Smart              *smart;
+      Evas_Object             *parent;
    } smart;
 
    Evas_Size_Hints            *size_hints;
@@ -449,6 +444,14 @@ struct _Evas_Object
    int                         last_mouse_up_counter;
    int                         mouse_grabbed;
 
+   int                         last_event;
+   
+   struct {
+      int                      in_move, in_resize;
+   } doing;
+
+   unsigned char               delete_me;
+
    Evas_Object_Pointer_Mode    pointer_mode : 1;
 
    Eina_Bool                   store : 1;
@@ -457,9 +460,10 @@ struct _Evas_Object
    Eina_Bool                   parent_cache_valid : 1;
    Eina_Bool                   repeat_events : 1;
    Eina_Bool                   restack : 1;
-   Eina_Bool                   changed : 1;
    Eina_Bool                   is_active : 1;
-   
+   Eina_Bool                   precise_is_inside : 1;
+   Eina_Bool                   is_static_clip : 1;
+
    Eina_Bool                   render_pre : 1;
    Eina_Bool                   rect_del : 1;
    Eina_Bool                   mouse_in : 1;
@@ -468,17 +472,17 @@ struct _Evas_Object
    Eina_Bool                   focused : 1;
    Eina_Bool                   in_layer : 1;
    Eina_Bool                   no_propagate : 1;
-   
-   Eina_Bool                   precise_is_inside : 1;
-   Eina_Bool                   havemap_parent : 1;
 
-   unsigned char               delete_me;
+   Eina_Bool                   changed : 1;
+   Eina_Bool                   changed_move : 1;
+   Eina_Bool                   changed_move_only : 1;
+   Eina_Bool                   changed_nomove : 1;
 };
 
 struct _Evas_Func_Node
 {
    EINA_INLIST;
-   void (*func) (void *data, Evas *e, Evas_Object *obj, void *event_info);
+   void (*func) ();
    void *data;
    Evas_Callback_Type type;
    unsigned char delete_me : 1;
@@ -542,7 +546,7 @@ struct _Evas_Object_Func
    int  (*was_inside) (Evas_Object *obj, Evas_Coord x, Evas_Coord y);
 
    void (*coords_recalc) (Evas_Object *obj);
-   
+
    void (*scale_update) (Evas_Object *obj);
 
    int (*has_opaque_rect) (Evas_Object *obj);
@@ -567,6 +571,7 @@ struct _Evas_Func
    void (*output_redraws_next_update_push) (void *data, void *surface, int x, int y, int w, int h);
    void (*output_flush)                    (void *data);
    void (*output_idle_flush)               (void *data);
+   void (*output_dump)                     (void *data);
 
    void *(*context_new)                    (void *data);
    Eina_Bool (*canvas_alpha_get)           (void *data, void *context);
@@ -595,50 +600,7 @@ struct _Evas_Func
 
    void *(*polygon_point_add)              (void *data, void *context, void *polygon, int x, int y);
    void *(*polygon_points_clear)           (void *data, void *context, void *polygon);
-   void (*polygon_draw)                    (void *data, void *context, void *surface, void *polygon);
-
-   void (*gradient2_color_np_stop_insert)   (void *data, void *gradient, int r, int g, int b, int a, float pos);
-   void (*gradient2_clear)                  (void *data, void *gradient);
-   void (*gradient2_fill_transform_set)     (void *data, void *gradient, void *transform);
-   void (*gradient2_fill_spread_set)        (void *data, void *gradient, int spread);
-
-   void *(*gradient2_linear_new)                   (void *data);
-   void (*gradient2_linear_free)                   (void *data, void *linear_gradient);
-   void (*gradient2_linear_fill_set)               (void *data, void *linear_gradient, float x0, float y0, float x1, float y1);
-   int  (*gradient2_linear_is_opaque)              (void *data, void *context, void *linear_gradient, int x, int y, int w, int h);
-   int  (*gradient2_linear_is_visible)             (void *data, void *context, void *linear_gradient, int x, int y, int w, int h);
-   void (*gradient2_linear_render_pre)             (void *data, void *context, void *linear_gradient);
-   void (*gradient2_linear_render_post)            (void *data, void *linear_gradient);
-   void (*gradient2_linear_draw)                   (void *data, void *context, void *surface, void *linear_gradient, int x, int y, int w, int h);
-
-   void *(*gradient2_radial_new)                   (void *data);
-   void (*gradient2_radial_free)                   (void *data, void *radial_gradient);
-   void (*gradient2_radial_fill_set)               (void *data, void *radial_gradient, float cx, float cy, float rx, float ry);
-   int  (*gradient2_radial_is_opaque)              (void *data, void *context, void *radial_gradient, int x, int y, int w, int h);
-   int  (*gradient2_radial_is_visible)             (void *data, void *context, void *radial_gradient, int x, int y, int w, int h);
-   void (*gradient2_radial_render_pre)             (void *data, void *context, void *radial_gradient);
-   void (*gradient2_radial_render_post)            (void *data, void *radial_gradient);
-   void (*gradient2_radial_draw)                   (void *data, void *context, void *surface, void *radial_gradient, int x, int y, int w, int h);
-
-   void *(*gradient_new)                   (void *data);
-   void (*gradient_free)                   (void *data, void *gradient);
-   void (*gradient_color_stop_add)         (void *data, void *gradient, int r, int g, int b, int a, int delta);
-   void (*gradient_alpha_stop_add)         (void *data, void *gradient, int a, int delta);
-   void (*gradient_color_data_set)         (void *data, void *gradient, void *map, int len, int alpha_flag);
-   void (*gradient_alpha_data_set)         (void *data, void *gradient, void *alpha_map, int len);
-   void (*gradient_clear)                  (void *data, void *gradient);
-   void (*gradient_fill_set)               (void *data, void *gradient, int x, int y, int w, int h);
-   void (*gradient_fill_angle_set)         (void *data, void *gradient, double fill_angle);
-   void (*gradient_fill_spread_set)        (void *data, void *gradient, int spread);
-   void (*gradient_angle_set)              (void *data, void *gradient, double angle);
-   void (*gradient_offset_set)             (void *data, void *gradient, float offset);
-   void (*gradient_direction_set)          (void *data, void *gradient, int direction);
-   void (*gradient_type_set)               (void *data, void *gradient, char *name, char *params);
-   int  (*gradient_is_opaque)              (void *data, void *context, void *gradient, int x, int y, int w, int h);
-   int  (*gradient_is_visible)             (void *data, void *context, void *gradient, int x, int y, int w, int h);
-   void (*gradient_render_pre)             (void *data, void *context, void *gradient);
-   void (*gradient_render_post)            (void *data, void *gradient);
-   void (*gradient_draw)                   (void *data, void *context, void *surface, void *gradient, int x, int y, int w, int h);
+   void (*polygon_draw)                    (void *data, void *context, void *surface, void *polygon, int x, int y);
 
    void *(*image_load)                     (void *data, const char *file, const char *key, int *error, Evas_Image_Load_Opts *lo);
    void *(*image_new_from_data)            (void *data, int w, int h, DATA32 *image_data, int alpha, int cspace);
@@ -661,7 +623,7 @@ struct _Evas_Func
    char *(*image_format_get)               (void *data, void *image);
    void (*image_colorspace_set)            (void *data, void *image, int cspace);
    int  (*image_colorspace_get)            (void *data, void *image);
-   void (*image_native_set)                (void *data, void *image, void *native);
+   void *(*image_native_set)               (void *data, void *image, void *native);
    void *(*image_native_get)               (void *data, void *image);
 
    void (*image_cache_flush)               (void *data);
@@ -677,13 +639,13 @@ struct _Evas_Func
    int  (*font_descent_get)                (void *data, void *font);
    int  (*font_max_ascent_get)             (void *data, void *font);
    int  (*font_max_descent_get)            (void *data, void *font);
-   void (*font_string_size_get)            (void *data, void *font, const char *text, int *w, int *h);
-   int  (*font_inset_get)                  (void *data, void *font, const char *text);
-   int  (*font_h_advance_get)              (void *data, void *font, const char *text);
-   int  (*font_v_advance_get)              (void *data, void *font, const char *text);
-   int  (*font_char_coords_get)            (void *data, void *font, const char *text, int pos, int *cx, int *cy, int *cw, int *ch);
-   int  (*font_char_at_coords_get)         (void *data, void *font, const char *text, int x, int y, int *cx, int *cy, int *cw, int *ch);
-   void (*font_draw)                       (void *data, void *context, void *surface, void *font, int x, int y, int w, int h, int ow, int oh, const char *text);
+   void (*font_string_size_get)            (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props, int *w, int *h);
+   int  (*font_inset_get)                  (void *data, void *font, const Eina_Unicode *text);
+   int  (*font_h_advance_get)              (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props);
+   int  (*font_v_advance_get)              (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props);
+   int  (*font_char_coords_get)            (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props, int pos, int *cx, int *cy, int *cw, int *ch);
+   int  (*font_char_at_coords_get)         (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props, int x, int y, int *cx, int *cy, int *cw, int *ch);
+   void (*font_draw)                       (void *data, void *context, void *surface, void *font, int x, int y, int w, int h, int ow, int oh, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props);
 
    void (*font_cache_flush)                (void *data);
    void (*font_cache_set)                  (void *data, int bytes);
@@ -698,17 +660,21 @@ struct _Evas_Func
 
    void (*image_scale_hint_set)            (void *data, void *image, int hint);
    int  (*image_scale_hint_get)            (void *data, void *image);
-   int  (*font_last_up_to_pos)             (void *data, void *font, const char *text, int x, int y);
+   int  (*font_last_up_to_pos)             (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props, int x, int y);
 
    void (*image_map4_draw)                 (void *data, void *context, void *surface, void *image, RGBA_Map_Point *p, int smooth, int level);
    void *(*image_map_surface_new)          (void *data, int w, int h, int alpha);
-   void *(*image_map_surface_free)         (void *data, void *surface);
+   void (*image_map_surface_free)          (void *data, void *surface);
+
+   void (*image_content_hint_set)          (void *data, void *surface, int hint);
+   int  (*image_content_hint_get)          (void *data, void *surface);
 };
 
 struct _Evas_Image_Load_Func
 {
-  int (*file_head) (Image_Entry *ie, const char *file, const char *key);
-  int (*file_data) (Image_Entry *ie, const char *file, const char *key);
+  Eina_Bool threadable;
+  Eina_Bool (*file_head) (Image_Entry *ie, const char *file, const char *key, int *error);
+  Eina_Bool (*file_data) (Image_Entry *ie, const char *file, const char *key, int *error);
 };
 
 struct _Evas_Image_Save_Func
@@ -720,7 +686,7 @@ struct _Evas_Image_Save_Func
 extern "C" {
 #endif
 
-Evas_Object *evas_object_new(void);
+Evas_Object *evas_object_new(Evas *e);
 void evas_object_free(Evas_Object *obj, int clean_layer);
 void evas_object_inject(Evas_Object *obj, Evas *e);
 void evas_object_release(Evas_Object *obj, int clean_layer);
@@ -748,6 +714,10 @@ int evas_object_was_opaque(Evas_Object *obj);
 int evas_object_is_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
 int evas_object_was_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
 int evas_object_clippers_was_visible(Evas_Object *obj);
+void evas_object_clip_across_check(Evas_Object *obj);
+void evas_object_clip_across_clippees_check(Evas_Object *obj);
+void evas_object_mapped_clip_across_mark(Evas_Object *obj);
+void evas_event_callback_call(Evas *e, Evas_Callback_Type type, void *event_info);
 void evas_object_event_callback_call(Evas_Object *obj, Evas_Callback_Type type, void *event_info);
 Eina_List *evas_event_objects_event_list(Evas *e, Evas_Object *stop, int x, int y);
 int evas_mem_free(int mem_required);
@@ -760,6 +730,10 @@ void evas_debug_generic(const char *str);
 const char *evas_debug_magic_string_get(DATA32 magic);
 void evas_object_smart_use(Evas_Smart *s);
 void evas_object_smart_unuse(Evas_Smart *s);
+void evas_smart_cb_descriptions_fix(Evas_Smart_Cb_Description_Array *a) EINA_ARG_NONNULL(1);
+Eina_Bool evas_smart_cb_descriptions_resize(Evas_Smart_Cb_Description_Array *a, unsigned int size) EINA_ARG_NONNULL(1);
+const Evas_Smart_Cb_Description *evas_smart_cb_description_find(const Evas_Smart_Cb_Description_Array *a, const char *name) EINA_ARG_NONNULL(1, 2) EINA_PURE;
+
 void evas_object_smart_del(Evas_Object *obj);
 void evas_object_smart_cleanup(Evas_Object *obj);
 void evas_object_smart_member_raise(Evas_Object *member);
@@ -767,10 +741,16 @@ void evas_object_smart_member_lower(Evas_Object *member);
 void evas_object_smart_member_stack_above(Evas_Object *member, Evas_Object *other);
 void evas_object_smart_member_stack_below(Evas_Object *member, Evas_Object *other);
 const Eina_Inlist *evas_object_smart_members_get_direct(const Evas_Object *obj);
+void _evas_object_smart_members_all_del(Evas_Object *obj);
 void evas_call_smarts_calculate(Evas *e);
 void *evas_mem_calloc(int size);
+void _evas_post_event_callback_call(Evas *e);
+void _evas_post_event_callback_free(Evas *e);
+void evas_event_callback_list_post_free(Eina_Inlist **list);
 void evas_object_event_callback_all_del(Evas_Object *obj);
 void evas_object_event_callback_cleanup(Evas_Object *obj);
+void evas_event_callback_all_del(Evas *e);
+void evas_event_callback_cleanup(Evas *e);
 void evas_object_inform_call_show(Evas_Object *obj);
 void evas_object_inform_call_hide(Evas_Object *obj);
 void evas_object_inform_call_move(Evas_Object *obj);
@@ -778,6 +758,7 @@ void evas_object_inform_call_resize(Evas_Object *obj);
 void evas_object_inform_call_restack(Evas_Object *obj);
 void evas_object_inform_call_changed_size_hints(Evas_Object *obj);
 void evas_object_inform_call_image_preloaded(Evas_Object *obj);
+void evas_object_inform_call_image_unloaded(Evas_Object *obj);
 void evas_object_intercept_cleanup(Evas_Object *obj);
 int evas_object_intercept_call_show(Evas_Object *obj);
 int evas_object_intercept_call_hide(Evas_Object *obj);
@@ -808,6 +789,7 @@ void _evas_object_text_rehint(Evas_Object *obj);
 void _evas_object_textblock_rehint(Evas_Object *obj);
 
 extern int _evas_alloc_error;
+extern int _evas_event_counter;
 
 struct _Evas_Imaging_Image
 {
@@ -822,7 +804,15 @@ struct _Evas_Imaging_Font
 int evas_async_events_init(void);
 int evas_async_events_shutdown(void);
 int evas_async_target_del(const void *target);
-       
+
+void _evas_preload_thread_init(void);
+void _evas_preload_thread_shutdown(void);
+Evas_Preload_Pthread *evas_preload_thread_run(void (*func_heavy)(void *data),
+					     void (*func_end)(void *data),
+					     void (*func_cancel)(void *data),
+					     const void *data);
+Eina_Bool evas_preload_thread_cancel(Evas_Preload_Pthread *thread);
+
 void _evas_walk(Evas *e);
 void _evas_unwalk(Evas *e);
 
@@ -833,7 +823,65 @@ void evas_render_object_recalc(Evas_Object *obj);
 
 Eina_Bool evas_map_inside_get(const Evas_Map *m, Evas_Coord x, Evas_Coord y);
 Eina_Bool evas_map_coords_get(const Evas_Map *m, Evas_Coord x, Evas_Coord y, Evas_Coord *mx, Evas_Coord *my, int grab);
-       
+
+/****************************************************************************/
+/*****************************************/
+/********************/
+//#define MPOOL 1
+
+#ifdef MPOOL 
+typedef struct _Evas_Mempool Evas_Mempool;
+  
+struct _Evas_Mempool
+{
+  int           count;
+  int           num_allocs;
+  int           num_frees;
+  Eina_Mempool *mp;
+};
+# define EVAS_MEMPOOL(x) \
+   static Evas_Mempool x = {0, 0, 0, NULL}
+# define EVAS_MEMPOOL_INIT(x, nam, siz, cnt, ret) \
+   do { \
+     if (!x.mp) { \
+       x.mp = eina_mempool_add("chained_mempool", nam, NULL, sizeof(siz), cnt); \
+       if (!x.mp) { \
+         return ret; \
+       } \
+     } \
+   } while (0)
+# define EVAS_MEMPOOL_ALLOC(x, siz) \
+   eina_mempool_malloc(x.mp, sizeof(siz))
+# define EVAS_MEMPOOL_PREP(x, p, siz) \
+   do { \
+     x.count++; \
+     x.num_allocs++; \
+     memset(p, 0, sizeof(siz)); \
+   } while (0)
+# define EVAS_MEMPOOL_FREE(x, p) \
+   do { \
+     eina_mempool_free(x.mp, p); \
+     x.count--; \
+     x.num_frees++; \
+     if (x.count <= 0) { \
+       eina_mempool_del(x.mp); \
+       x.mp = NULL; \
+       x.count = 0; \
+     } \
+   } while (0)
+#else
+# define EVAS_MEMPOOL(x)
+# define EVAS_MEMPOOL_INIT(x, nam, siz, cnt, ret)
+# define EVAS_MEMPOOL_PREP(x, p, siz)
+# define EVAS_MEMPOOL_ALLOC(x, siz) \
+   calloc(1, sizeof(siz))
+# define EVAS_MEMPOOL_FREE(x, p) \
+   free(p)
+#endif  
+/********************/
+/*****************************************/
+/****************************************************************************/
+  
 #define EVAS_API_OVERRIDE(func, api, prefix) \
      (api)->func = prefix##func
 
