@@ -74,7 +74,6 @@ _output_setup(int w,
    evas_common_convert_init();
    evas_common_scale_init();
    evas_common_rectangle_init();
-   evas_common_gradient_init();
    evas_common_polygon_init();
    evas_common_line_init();
    evas_common_font_init();
@@ -348,14 +347,15 @@ module_open(Evas_Module *em)
    if (!em) return 0;
    /* get whatever engine module we inherit from */
    if (!_evas_module_engine_inherit(&pfunc, "software_generic")) return 0;
-   
-   _evas_engine_buffer_log_dom = eina_log_domain_register("EvasBufferEngine", EINA_COLOR_BLUE);
-   if(_evas_engine_buffer_log_dom < 0)
+
+   _evas_engine_buffer_log_dom = eina_log_domain_register
+     ("evas-buffer", EINA_COLOR_BLUE);
+   if (_evas_engine_buffer_log_dom < 0)
      {
-       EINA_LOG_ERR("Impossible to create a log domain for Eina.buffer.\n");
-       return 0;
+        EINA_LOG_ERR("Can not create a module log domain.");
+        return 0;
      }
-   
+
    /* store it for later use */
    func = pfunc;
    /* now to override methods */
@@ -380,7 +380,7 @@ module_open(Evas_Module *em)
 }
 
 static void
-module_close(Evas_Module *em)
+module_close(Evas_Module *em __UNUSED__)
 {
   eina_log_domain_unregister(_evas_engine_buffer_log_dom);
 }
