@@ -4,6 +4,7 @@
 #include <sys/time.h>
 #include <time.h>
 #include "evas_engine.h"
+#include "evas_common.h" /* Also includes international specific stuff */
 
 /* Uses Evas own image_draw primitive, for comparison purposes only. */
 //#define DFB_USE_EVAS_IMAGE_DRAW 1
@@ -354,7 +355,7 @@ _dfb_surface_set_blit_params(DirectFB_Engine_Image_Entry *d, DirectFB_Engine_Ima
      goto error;
 
 #ifdef DFB_DEBUG_FLAGS
-   DBG("sfunc=%s, dfunc=%s, color=%d %d %d %d\n\tblit=%s\n\tdraw=%s",
+   DBG("sfunc=%s, dfunc=%s, color=%d %d %d %d, blit=%s, draw=%s",
 	  _dfb_blend_func_str(src_func), _dfb_blend_func_str(dst_func),
 	  r, g, b, a,
 	  _dfb_blit_flags_str(blit_flags), _dfb_draw_flags_str(draw_flags));
@@ -631,15 +632,11 @@ evas_cache_image_dfb_debug(const char *context, Engine_Image_Entry* eie)
    DBG("*** %s image (%p) ***", context, eim);
    if (eim)
      {
-       DBG("* W: %d\n"
-		"* H: %d\n"
-		"* R: %d\n"
-		"* Key: %s\n"
-		"* DFB Surface: %p",
-		eie->w, eie->h, eie->references, eie->cache_key, eim->surface);
+       DBG("W: %d, H: %d, R: %d, Key: %s, DFB Surface: %p",
+           eie->w, eie->h, eie->references, eie->cache_key, eim->surface);
 
         if (eie->src)
-          DBG("* Pixels: %p", ((RGBA_Image*) eie->src)->image.data);
+          DBG("Pixels: %p", ((RGBA_Image*) eie->src)->image.data);
      }
    DBG("*** ***");
 }
@@ -737,7 +734,6 @@ _evas_common_init(void)
    evas_common_convert_init();
    evas_common_scale_init();
    evas_common_rectangle_init();
-   evas_common_gradient_init();
    evas_common_polygon_init();
    evas_common_line_init();
    evas_common_font_init();
@@ -1041,7 +1037,7 @@ evas_engine_dfb_output_idle_flush(void *data)
  * memory.
  */
 static void
-evas_engine_dfb_font_draw(void *data, void *context, void *surface, void *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const char *text)
+evas_engine_dfb_font_draw(void *data, void *context, void *surface, void *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props)
 {
    DirectFB_Engine_Image_Entry *eim = surface;
    IDirectFBSurface *screen;
@@ -1053,7 +1049,7 @@ evas_engine_dfb_font_draw(void *data, void *context, void *surface, void *font,
    if (!_dfb_lock_and_sync_image(screen, im, DSLF_READ | DSLF_WRITE))
      return;
 
-   evas_common_font_draw(im, context, font, x, y, text);
+   evas_common_font_draw(im, context, font, x, y, text, intl_props);
    evas_common_cpu_end_opt();
 
    im->image.data = NULL;
@@ -1167,29 +1163,6 @@ evas_engine_dfb_polygon_draw(void *data, void *context, void *surface, void *pol
 }
 #endif
 
-static void
-evas_engine_dfb_gradient_draw(void *data, void *context, void *surface, void *gradient, int x, int y, int w, int h)
-{
-   DirectFB_Engine_Image_Entry *eim = surface;
-   IDirectFBSurface *screen;
-   Render_Engine *re = data;
-   RGBA_Image *dst, *src;
-
-   dst = (RGBA_Image *)eim->cache_entry.src;
-   screen = eim->surface;
-   if (!_dfb_lock_and_sync_image(screen, dst, DSLF_READ | DSLF_WRITE))
-     return;
-
-   evas_common_gradient_draw(dst, context, x, y, w, h, gradient);
-   evas_common_cpu_end_opt();
-
-   dst->image.data = NULL;
-
-   screen->Unlock(screen);
-
-   return;
-}
-
 /** Image Object *******************************************************/
 static void *
 evas_engine_dfb_image_load(void *data, const char *file, const char *key, int *error, Evas_Image_Load_Opts *lo)
@@ -1693,11 +1666,12 @@ module_open(Evas_Module *em)
    if (!em) return 0;
    /* get whatever engine module we inherit from */
    if (!_evas_module_engine_inherit(&parent_func, "software_generic")) return 0;
-   _evas_engine_directfb_log_dom = eina_log_domain_register("EvasEngineDirectFB",EVAS_DEFAULT_LOG_COLOR);
-   if(_evas_engine_directfb_log_dom < 0)
+   _evas_engine_directfb_log_dom = eina_log_domain_register
+     ("evas-directfb", EVAS_DEFAULT_LOG_COLOR);
+   if (_evas_engine_directfb_log_dom < 0)
      {
-       EINA_LOG_ERR("Impossible to create a log domain for the DirectFb engine.\n");
-       return 0;
+        EINA_LOG_ERR("Can not create a module log domain.");
+        return 0;
      }
    /* store it for later use */
    func = parent_func;
@@ -1741,7 +1715,6 @@ module_open(Evas_Module *em)
    ORD(line_draw);
    ORD(rectangle_draw);
    ORD(polygon_draw);
-   ORD(gradient_draw);
    ORD(image_scale_hint_set);
    ORD(image_scale_hint_get);
 
