@@ -45,10 +45,51 @@
 #endif
 
 #ifndef GL_TEXTURE_RECTANGLE_NV
-#define GL_TEXTURE_RECTANGLE_NV 0x84F5
+# define GL_TEXTURE_RECTANGLE_NV 0x84F5
 #endif
 #ifndef GL_BGRA
-#define GL_BGRA 0x80E1
+# define GL_BGRA 0x80E1
+#endif
+
+#ifndef EGL_NO_CONTEXT
+# define EGL_NO_CONTEXT 0
+#endif
+#ifndef EGL_NONE
+# define EGL_NONE 0x3038
+#endif
+#ifndef EGL_TRUE
+# define EGL_TRUE 1
+#endif
+#ifndef EGL_FALSE
+# define EGL_FALSE 0
+#endif
+
+#ifndef EGL_MAP_GL_TEXTURE_2D_SEC
+# define EGL_MAP_GL_TEXTURE_2D_SEC 0x3201
+#endif
+#ifndef  EGL_MAP_GL_TEXTURE_HEIGHT_SEC
+# define EGL_MAP_GL_TEXTURE_HEIGHT_SEC 0x3202
+#endif
+#ifndef EGL_MAP_GL_TEXTURE_WIDTH_SEC
+# define EGL_MAP_GL_TEXTURE_WIDTH_SEC 0x3203
+#endif
+#ifndef EGL_MAP_GL_TEXTURE_FORMAT_SEC
+# define EGL_MAP_GL_TEXTURE_FORMAT_SEC 0x3204
+#endif
+#ifndef EGL_MAP_GL_TEXTURE_RGB_SEC
+# define EGL_MAP_GL_TEXTURE_RGB_SEC 0x3205
+#endif
+#ifndef EGL_MAP_GL_TEXTURE_RGBA_SEC
+# define EGL_MAP_GL_TEXTURE_RGBA_SEC 0x3206
+#endif
+#ifndef EGL_MAP_GL_TEXTURE_PIXEL_TYPE_SEC
+# define EGL_MAP_GL_TEXTURE_PIXEL_TYPE_SEC 0x3206
+#endif
+#ifndef EGL_MAP_GL_TEXTURE_UNSIGNED_BYTE_SEC
+# define EGL_MAP_GL_TEXTURE_UNSIGNED_BYTE_SEC 0x3207
+#endif
+#ifndef EGL_MAP_GL_TEXTURE_STRIDE_IN_BYTES_SEC
+# define EGL_MAP_GL_TEXTURE_STRIDE_IN_BYTES_SEC 0x3208
 #endif
 
 #define SHAD_VERTEX 0
@@ -67,9 +108,6 @@ typedef struct _Evas_GL_Image                        Evas_GL_Image;
 typedef struct _Evas_GL_Font_Texture                 Evas_GL_Font_Texture;
 typedef struct _Evas_GL_Polygon                      Evas_GL_Polygon;
 typedef struct _Evas_GL_Polygon_Point                Evas_GL_Polygon_Point;
-/*
-typedef struct _Evas_GL_Gradient                     Evas_GL_Gradient;
-*/
 
 struct _Evas_GL_Program
 {
@@ -90,11 +128,56 @@ struct _Evas_GL_Shared
    struct {
       GLint max_texture_units;
       GLint max_texture_size;
+      GLint max_vertex_elements;
       GLfloat anisotropic;
       Eina_Bool rgb : 1;
       Eina_Bool bgra : 1;
       Eina_Bool tex_npo2 : 1;
       Eina_Bool tex_rect : 1;
+      Eina_Bool sec_image_map : 1;
+      // tuning params - per gpu/cpu combo?
+#define MAX_CUTOUT             512
+#define DEF_CUTOUT                  512
+      
+// FIXME bug with pipes > 1 right now, should default to 32      
+#define MAX_PIPES              128
+#define DEF_PIPES                    32
+#define DEF_PIPES_SGX_540            32
+#define DEF_PIPES_TEGRA_2             1
+      
+#define MIN_ATLAS_ALLOC         16
+#define MAX_ATLAS_ALLOC       1024
+#define DEF_ATLAS_ALLOC            1024
+      
+#define MIN_ATLAS_ALLOC_ALPHA   16
+#define MAX_ATLAS_ALLOC_ALPHA 4096
+#define DEF_ATLAS_ALLOC_ALPHA      4096
+      
+#define MAX_ATLAS_W            512
+#define DEF_ATLAS_W                 512
+      
+#define MAX_ATLAS_H            512
+#define DEF_ATLAS_H                 512
+      
+#define MIN_ATLAS_SLOT          16
+#define MAX_ATLAS_SLOT         512
+#define DEF_ATLAS_SLOT               16
+      
+      struct {
+         struct {
+            int max;
+         } cutout;
+         struct {
+            int max;
+         } pipes;
+         struct {
+            int max_alloc_size;
+            int max_alloc_alpha_size;
+            int max_w;
+            int max_h;
+            int slot_size;
+         } atlas;
+      } tune;
    } info;
    
    struct {
@@ -102,6 +185,8 @@ struct _Evas_GL_Shared
       Eina_List       *atlas[33][3];
    } tex;
    
+   Eina_Hash          *native_hash;
+   
    struct {
       Evas_GL_Program  rect;
       Evas_GL_Program  font;
@@ -114,60 +199,92 @@ struct _Evas_GL_Shared
    int references;
    int w, h;
    int rot;
+   int mflip;
+   // persp map
+   int foc, z0, px, py;
+   int ax, ay;
 };
 
+#define RTYPE_RECT  1
+#define RTYPE_IMAGE 2
+#define RTYPE_FONT  3
+#define RTYPE_YUV   4
+#define RTYPE_MAP   5 /* need to merge with image */ 
+
+
+
 struct _Evas_GL_Context
 {
    int                references;
    int                w, h;
    int                rot;
-   RGBA_Draw_Context  *dc;
+   int                foc, z0, px, py;
+   RGBA_Draw_Context *dc;
    
    Evas_GL_Shared     *shared;
+
+   int flushnum;
    struct {
-      int             x, y, w, h;
-      Eina_Bool       active : 1;
-   } clip;
-   struct {
-      Evas_GL_Image  *surface;
-      GLuint          cur_prog;
-      GLuint          cur_tex, cur_texu, cur_texv;
-      int             render_op;
-      int             cx, cy, cw, ch;
-      Eina_Bool       smooth : 1;
-      Eina_Bool       blend : 1;
-      Eina_Bool       clip : 1;
+      int                top_pipe;
       struct {
          GLuint          cur_prog;
          GLuint          cur_tex, cur_texu, cur_texv;
          int             render_op;
          int             cx, cy, cw, ch;
-         Eina_Bool       smooth : 1;
-         Eina_Bool       blend : 1;
-         Eina_Bool       clip : 1;
+         int             smooth;
+         int             blend;
+         int             clip;
       } current;
-   } shader;
+   } state;
+   
    struct {
-      int num;
-      int alloc;
-      GLshort *vertex;
-      GLubyte *color;
-      GLfloat *texuv;
-      GLfloat *texuv2;
-      GLfloat *texuv3;
-      Eina_Bool line : 1;
-      Eina_Bool use_vertex : 1;
-      Eina_Bool use_color : 1;
-      Eina_Bool use_texuv : 1;
-      Eina_Bool use_texuv2 : 1;
-      Eina_Bool use_texuv3 : 1;
-      Evas_GL_Image *im;
-   } array;
+      struct {
+         int             x, y, w, h;
+         int             type;
+      } region;
+      struct {
+         int             x, y, w, h;
+         Eina_Bool       active : 1;
+      } clip;
+      struct {
+         Evas_GL_Image  *surface;
+         GLuint          cur_prog;
+         GLuint          cur_tex, cur_texu, cur_texv;
+         int             render_op;
+         int             cx, cy, cw, ch;
+         int             smooth;
+         int             blend;
+         int             clip;
+      } shader;
+      struct {
+         int num, alloc;
+         GLshort *vertex;
+         GLubyte *color;
+         GLfloat *texuv;
+         GLfloat *texuv2;
+         GLfloat *texuv3;
+         Eina_Bool line : 1;
+         Eina_Bool use_vertex : 1;
+         Eina_Bool use_color : 1;
+         Eina_Bool use_texuv : 1;
+         Eina_Bool use_texuv2 : 1;
+         Eina_Bool use_texuv3 : 1;
+         Evas_GL_Image *im;
+      } array;
+   } pipe[MAX_PIPES];
+   
    struct {
       Eina_Bool size : 1;
    } change;
    
+   Eina_Bool havestuff : 1;
+   
    Evas_GL_Image *def_surface;
+   
+#if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
+// FIXME: hack. expose egl display to gl core for egl image sec extn.   
+   void *egldisp;
+#endif   
 };
 
 struct _Evas_GL_Texture_Pool
@@ -178,10 +295,17 @@ struct _Evas_GL_Texture_Pool
    int              w, h;
    int              references;
    int              slot, fslot;
+   struct {
+      void         *img;
+      unsigned int *data;
+      int           w, h;
+      int           stride;
+   } dyn;
    Eina_List       *allocations;
    Eina_Bool        whole : 1;
    Eina_Bool        render : 1;
    Eina_Bool        native : 1;
+   Eina_Bool        dynamic : 1;
 };
 
 struct _Evas_GL_Texture
@@ -224,6 +348,8 @@ struct _Evas_GL_Image
       int           mipmap;
       unsigned char loose : 1;
    } native;
+
+   int scale_hint, content_hint;
    
    unsigned char    dirty : 1;
    unsigned char    cached : 1;
@@ -247,16 +373,6 @@ struct _Evas_GL_Polygon_Point
    int x, y;
 };
 
-/*
-struct _Evas_GL_Gradient
-{
-   RGBA_Gradient   *grad;
-   Evas_GL_Texture *tex;
-   int              tw, th;
-   unsigned char    changed : 1;
-};
-*/
-
 extern Evas_GL_Program_Source shader_rect_frag_src;
 extern Evas_GL_Program_Source shader_rect_vert_src;
 extern Evas_GL_Program_Source shader_font_frag_src;
@@ -320,7 +436,8 @@ void             evas_gl_common_context_image_map4_push(Evas_GL_Context *gc,
                                                         int clip, int cx, int cy, int cw, int ch,
                                                         int r, int g, int b, int a,
                                                         Eina_Bool smooth, 
-                                                        Eina_Bool tex_only);
+                                                        Eina_Bool tex_only,
+                                                        Eina_Bool yuv);
 void              evas_gl_common_context_flush(Evas_GL_Context *gc);
 
 int               evas_gl_common_shader_program_init(Evas_GL_Program *p,
@@ -331,28 +448,32 @@ void              evas_gl_common_shader_program_shutdown(Evas_GL_Program *p);
     
 void              evas_gl_common_rect_draw(Evas_GL_Context *gc, int x, int y, int w, int h);
 
+void              evas_gl_texture_pool_empty(Evas_GL_Texture_Pool *pt);
 Evas_GL_Texture  *evas_gl_common_texture_new(Evas_GL_Context *gc, RGBA_Image *im);
-Evas_GL_Texture  *evas_gl_common_texture_native_new(Evas_GL_Context *gc, int w, int h, int alpha, Evas_GL_Image *im);
-Evas_GL_Texture  *evas_gl_common_texture_render_new(Evas_GL_Context *gc, int w, int h, int alpha);
+Evas_GL_Texture  *evas_gl_common_texture_native_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, int alpha, Evas_GL_Image *im);
+Evas_GL_Texture  *evas_gl_common_texture_render_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, int alpha);
+Evas_GL_Texture  *evas_gl_common_texture_dynamic_new(Evas_GL_Context *gc, Evas_GL_Image *im);
 void              evas_gl_common_texture_update(Evas_GL_Texture *tex, RGBA_Image *im);
 void              evas_gl_common_texture_free(Evas_GL_Texture *tex);
-Evas_GL_Texture  *evas_gl_common_texture_alpha_new(Evas_GL_Context *gc, DATA8 *pixels, int w, int h, int fh);
-void              evas_gl_common_texture_alpha_update(Evas_GL_Texture *tex, DATA8 *pixels, int w, int h, int fh);
-Evas_GL_Texture  *evas_gl_common_texture_yuv_new(Evas_GL_Context *gc, DATA8 **rows, int w, int h);
-void              evas_gl_common_texture_yuv_update(Evas_GL_Texture *tex, DATA8 **rows, int w, int h);
+Evas_GL_Texture  *evas_gl_common_texture_alpha_new(Evas_GL_Context *gc, DATA8 *pixels, unsigned int w, unsigned int h, int fh);
+void              evas_gl_common_texture_alpha_update(Evas_GL_Texture *tex, DATA8 *pixels, unsigned int w, unsigned int h, int fh);
+Evas_GL_Texture  *evas_gl_common_texture_yuv_new(Evas_GL_Context *gc, DATA8 **rows, unsigned int w, unsigned int h);
+void              evas_gl_common_texture_yuv_update(Evas_GL_Texture *tex, DATA8 **rows, unsigned int w, unsigned int h);
 
 void              evas_gl_common_image_all_unload(Evas_GL_Context *gc);
 
 Evas_GL_Image    *evas_gl_common_image_load(Evas_GL_Context *gc, const char *file, const char *key, Evas_Image_Load_Opts *lo, int *error);
-Evas_GL_Image    *evas_gl_common_image_new_from_data(Evas_GL_Context *gc, int w, int h, DATA32 *data, int alpha, int cspace);
-Evas_GL_Image    *evas_gl_common_image_new_from_copied_data(Evas_GL_Context *gc, int w, int h, DATA32 *data, int alpha, int cspace);
-Evas_GL_Image    *evas_gl_common_image_new(Evas_GL_Context *gc, int w, int h, int alpha, int cspace);
+Evas_GL_Image    *evas_gl_common_image_new_from_data(Evas_GL_Context *gc, unsigned int w, unsigned int h, DATA32 *data, int alpha, int cspace);
+Evas_GL_Image    *evas_gl_common_image_new_from_copied_data(Evas_GL_Context *gc, unsigned int w, unsigned int h, DATA32 *data, int alpha, int cspace);
+Evas_GL_Image    *evas_gl_common_image_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, int alpha, int cspace);
 Evas_GL_Image    *evas_gl_common_image_alpha_set(Evas_GL_Image *im, int alpha);
 void              evas_gl_common_image_native_enable(Evas_GL_Image *im);
 void              evas_gl_common_image_native_disable(Evas_GL_Image *im);
+void              evas_gl_common_image_scale_hint_set(Evas_GL_Image *im, int hint);
+void              evas_gl_common_image_content_hint_set(Evas_GL_Image *im, int hint);
 void              evas_gl_common_image_free(Evas_GL_Image *im);
-Evas_GL_Image    *evas_gl_common_image_surface_new(Evas_GL_Context *gc, int w, int h, int alpha);
-void              evas_gl_common_image_dirty(Evas_GL_Image *im, int x, int y, int w, int h);
+Evas_GL_Image    *evas_gl_common_image_surface_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, int alpha);
+void              evas_gl_common_image_dirty(Evas_GL_Image *im, unsigned int x, unsigned int y, unsigned int w, unsigned int h);
 void              evas_gl_common_image_map4_draw(Evas_GL_Context *gc, Evas_GL_Image *im, RGBA_Map_Point *p, int smooth, int level);
 void              evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy, int sw, int sh, int dx, int dy, int dw, int dh, int smooth);
 
@@ -366,12 +487,21 @@ void              evas_gl_common_poly_draw(Evas_GL_Context *gc, Evas_GL_Polygon
 
 void              evas_gl_common_line_draw(Evas_GL_Context *gc, int x1, int y1, int x2, int y2);
 
-void (*glsym_glGenFramebuffers)      (GLsizei a, GLuint *b);
-void (*glsym_glBindFramebuffer)      (GLenum a, GLuint b);
-void (*glsym_glFramebufferTexture2D) (GLenum a, GLenum b, GLenum c, GLuint d, GLint e);
-void (*glsym_glDeleteFramebuffers)   (GLsizei a, const GLuint *b);
+extern void (*glsym_glGenFramebuffers)      (GLsizei a, GLuint *b);
+extern void (*glsym_glBindFramebuffer)      (GLenum a, GLuint b);
+extern void (*glsym_glFramebufferTexture2D) (GLenum a, GLenum b, GLenum c, GLuint d, GLint e);
+extern void (*glsym_glDeleteFramebuffers)   (GLsizei a, const GLuint *b);
 
-#define GL_ERRORS 1
+#if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
+extern void          *(*secsym_eglCreateImage)               (void *a, void *b, GLenum c, void *d, const int *e);
+extern unsigned int   (*secsym_eglDestroyImage)              (void *a, void *b);
+extern void           (*secsym_glEGLImageTargetTexture2DOES) (int a, void *b);
+extern void          *(*secsym_eglMapImageSEC)               (void *a, void *b);
+extern unsigned int   (*secsym_eglUnmapImageSEC)             (void *a, void *b);
+extern unsigned int   (*secsym_eglGetImageAttribSEC)         (void *a, void *b, int c, int *d);
+#endif
+
+//#define GL_ERRORS 1
 
 #ifdef GL_ERRORS
 # define GLERR(fn, fl, ln, op) \
@@ -383,4 +513,7 @@ void (*glsym_glDeleteFramebuffers)   (GLsizei a, const GLuint *b);
 # define GLERR(fn, fl, ln, op)
 #endif
 
+Eina_Bool evas_gl_common_module_open(void);
+void      evas_gl_common_module_close(void);
+
 #endif
