@@ -1,13 +1,33 @@
 #include "evas_gl_private.h"
 
+void
+evas_gl_common_image_all_unload(Evas_GL_Context *gc)
+{
+   Eina_List *l;
+   Evas_GL_Image *im;
+   
+   EINA_LIST_FOREACH(gc->shared->images, l, im)
+     {
+        if (im->im) evas_cache_image_unload_data(&im->im->cache_entry);
+        if (im->tex)
+          {
+             if (!im->tex->pt->dyn.img)
+               {
+                  evas_gl_common_texture_free(im->tex);
+                  im->tex = NULL;
+               }
+          }
+     }
+}
+
 Evas_GL_Image *
-evas_gl_common_image_load(Evas_GL_Context *gc, const char *file, const char *key, Evas_Image_Load_Opts *lo)
+evas_gl_common_image_load(Evas_GL_Context *gc, const char *file, const char *key, Evas_Image_Load_Opts *lo, int *error)
 {
    Evas_GL_Image        *im;
    RGBA_Image           *im_im;
    Eina_List            *l;
 
-   im_im = evas_common_load_image_from_file(file, key, lo);
+   im_im = evas_common_load_image_from_file(file, key, lo, error);
    if (!im_im) return NULL;
 
    EINA_LIST_FOREACH(gc->shared->images, l, im)
@@ -18,12 +38,17 @@ evas_gl_common_image_load(Evas_GL_Context *gc, const char *file, const char *key
 	     gc->shared->images = eina_list_remove_list(gc->shared->images, l);
 	     gc->shared->images = eina_list_prepend(gc->shared->images, im);
 	     im->references++;
+	     *error = EVAS_LOAD_ERROR_NONE;
 	     return im;
 	  }
      }
 
    im = calloc(1, sizeof(Evas_GL_Image));
-   if (!im) return NULL;
+   if (!im)
+     {
+	*error = EVAS_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED;
+	return NULL;
+     }
    im->references = 1;
    im->im = im_im;
    im->gc = gc;
@@ -39,22 +64,25 @@ evas_gl_common_image_load(Evas_GL_Context *gc, const char *file, const char *key
 }
 
 Evas_GL_Image *
-evas_gl_common_image_new_from_data(Evas_GL_Context *gc, int w, int h, DATA32 *data, int alpha, int cspace)
+evas_gl_common_image_new_from_data(Evas_GL_Context *gc, unsigned int w, unsigned int h, DATA32 *data, int alpha, int cspace)
 {
    Evas_GL_Image *im;
    Eina_List *l;
 
-   EINA_LIST_FOREACH(gc->shared->images, l, im)
+   if (data)
      {
-	if (((void *)(im->im->image.data) == (void *)data) &&
-	    (im->im->cache_entry.w == w) &&
-	    (im->im->cache_entry.h == h))
-	  {
-	     gc->shared->images = eina_list_remove_list(gc->shared->images, l);
-	     gc->shared->images = eina_list_prepend(gc->shared->images, im);
-	     im->references++;
-	     return im;
-	  }
+        EINA_LIST_FOREACH(gc->shared->images, l, im)
+          {
+             if (((void *)(im->im->image.data) == (void *)data) &&
+                 (im->im->cache_entry.w == w) &&
+                 (im->im->cache_entry.h == h))
+               {
+                  gc->shared->images = eina_list_remove_list(gc->shared->images, l);
+                  gc->shared->images = eina_list_prepend(gc->shared->images, im);
+                  im->references++;
+                  return im;
+               }
+          }
      }
    im = calloc(1, sizeof(Evas_GL_Image));
    if (!im) return NULL;
@@ -86,15 +114,11 @@ evas_gl_common_image_new_from_data(Evas_GL_Context *gc, int w, int h, DATA32 *da
 	abort();
 	break;
      }
-   /*
-    im->cached = 1;
-    gc->shared->images = eina_list_prepend(gc->shared->images, im);
-    */
    return im;
 }
 
 Evas_GL_Image *
-evas_gl_common_image_new_from_copied_data(Evas_GL_Context *gc, int w, int h, DATA32 *data, int alpha, int cspace)
+evas_gl_common_image_new_from_copied_data(Evas_GL_Context *gc, unsigned int w, unsigned int h, DATA32 *data, int alpha, int cspace)
 {
    Evas_GL_Image *im;
 
@@ -122,7 +146,8 @@ evas_gl_common_image_new_from_copied_data(Evas_GL_Context *gc, int w, int h, DAT
         if (im->tex) evas_gl_common_texture_free(im->tex);
         im->tex = NULL;
 	im->cs.no_free = 0;
-        im->cs.data = calloc(1, im->im->cache_entry.h * sizeof(unsigned char *) * 2);
+        if (im->im->cache_entry.h > 0)
+          im->cs.data = calloc(1, im->im->cache_entry.h * sizeof(unsigned char *) * 2);
         if ((data) && (im->cs.data))
 	  memcpy(im->cs.data, data, im->im->cache_entry.h * sizeof(unsigned char *) * 2);
 	break;
@@ -134,14 +159,14 @@ evas_gl_common_image_new_from_copied_data(Evas_GL_Context *gc, int w, int h, DAT
 }
 
 Evas_GL_Image *
-evas_gl_common_image_new(Evas_GL_Context *gc, int w, int h, int alpha, int cspace)
+evas_gl_common_image_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, int alpha, int cspace)
 {
    Evas_GL_Image *im;
 
    im = calloc(1, sizeof(Evas_GL_Image));
    if (!im) return NULL;
    im->references = 1;
-   im->im = (RGBA_Image *) evas_cache_image_empty(evas_common_image_cache_get());
+   im->im = (RGBA_Image *)evas_cache_image_empty(evas_common_image_cache_get());
    if (!im->im)
      {
 	free(im);
@@ -151,10 +176,12 @@ evas_gl_common_image_new(Evas_GL_Context *gc, int w, int h, int alpha, int cspac
    im->im->cache_entry.flags.alpha = alpha ? 1 : 0;
    im->cs.space = cspace;
    im->alpha = im->im->cache_entry.flags.alpha;
+   im->im->cache_entry.w = w;
+   im->im->cache_entry.h = h;
    im->w = im->im->cache_entry.w;
    im->h = im->im->cache_entry.h;
    evas_cache_image_colorspace(&im->im->cache_entry, cspace);
-   im->im = (RGBA_Image *) evas_cache_image_size_set(&im->im->cache_entry, w, h);
+   im->im = (RGBA_Image *)evas_cache_image_size_set(&im->im->cache_entry, w, h);
    switch (cspace)
      {
       case EVAS_COLORSPACE_ARGB8888:
@@ -164,7 +191,8 @@ evas_gl_common_image_new(Evas_GL_Context *gc, int w, int h, int alpha, int cspac
 //        if (im->tex) evas_gl_common_texture_free(im->tex);
 	im->tex = NULL;
 	im->cs.no_free = 0;
-        im->cs.data = calloc(1, im->im->cache_entry.h * sizeof(unsigned char *) * 2);
+        if (im->im->cache_entry.h > 0)
+          im->cs.data = calloc(1, im->im->cache_entry.h * sizeof(unsigned char *) * 2);
 	break;
       default:
 	abort();
@@ -173,12 +201,156 @@ evas_gl_common_image_new(Evas_GL_Context *gc, int w, int h, int alpha, int cspac
    return im;
 }
 
+Evas_GL_Image *
+evas_gl_common_image_alpha_set(Evas_GL_Image *im, int alpha)
+{
+   if (!im) return NULL;
+   if (im->alpha == alpha) return im;
+   im->alpha = alpha;
+   if (!im->im) return im;
+   im->im->cache_entry.flags.alpha = alpha ? 1 : 0;
+   if (im->tex)
+     {
+        evas_gl_common_texture_free(im->tex);
+        im->tex = NULL;
+     }
+   if (!im->tex)
+     im->tex = evas_gl_common_texture_new(im->gc, im->im);
+   return im;
+}
+
+void
+evas_gl_common_image_native_enable(Evas_GL_Image *im)
+{
+   if (im->cs.data)
+     {
+	if (!im->cs.no_free) free(im->cs.data);
+        im->cs.data = NULL;
+     }
+   im->cs.no_free = 0;
+   if (im->cached)
+     {
+        im->gc->shared->images = eina_list_remove(im->gc->shared->images, im);
+        im->cached = 0;
+     }
+   if (im->im)
+     {
+        evas_cache_image_drop(&im->im->cache_entry);
+        im->im = NULL;
+     }
+   if (im->tex)
+     {
+        evas_gl_common_texture_free(im->tex);
+        im->tex = NULL;
+     }
+   
+   im->cs.space = EVAS_COLORSPACE_ARGB8888;
+   im->tex = evas_gl_common_texture_native_new(im->gc, im->w, im->h, im->alpha, im);
+   im->tex_only = 1;
+}
+
+void
+evas_gl_common_image_native_disable(Evas_GL_Image *im)
+{
+   if (im->im)
+     {
+        evas_cache_image_drop(&im->im->cache_entry);
+        im->im = NULL;
+     }
+   if (im->tex)
+     {
+        evas_gl_common_texture_free(im->tex);
+        im->tex = NULL;
+     }
+   im->tex_only = 0;
+   
+   im->im = (RGBA_Image *)evas_cache_image_empty(evas_common_image_cache_get());
+   im->im->cache_entry.flags.alpha = im->alpha;
+   im->cs.space = EVAS_COLORSPACE_ARGB8888;
+   evas_cache_image_colorspace(&im->im->cache_entry, im->cs.space);
+   im->im = (RGBA_Image *)evas_cache_image_size_set(&im->im->cache_entry, im->w, im->h);
+   if (!im->tex)
+     im->tex = evas_gl_common_texture_new(im->gc, im->im);
+}
+
+void
+evas_gl_common_image_scale_hint_set(Evas_GL_Image *im, int hint)
+{
+   im->scale_hint = hint;
+   // FIXME: take advantage of this even in gl (eg if image is
+   // 1600x1200 but we always use it at 800x600 or even less - drop
+   // the texture res down for "non dynamic" stuff to save memory)
+}
+
+void
+evas_gl_common_image_content_hint_set(Evas_GL_Image *im, int hint)
+{
+   if (im->content_hint == hint) return;
+   im->content_hint = hint;
+   if (!im->gc) return;
+   if (!im->gc->shared->info.sec_image_map) return;
+   if (!im->gc->shared->info.bgra) return;
+   // does not handle yuv yet.
+   if (im->cs.space != EVAS_COLORSPACE_ARGB8888) return;
+   if (im->content_hint == EVAS_IMAGE_CONTENT_HINT_DYNAMIC)
+     {
+        if (im->cs.data)
+          {
+             if (!im->cs.no_free) free(im->cs.data);
+             im->cs.data = NULL;
+          }
+        im->cs.no_free = 0;
+        if (im->cached)
+          {
+             im->gc->shared->images = eina_list_remove(im->gc->shared->images, im);
+             im->cached = 0;
+          }
+        if (im->im)
+          {
+             evas_cache_image_drop(&im->im->cache_entry);
+             im->im = NULL;
+          }
+        if (im->tex)
+          {
+             evas_gl_common_texture_free(im->tex);
+             im->tex = NULL;
+          }
+        im->tex = evas_gl_common_texture_dynamic_new(im->gc, im);
+        im->tex_only = 1;
+     }
+   else
+     {
+        if (im->im)
+          {
+             evas_cache_image_drop(&im->im->cache_entry);
+             im->im = NULL;
+          }
+        if (im->tex)
+          {
+             evas_gl_common_texture_free(im->tex);
+             im->tex = NULL;
+          }
+        im->tex_only = 0;
+        
+        im->im = (RGBA_Image *)evas_cache_image_empty(evas_common_image_cache_get());
+        im->im->cache_entry.flags.alpha = im->alpha;
+        im->cs.space = EVAS_COLORSPACE_ARGB8888;
+        evas_cache_image_colorspace(&im->im->cache_entry, im->cs.space);
+        im->im = (RGBA_Image *)evas_cache_image_size_set(&im->im->cache_entry, im->w, im->h);
+        if (!im->tex)
+           im->tex = evas_gl_common_texture_new(im->gc, im->im);
+     }
+}
+
 void
 evas_gl_common_image_free(Evas_GL_Image *im)
 {
    im->references--;
    if (im->references > 0) return;
-
+   
+   if (im->native.func.free)
+     im->native.func.free(im->native.func.data, im);
+   
    if (im->cs.data)
      {
 	if (!im->cs.no_free) free(im->cs.data);
@@ -190,7 +362,7 @@ evas_gl_common_image_free(Evas_GL_Image *im)
 }
 
 Evas_GL_Image *
-evas_gl_common_image_surface_new(Evas_GL_Context *gc, int w, int h, int alpha)
+evas_gl_common_image_surface_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, int alpha)
 {
    Evas_GL_Image *im;
 
@@ -208,11 +380,16 @@ evas_gl_common_image_surface_new(Evas_GL_Context *gc, int w, int h, int alpha)
 }
 
 void
-evas_gl_common_image_dirty(Evas_GL_Image *im)
+evas_gl_common_image_dirty(Evas_GL_Image *im, unsigned int x, unsigned int y, unsigned int w, unsigned int h)
 {
+   if ((w == 0) && (h == 0) && (x == 0) && (y == 0))
+     {
+        w = im->w;
+        h = im->h;
+     }
    if (im->im)
      {
-        im->im = (RGBA_Image *) evas_cache_image_dirty(&im->im->cache_entry, 0, 0, im->im->cache_entry.w, im->im->cache_entry.h);
+        im->im = (RGBA_Image *)evas_cache_image_dirty(&im->im->cache_entry, x, y, w, h);
      }
    im->dirty = 1;
 }
@@ -245,14 +422,19 @@ _evas_gl_common_image_update(Evas_GL_Context *gc, Evas_GL_Image *im)
    switch (im->cs.space)
      {
       case EVAS_COLORSPACE_ARGB8888:
-        evas_cache_image_load_data(&im->im->cache_entry);
 	if ((im->tex) && (im->dirty))
-	  {
-	     evas_gl_common_texture_update(im->tex, im->im);
-	     im->dirty = 0;
-	  }
+          {
+             evas_cache_image_load_data(&im->im->cache_entry);
+             evas_gl_common_texture_update(im->tex, im->im);
+             evas_cache_image_unload_data(&im->im->cache_entry);
+          }
 	if (!im->tex)
-	  im->tex = evas_gl_common_texture_new(gc, im->im);
+          {
+             evas_cache_image_load_data(&im->im->cache_entry);
+             im->tex = evas_gl_common_texture_new(gc, im->im);
+             evas_cache_image_unload_data(&im->im->cache_entry);
+          }
+        im->dirty = 0;
         if (!im->tex) return;
 	break;
       case EVAS_COLORSPACE_YCBCR422P601_PL:
@@ -274,18 +456,19 @@ _evas_gl_common_image_update(Evas_GL_Context *gc, Evas_GL_Image *im)
         if (!im->tex) return;
         break;
       default:
-        printf("unhandled img format\n");
+         ERR("unhandled img format colorspace=%d", im->cs.space);
 	break;
     }
 }
 
 void
 evas_gl_common_image_map4_draw(Evas_GL_Context *gc, Evas_GL_Image *im, 
-                               RGBA_Map_Point *p, int smooth, int level)
+                               RGBA_Map_Point *p, int smooth, int level __UNUSED__)
 {
    RGBA_Draw_Context *dc;
    int r, g, b, a;
    int c, cx, cy, cw, ch;
+   Eina_Bool yuv = 0;
    
    dc = gc->dc;
    if (dc->mul.use)
@@ -302,16 +485,17 @@ evas_gl_common_image_map4_draw(Evas_GL_Context *gc, Evas_GL_Image *im,
    
    _evas_gl_common_image_update(gc, im);
 
-   // FIXME: need to handle clipping with a scissor. only sane way to do it.
-
-   glFlush();
-   
    c = gc->dc->clip.use; 
    cx = gc->dc->clip.x; cy = gc->dc->clip.y; 
    cw = gc->dc->clip.w; ch = gc->dc->clip.h;
+   im->tex->im = im;
+   if ((im->cs.space == EVAS_COLORSPACE_YCBCR422P601_PL) ||
+       (im->cs.space == EVAS_COLORSPACE_YCBCR422P709_PL))
+      yuv = 1;
    evas_gl_common_context_image_map4_push(gc, im->tex, p, 
                                           c, cx, cy, cw, ch, 
-                                          r, g, b, a, smooth, im->tex_only);
+                                          r, g, b, a, smooth, im->tex_only,
+                                          yuv);
 }
 
 void
@@ -342,14 +526,20 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
      }
    
    _evas_gl_common_image_update(gc, im);
+   if (!im->tex)
+     {
+        evas_gl_common_rect_draw(gc, dx, dy, dw, dh);
+        return;
+     }
 
    if ((im->cs.space == EVAS_COLORSPACE_YCBCR422P601_PL) ||
        (im->cs.space == EVAS_COLORSPACE_YCBCR422P709_PL))
      yuv = 1;
    
-   if ((!gc->dc->cutout.rects)
-//       || (gc->dc->cutout.active > 32)
-       )
+   im->tex->im = im;
+   if ((!gc->dc->cutout.rects) || 
+       ((gc->shared->info.tune.cutout.max > 0) &&
+           (gc->dc->cutout.active > gc->shared->info.tune.cutout.max)))
      {
         if (gc->dc->clip.use)
           {
@@ -375,7 +565,7 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
                                                       sx, sy, sw, sh,
                                                       dx, dy, dw, dh,
                                                       r, g, b, a,
-                                                      smooth);
+                                                      smooth, im->tex_only);
                   return;
                }
              
@@ -396,7 +586,7 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
                                                  ssx, ssy, ssw, ssh,
                                                  nx, ny, nw, nh,
                                                  r, g, b, a,
-                                                 smooth);
+                                                 smooth, im->tex_only);
           }
         else
           {
@@ -413,7 +603,7 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
                                                  sx, sy, sw, sh,
                                                  dx, dy, dw, dh,
                                                  r, g, b, a,
-                                                 smooth);
+                                                 smooth, im->tex_only);
           }
         return;
      }
@@ -452,7 +642,7 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
                                                  sx, sy, sw, sh,
                                                  dx, dy, dw, dh,
                                                  r, g, b, a,
-                                                 smooth);
+                                                 smooth, im->tex_only);
              continue;
           }
         ssx = (double)sx + ((double)(sw * (nx - dx)) / (double)(dw));
@@ -472,7 +662,7 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
                                             ssx, ssy, ssw, ssh,
                                             nx, ny, nw, nh,
                                             r, g, b, a,
-                                            smooth);
+                                            smooth, im->tex_only);
      }
    evas_common_draw_context_apply_clear_cutouts(rects);
    /* restore clip info */
