@@ -406,7 +406,21 @@ gl_compile_link_error(GLuint target, const char *action)
 {
    int loglen = 0, chars = 0;
    char *logtxt;
+
+   /* Shader info log */
+   glGetShaderiv(target, GL_INFO_LOG_LENGTH, &loglen);
+   if (loglen > 0)
+     {
+        logtxt = calloc(loglen, sizeof(char));
+        if (logtxt)
+          {
+             glGetShaderInfoLog(target, loglen, &chars, logtxt);
+             ERR("Failed to %s: %s", action, logtxt);
+             free(logtxt);
+          }
+     }
    
+   /* Program info log */
    glGetProgramiv(target, GL_INFO_LOG_LENGTH, &loglen);
    if (loglen > 0)
      {
@@ -414,7 +428,7 @@ gl_compile_link_error(GLuint target, const char *action)
         if (logtxt)
           {
              glGetProgramInfoLog(target, loglen, &chars, logtxt);
-             printf("Failed to %s: %s\n", action, logtxt);
+             ERR("Failed to %s: %s", action, logtxt);
              free(logtxt);
           }
      }
@@ -441,12 +455,13 @@ evas_gl_common_shader_program_init(Evas_GL_Program *p,
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    glCompileShader(p->vert);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+   ok = 0;
    glGetShaderiv(p->vert, GL_COMPILE_STATUS, &ok);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    if (!ok)
      {
         gl_compile_link_error(p->vert, "compile vertex shader");
-        printf("Abort compile of shader vert (%s):\n%s\n", name, vert->src);
+        ERR("Abort compile of shader vert (%s): %s", name, vert->src);
         return 0;
      }
    glShaderSource(p->frag, 1,
@@ -454,12 +469,13 @@ evas_gl_common_shader_program_init(Evas_GL_Program *p,
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    glCompileShader(p->frag);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+   ok = 0;
    glGetShaderiv(p->frag, GL_COMPILE_STATUS, &ok);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    if (!ok)
      {
         gl_compile_link_error(p->frag, "compile fragment shader");
-        printf("Abort compile of shader frag (%s):\n%s\n", name, frag->src);
+        ERR("Abort compile of shader frag (%s): %s", name, frag->src);
         return 0;
      }
 #endif
@@ -482,13 +498,14 @@ evas_gl_common_shader_program_init(Evas_GL_Program *p,
    
    glLinkProgram(p->prog);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+   ok = 0;
    glGetProgramiv(p->prog, GL_LINK_STATUS, &ok);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    if (!ok)
      {
         gl_compile_link_error(p->prog, "link fragment and vertex shaders");
-        printf("Abort compile of shader frag (%s):\n%s\n", name, frag->src);
-        printf("Abort compile of shader vert (%s):\n%s\n", name, vert->src);
+        ERR("Abort compile of shader frag (%s): %s", name, frag->src);
+        ERR("Abort compile of shader vert (%s): %s", name, vert->src);
         return 0;
      }
    return 1;
