@@ -10,16 +10,46 @@ static const GLenum rgb_fmt    = GL_RGBA;
 static const GLenum rgb_ifmt   = GL_RGB;
 //#endif
 #ifdef GL_BGRA
+# if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
+static const GLenum bgra_fmt   = GL_BGRA;
+static const GLenum bgra_ifmt  = GL_BGRA;
+static const GLenum bgr_fmt    = GL_BGRA;
+static const GLenum bgr_ifmt   = GL_BGRA;
+# else
 static const GLenum bgra_fmt   = GL_BGRA;
 static const GLenum bgra_ifmt  = GL_RGBA;
 static const GLenum bgr_fmt    = GL_BGRA;
 static const GLenum bgr_ifmt   = GL_RGB;
+# endif
 #endif
 static const GLenum alpha_fmt  = GL_ALPHA;
 static const GLenum alpha_ifmt = GL_ALPHA;
 static const GLenum lum_fmt    = GL_LUMINANCE;
 static const GLenum lum_ifmt   = GL_LUMINANCE;
 
+static struct {
+   struct {
+      int num, pix;
+   } c, a, v, r, n, d;
+} texinfo = {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}};
+
+static void
+_print_tex_count(void)
+{
+   if (getenv("EVAS_GL_MEMINFO"))
+     {
+        fprintf(stderr,
+                "T: c:%i/%ik | a:%i/%ik | v:%i/%ik | r:%i/%ik | n:%i/%ik | d:%i/%ik\n",
+                texinfo.c.num, (texinfo.c.pix * 4) / 1024,
+                texinfo.a.num, (texinfo.a.pix    ) / 1024,
+                texinfo.v.num, (texinfo.v.pix    ) / 1024,
+                texinfo.r.num, (texinfo.r.pix * 4) / 1024,
+                texinfo.n.num, (texinfo.n.pix * 4) / 1024,
+                texinfo.d.num, (texinfo.d.pix * 4) / 1024
+               );
+     }
+}
+
 static int
 _nearest_pow2(int num)
 {
@@ -35,8 +65,6 @@ _nearest_pow2(int num)
 static void
 _tex_adjust(Evas_GL_Context *gc, int *w, int *h)
 {
-   unsigned int n;
-   
    if (gc->shared->info.tex_npo2) return;
    /*if (gc->shared->info.tex_rect) return;*/
    *w = _nearest_pow2(*w);
@@ -48,7 +76,8 @@ _tex_round_slot(Evas_GL_Context *gc, int h)
 {
    if (!gc->shared->info.tex_npo2)
      h = _nearest_pow2(h);
-   return (h + 15) >> 4;
+   return (h + gc->shared->info.tune.atlas.slot_size - 1) /
+      gc->shared->info.tune.atlas.slot_size;
 }
 
 static int
@@ -65,7 +94,7 @@ _tex_format_index(GLuint format)
         return 1;
      case GL_ALPHA:
         return 2;
-     case GL_LUMINANCE:
+     case GL_LUMINANCE: // never used in atlas
         return 3;
      default:
         return 0;
@@ -88,13 +117,13 @@ _tex_sub_2d(int x, int y, int w, int h, int fmt, int type, const void *pix)
 }
 
 static Evas_GL_Texture_Pool *
-_pool_tex_new(Evas_GL_Context *gc, int w, int h, int intformat, int format)
+_pool_tex_new(Evas_GL_Context *gc, int w, int h, int intformat, GLenum format)
 {
    Evas_GL_Texture_Pool *pt;
    
    pt = calloc(1, sizeof(Evas_GL_Texture_Pool));
    if (!pt) return NULL;
-   h = _tex_round_slot(gc, h) << 4;
+   h = _tex_round_slot(gc, h) * gc->shared->info.tune.atlas.slot_size;
    _tex_adjust(gc, &w, &h);
    pt->gc = gc;
    pt->w = w;
@@ -103,6 +132,25 @@ _pool_tex_new(Evas_GL_Context *gc, int w, int h, int intformat, int format)
    pt->format = format;
    pt->dataformat = GL_UNSIGNED_BYTE;
    pt->references = 0;
+   
+   if (format == alpha_fmt)
+      {
+         texinfo.a.num++;
+         texinfo.a.pix += pt->w * pt->h;
+      }
+   else if (format == lum_fmt)
+      {
+         texinfo.v.num++;
+         texinfo.v.pix += pt->w * pt->h;
+      }
+   else
+      {
+         texinfo.c.num++;
+         texinfo.c.pix += pt->w * pt->h;
+      }
+
+   _print_tex_count();
+   
    glGenTextures(1, &(pt->texture));
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    glBindTexture(GL_TEXTURE_2D, pt->texture);
@@ -116,13 +164,13 @@ _pool_tex_new(Evas_GL_Context *gc, int w, int h, int intformat, int format)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    _tex_2d(pt->intformat, w, h, pt->format, pt->dataformat);
-   glBindTexture(GL_TEXTURE_2D, gc->shader.cur_tex);
+   glBindTexture(GL_TEXTURE_2D, gc->pipe[0].shader.cur_tex);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    return pt;
 }
 
 static int
-_pool_tex_alloc(Evas_GL_Texture_Pool *pt, int w, int h, int *u, int *v, Eina_List **l_after)
+_pool_tex_alloc(Evas_GL_Texture_Pool *pt, int w, int h __UNUSED__, int *u, int *v, Eina_List **l_after)
 {
    Eina_List *l;
    Evas_GL_Texture *tex, *tex2;
@@ -174,8 +222,11 @@ _pool_tex_find(Evas_GL_Context *gc, int w, int h,
    Evas_GL_Texture_Pool *pt = NULL;
    Eina_List *l;
    int th, th2;
-   
-   if ((w > 512) || (h > 512))
+
+   if (atlas_w > gc->shared->info.max_texture_size)
+      atlas_w = gc->shared->info.max_texture_size;
+   if ((w > gc->shared->info.tune.atlas.max_w) || 
+       (h > gc->shared->info.tune.atlas.max_h))
      {
         pt = _pool_tex_new(gc, w, h, intformat, format);
         gc->shared->tex.whole = eina_list_prepend(gc->shared->tex.whole, pt);
@@ -228,35 +279,39 @@ evas_gl_common_texture_new(Evas_GL_Context *gc, RGBA_Image *im)
    if (im->cache_entry.flags.alpha)
      {
         if (gc->shared->info.bgra)
-          tex->pt = _pool_tex_find(gc, im->cache_entry.w + 2,
-                                   im->cache_entry.h + 1, bgra_ifmt, bgra_fmt, 
-                                   &u, &v, &l_after, 1024);
+           tex->pt = _pool_tex_find(gc, im->cache_entry.w + 2,
+                                    im->cache_entry.h + 1, bgra_ifmt, bgra_fmt, 
+                                    &u, &v, &l_after,
+                                    gc->shared->info.tune.atlas.max_alloc_size);
         else
-          tex->pt = _pool_tex_find(gc, im->cache_entry.w + 2,
-                                   im->cache_entry.h + 1, rgba_ifmt, rgba_fmt, 
-                                   &u, &v, &l_after, 1024);
+           tex->pt = _pool_tex_find(gc, im->cache_entry.w + 2,
+                                    im->cache_entry.h + 1, rgba_ifmt, rgba_fmt, 
+                                    &u, &v, &l_after,
+                                    gc->shared->info.tune.atlas.max_alloc_size);
         tex->alpha = 1;
      }
    else
      {
         if (gc->shared->info.bgra)
-          tex->pt = _pool_tex_find(gc, im->cache_entry.w + 3, 
-                                 im->cache_entry.h + 1, bgr_ifmt, bgr_fmt,
-                                 &u, &v, &l_after, 1024);
+           tex->pt = _pool_tex_find(gc, im->cache_entry.w + 3, 
+                                    im->cache_entry.h + 1, bgr_ifmt, bgr_fmt,
+                                    &u, &v, &l_after,
+                                    gc->shared->info.tune.atlas.max_alloc_size);
         else
 #if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
-          tex->pt = _pool_tex_find(gc, im->cache_entry.w + 3, 
-                                 im->cache_entry.h + 1, rgba_ifmt, rgba_fmt,
-                                 &u, &v, &l_after, 1024);
+           tex->pt = _pool_tex_find(gc, im->cache_entry.w + 3, 
+                                    im->cache_entry.h + 1, rgba_ifmt, rgba_fmt,
+                                    &u, &v, &l_after,
+                                    gc->shared->info.tune.atlas.max_alloc_size);
 #else
-          tex->pt = _pool_tex_find(gc, im->cache_entry.w + 3, 
-                                 im->cache_entry.h + 1, rgb_ifmt, rgb_fmt,
-                                 &u, &v, &l_after, 1024);
+           tex->pt = _pool_tex_find(gc, im->cache_entry.w + 3, 
+                                    im->cache_entry.h + 1, rgb_ifmt, rgb_fmt,
+                                    &u, &v, &l_after,
+                                    gc->shared->info.tune.atlas.max_alloc_size);
 #endif
      }
    if (!tex->pt)
      {
-        memset(tex, 0x11, sizeof(Evas_GL_Texture)); // mark as freed
         free(tex);
         return NULL;
      }
@@ -282,7 +337,7 @@ _pool_tex_render_new(Evas_GL_Context *gc, int w, int h, int intformat, int forma
    
    pt = calloc(1, sizeof(Evas_GL_Texture_Pool));
    if (!pt) return NULL;
-   h = _tex_round_slot(gc, h) << 4;
+   h = _tex_round_slot(gc, h) * gc->shared->info.tune.atlas.slot_size;
    _tex_adjust(gc, &w, &h);
    pt->gc = gc;
    pt->w = w;
@@ -307,6 +362,11 @@ _pool_tex_render_new(Evas_GL_Context *gc, int w, int h, int intformat, int forma
 #  define GL_COLOR_ATTACHMENT0 GL_COLOR_ATTACHMENT0_EXT
 # endif
 #endif  
+   texinfo.r.num++;
+   texinfo.r.pix += pt->w * pt->h;
+   
+   _print_tex_count();
+   
    glGenTextures(1, &(pt->texture));
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    glBindTexture(GL_TEXTURE_2D, pt->texture);
@@ -330,7 +390,7 @@ _pool_tex_render_new(Evas_GL_Context *gc, int w, int h, int intformat, int forma
    glsym_glBindFramebuffer(GL_FRAMEBUFFER, 0);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    
-   glBindTexture(GL_TEXTURE_2D, gc->shader.cur_tex);
+   glBindTexture(GL_TEXTURE_2D, gc->pipe[0].shader.cur_tex);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    return pt;
 }
@@ -347,6 +407,8 @@ _pool_tex_native_new(Evas_GL_Context *gc, int w, int h, int intformat, int forma
    if (im->native.target == GL_TEXTURE_RECTANGLE_ARB)
      {
         printf("REEEEEEEEECT\n");
+        pt->w = w;
+        pt->h = h;
      }
    else
 #endif     
@@ -360,6 +422,11 @@ _pool_tex_native_new(Evas_GL_Context *gc, int w, int h, int intformat, int forma
    pt->dataformat = GL_UNSIGNED_BYTE;
    pt->references = 0;
    pt->native = 1;
+   texinfo.n.num++;
+   texinfo.n.pix += pt->w * pt->h;
+   
+   _print_tex_count();
+   
    glGenTextures(1, &(pt->texture));
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    glBindTexture(im->native.target, pt->texture);
@@ -384,24 +451,212 @@ _pool_tex_native_new(Evas_GL_Context *gc, int w, int h, int intformat, int forma
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    glBindTexture(im->native.target, 0);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
-   glBindTexture(im->native.target, gc->shader.cur_tex);
+   glBindTexture(im->native.target, gc->pipe[0].shader.cur_tex);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    return pt;
 }
 
-static void
-pt_unref(Evas_GL_Texture_Pool *pt)
+static Evas_GL_Texture_Pool *
+_pool_tex_dynamic_new(Evas_GL_Context *gc, int w, int h, int intformat, int format)
 {
-   pt->references--;
-   if (pt->references != 0) return;
-   if (!((pt->render) || (pt->native)))
+   Evas_GL_Texture_Pool *pt = NULL;
+   
+#if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
+   int fmt; // EGL_MAP_GL_TEXTURE_RGBA_SEC or EGL_MAP_GL_TEXTURE_RGB_SEC or bust
+   int pixtype; // EGL_MAP_GL_TEXTURE_UNSIGNED_BYTE_SEC or bust
+   int attr[] =
      {
-        if (pt->whole)
-          pt->gc->shared->tex.whole = eina_list_remove(pt->gc->shared->tex.whole, pt);
-        else
-          pt->gc->shared->tex.atlas [pt->slot][pt->fslot] =
-          eina_list_remove(pt->gc->shared->tex.atlas[pt->slot][pt->fslot], pt);
+        EGL_MAP_GL_TEXTURE_WIDTH_SEC, 32,
+        EGL_MAP_GL_TEXTURE_HEIGHT_SEC, 32,
+        EGL_MAP_GL_TEXTURE_FORMAT_SEC, EGL_MAP_GL_TEXTURE_RGBA_SEC,
+        EGL_MAP_GL_TEXTURE_PIXEL_TYPE_SEC, EGL_MAP_GL_TEXTURE_UNSIGNED_BYTE_SEC,
+        EGL_NONE
+     };
+   void *egldisplay;
+   
+   pt = calloc(1, sizeof(Evas_GL_Texture_Pool));
+   if (!pt) return NULL;
+   h = _tex_round_slot(gc, h) * gc->shared->info.tune.atlas.slot_size;
+   _tex_adjust(gc, &w, &h);
+   pt->gc = gc;
+   pt->w = w;
+   pt->h = h;
+   pt->intformat = intformat;
+   pt->format = format;
+   pt->dataformat = GL_UNSIGNED_BYTE;
+   pt->render = 1;
+   pt->references = 0;
+   texinfo.d.num++;
+   texinfo.d.pix += pt->w * pt->h;
+   
+   _print_tex_count();
+   
+   glGenTextures(1, &(pt->texture));
+   GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+   glBindTexture(GL_TEXTURE_2D, pt->texture);
+   GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+   GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+   GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+   GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+   GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+
+   egldisplay = pt->gc->egldisp;
+   
+   attr[1] = pt->w;
+   attr[3] = pt->h;
+
+   // FIXME: seems a bit slower than i'd like - maybe too many flushes?
+   // FIXME: YCbCr no support as yet
+   pt->dyn.img = secsym_eglCreateImage(egldisplay,
+                                       EGL_NO_CONTEXT, 
+                                       EGL_MAP_GL_TEXTURE_2D_SEC,
+                                       0, attr);
+   if (!pt->dyn.img)
+     {
+        GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+        return pt;
      }
+   pt->dyn.data = secsym_eglMapImageSEC(egldisplay,
+                                        pt->dyn.img);
+   if (!pt->dyn.data)
+     {
+        secsym_eglDestroyImage(egldisplay, pt->dyn.img);
+        pt->dyn.img = NULL;
+        GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+        return pt;
+     }
+   if (secsym_eglGetImageAttribSEC(egldisplay,
+                                   pt->dyn.img,
+                                   EGL_MAP_GL_TEXTURE_WIDTH_SEC,
+                                   &(pt->dyn.w)) != EGL_TRUE)
+     {
+        secsym_eglDestroyImage(egldisplay, pt->dyn.img);
+        pt->dyn.img = NULL;
+        GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+        return pt;
+     }
+   if (secsym_eglGetImageAttribSEC(egldisplay,
+                                   pt->dyn.img,
+                                   EGL_MAP_GL_TEXTURE_HEIGHT_SEC,
+                                   &(pt->dyn.h)) != EGL_TRUE)
+     {
+        secsym_eglDestroyImage(egldisplay, pt->dyn.img);
+        pt->dyn.img = NULL;
+        GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+        return pt;
+     }
+   if (secsym_eglGetImageAttribSEC(egldisplay,
+                                   pt->dyn.img,
+                                   EGL_MAP_GL_TEXTURE_STRIDE_IN_BYTES_SEC,
+                                   &(pt->dyn.stride)) != EGL_TRUE)
+     {
+        secsym_eglDestroyImage(egldisplay, pt->dyn.img);
+        pt->dyn.img = NULL;
+        GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+        return pt;
+     }
+   if (secsym_eglGetImageAttribSEC(egldisplay,
+                                   pt->dyn.img,
+                                   EGL_MAP_GL_TEXTURE_FORMAT_SEC,
+                                   &(fmt)) != EGL_TRUE)
+     {
+        secsym_eglDestroyImage(egldisplay, pt->dyn.img);
+        pt->dyn.img = NULL;
+        GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+        return pt;
+     }
+   if (fmt != EGL_MAP_GL_TEXTURE_RGBA_SEC)
+     {
+        secsym_eglDestroyImage(egldisplay, pt->dyn.img);
+        pt->dyn.img = NULL;
+        GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+        return pt;
+     }
+   if (secsym_eglGetImageAttribSEC(egldisplay,
+                                   pt->dyn.img,
+                                   EGL_MAP_GL_TEXTURE_PIXEL_TYPE_SEC,
+                                   &(pixtype)) != EGL_TRUE)
+     {
+        secsym_eglDestroyImage(egldisplay, pt->dyn.img);
+        pt->dyn.img = NULL;
+        GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+        return pt;
+     }
+   if (pixtype != EGL_MAP_GL_TEXTURE_UNSIGNED_BYTE_SEC)
+     {
+        secsym_eglDestroyImage(egldisplay, pt->dyn.img);
+        pt->dyn.img = NULL;
+        GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+        return pt;
+     }
+   
+   glBindTexture(GL_TEXTURE_2D, gc->pipe[0].shader.cur_tex);
+   GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+#else
+   gc = NULL;
+   w = 0;
+   h = 0;
+   intformat = 0;
+   format = 0;
+#endif  
+   return pt;
+}
+
+void
+evas_gl_texture_pool_empty(Evas_GL_Texture_Pool *pt)
+{
+   if (!pt->gc) return;
+   
+   if (pt->format == alpha_fmt)
+      {
+         texinfo.a.num--;
+         texinfo.a.pix -= pt->w * pt->h;
+      }
+   else if (pt->format == lum_fmt)
+      {
+         texinfo.v.num--;
+         texinfo.v.pix -= pt->w * pt->h;
+      }
+   else if (pt->dyn.img)
+      {
+         texinfo.d.num--;
+         texinfo.d.pix -= pt->w * pt->h;
+      }
+   else if (pt->render)
+      {
+         texinfo.r.num--;
+         texinfo.r.pix -= pt->w * pt->h;
+      }
+   else if (pt->native)
+      {
+         texinfo.n.num--;
+         texinfo.n.pix -= pt->w * pt->h;
+      }
+   else
+      {
+         texinfo.c.num--;
+         texinfo.c.pix -= pt->w * pt->h;
+      }
+   
+   _print_tex_count();
+   
+#if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
+   if (pt->dyn.img)
+     {
+        void *egldisplay = pt->gc->egldisp;
+        
+        secsym_eglDestroyImage(pt->gc->egldisp, pt->dyn.img);
+        pt->dyn.img = NULL;
+        pt->dyn.data = NULL;
+        pt->dyn.w = 0;
+        pt->dyn.h = 0;
+        pt->dyn.stride = 0;
+     }
+#endif   
    
    glDeleteTextures(1, &(pt->texture));
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
@@ -409,17 +664,41 @@ pt_unref(Evas_GL_Texture_Pool *pt)
      {
         glsym_glDeleteFramebuffers(1, &(pt->fb));
         GLERR(__FUNCTION__, __FILE__, __LINE__, "");
+        pt->fb = 0;
      }
-   memset(pt, 0x22, sizeof(Evas_GL_Texture_Pool)); // mark as freed
+   while (pt->allocations)
+      pt->allocations = 
+      eina_list_remove_list(pt->allocations, pt->allocations);
+   pt->texture = 0;
+   pt->gc = NULL;
+   pt->w = 0;
+   pt->h = 0;
+}
+
+static void
+pt_unref(Evas_GL_Texture_Pool *pt)
+{
+   if (!pt->gc) return;
+   pt->references--;
+   if (pt->references != 0) return;
+
+   if (!((pt->render) || (pt->native)))
+     {
+        if (pt->whole)
+           pt->gc->shared->tex.whole = 
+           eina_list_remove(pt->gc->shared->tex.whole, pt);
+        else
+           pt->gc->shared->tex.atlas [pt->slot][pt->fslot] =
+           eina_list_remove(pt->gc->shared->tex.atlas[pt->slot][pt->fslot], pt);
+     }
+   evas_gl_texture_pool_empty(pt);
    free(pt);
 }
 
 Evas_GL_Texture *
-evas_gl_common_texture_native_new(Evas_GL_Context *gc, int w, int h, int alpha, Evas_GL_Image *im)
+evas_gl_common_texture_native_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, int alpha, Evas_GL_Image *im)
 {
    Evas_GL_Texture *tex;
-   Eina_List *l_after = NULL;
-   int u = 0, v = 0;
 
    tex = calloc(1, sizeof(Evas_GL_Texture));
    if (!tex) return NULL;
@@ -443,7 +722,6 @@ evas_gl_common_texture_native_new(Evas_GL_Context *gc, int w, int h, int alpha,
      }
    if (!tex->pt)
      {
-        memset(tex, 0x33, sizeof(Evas_GL_Texture)); // mark as freed
         free(tex);
         return NULL;
      }
@@ -456,11 +734,9 @@ evas_gl_common_texture_native_new(Evas_GL_Context *gc, int w, int h, int alpha,
 }
 
 Evas_GL_Texture *
-evas_gl_common_texture_render_new(Evas_GL_Context *gc, int w, int h, int alpha)
+evas_gl_common_texture_render_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, int alpha)
 {
    Evas_GL_Texture *tex;
-   Eina_List *l_after = NULL;
-   int u = 0, v = 0;
 
    tex = calloc(1, sizeof(Evas_GL_Texture));
    if (!tex) return NULL;
@@ -484,7 +760,6 @@ evas_gl_common_texture_render_new(Evas_GL_Context *gc, int w, int h, int alpha)
      }
    if (!tex->pt)
      {
-        memset(tex, 0x44, sizeof(Evas_GL_Texture)); // mark as freed
         free(tex);
         return NULL;
      }
@@ -492,6 +767,44 @@ evas_gl_common_texture_render_new(Evas_GL_Context *gc, int w, int h, int alpha)
    tex->y = 0;
    tex->w = w;
    tex->h = h;
+   tex->pt->references++; 
+   return tex;
+}
+
+Evas_GL_Texture *
+evas_gl_common_texture_dynamic_new(Evas_GL_Context *gc, Evas_GL_Image *im)
+{
+   Evas_GL_Texture *tex;
+
+   tex = calloc(1, sizeof(Evas_GL_Texture));
+   if (!tex) return NULL;
+   
+   tex->gc = gc;
+   tex->references = 1;
+   tex->alpha = im->alpha;
+   tex->x = 0;
+   tex->y = 0;
+   tex->w = im->w;
+   tex->h = im->h;
+   if (tex->alpha)
+     {
+        if (gc->shared->info.bgra)
+          tex->pt = _pool_tex_dynamic_new(gc, tex->w, tex->h, bgra_ifmt, bgra_fmt);
+        else
+          tex->pt = _pool_tex_dynamic_new(gc, tex->w, tex->h, bgra_ifmt, bgra_fmt);
+     }
+   else
+     {
+        if (gc->shared->info.bgra)
+          tex->pt = _pool_tex_dynamic_new(gc, tex->w, tex->h, bgra_ifmt, bgra_fmt);
+        else
+          tex->pt = _pool_tex_dynamic_new(gc, tex->w, tex->h, bgra_ifmt, bgra_fmt);
+     }
+   if (!tex->pt)
+     {
+        free(tex);
+        return NULL;
+     }
    tex->pt->references++;
    return tex;
 }
@@ -500,7 +813,7 @@ void
 evas_gl_common_texture_update(Evas_GL_Texture *tex, RGBA_Image *im)
 {
    GLuint fmt;
-   
+
    if (tex->alpha != im->cache_entry.flags.alpha)
      {
         tex->pt->allocations = eina_list_remove(tex->pt->allocations, tex);
@@ -522,6 +835,8 @@ evas_gl_common_texture_update(Evas_GL_Texture *tex, RGBA_Image *im)
           }
      }
    if (!tex->pt) return;
+   if (!im->image.data) return;
+   
    fmt = tex->pt->format;
    glBindTexture(GL_TEXTURE_2D, tex->pt->texture);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
@@ -574,9 +889,9 @@ evas_gl_common_texture_update(Evas_GL_Texture *tex, RGBA_Image *im)
                1, 1,
                fmt, tex->pt->dataformat,
                im->image.data + ((im->cache_entry.h - 1) * im->cache_entry.w) + (im->cache_entry.w - 1));
-   if (tex->pt->texture != tex->gc->shader.cur_tex)
+   if (tex->pt->texture != tex->gc->pipe[0].shader.cur_tex)
      {
-        glBindTexture(GL_TEXTURE_2D, tex->gc->shader.cur_tex);
+        glBindTexture(GL_TEXTURE_2D, tex->gc->pipe[0].shader.cur_tex);
         GLERR(__FUNCTION__, __FILE__, __LINE__, "");
      }
 }
@@ -596,31 +911,27 @@ evas_gl_common_texture_free(Evas_GL_Texture *tex)
      }
    if (tex->ptu) pt_unref(tex->ptu);
    if (tex->ptv) pt_unref(tex->ptv);
-   memset(tex, 0x55, sizeof(Evas_GL_Texture)); // mark as freed
    free(tex);
 }
 
 Evas_GL_Texture *
-evas_gl_common_texture_alpha_new(Evas_GL_Context *gc, DATA8 *pixels, 
-                                 int w, int h, int fh)
+evas_gl_common_texture_alpha_new(Evas_GL_Context *gc, DATA8 *pixels,
+                                 unsigned int w, unsigned int h, int fh)
 {
    Evas_GL_Texture *tex;
    Eina_List *l_after = NULL;
    int u = 0, v = 0;
-   int tw = 4096;
 
    tex = calloc(1, sizeof(Evas_GL_Texture));
    if (!tex) return NULL;
    
    tex->gc = gc;
    tex->references = 1;
-   if (tw > gc->shared->info.max_texture_size)
-     tw = gc->shared->info.max_texture_size;
-   tex->pt = _pool_tex_find(gc, w + 3, fh, alpha_ifmt, alpha_fmt, &u, &v, 
-                            &l_after, tw);
+   tex->pt = _pool_tex_find(gc, w + 3, fh, alpha_ifmt, alpha_fmt, &u, &v,
+                            &l_after, 
+                            gc->shared->info.tune.atlas.max_alloc_alpha_size);
    if (!tex->pt)
      {
-        memset(tex, 0x66, sizeof(Evas_GL_Texture)); // mark as freed
         free(tex);
         return NULL;
      }
@@ -639,8 +950,8 @@ evas_gl_common_texture_alpha_new(Evas_GL_Context *gc, DATA8 *pixels,
 }
 
 void
-evas_gl_common_texture_alpha_update(Evas_GL_Texture *tex, DATA8 *pixels, 
-                                    int w, int h, int fh)
+evas_gl_common_texture_alpha_update(Evas_GL_Texture *tex, DATA8 *pixels,
+                                    unsigned int w, unsigned int h, int fh __UNUSED__)
 {
    if (!tex->pt) return;
    glBindTexture(GL_TEXTURE_2D, tex->pt->texture);
@@ -653,19 +964,17 @@ evas_gl_common_texture_alpha_update(Evas_GL_Texture *tex, DATA8 *pixels,
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    _tex_sub_2d(tex->x, tex->y, w, h, tex->pt->format, tex->pt->dataformat, 
                pixels);
-   if (tex->pt->texture != tex->gc->shader.cur_tex)
+   if (tex->pt->texture != tex->gc->pipe[0].shader.cur_tex)
      {
-        glBindTexture(GL_TEXTURE_2D, tex->gc->shader.cur_tex);
+        glBindTexture(GL_TEXTURE_2D, tex->gc->pipe[0].shader.cur_tex);
         GLERR(__FUNCTION__, __FILE__, __LINE__, "");
      }
 }
 
 Evas_GL_Texture *
-evas_gl_common_texture_yuv_new(Evas_GL_Context *gc, DATA8 **rows, int w, int h)
+evas_gl_common_texture_yuv_new(Evas_GL_Context *gc, DATA8 **rows, unsigned int w, unsigned int h)
 {
    Evas_GL_Texture *tex;
-   Eina_List *l_after = NULL;
-   int u = 0, v = 0;
 
    tex = calloc(1, sizeof(Evas_GL_Texture));
    if (!tex) return NULL;
@@ -675,7 +984,6 @@ evas_gl_common_texture_yuv_new(Evas_GL_Context *gc, DATA8 **rows, int w, int h)
    tex->pt = _pool_tex_new(gc, w + 1, h  + 1, lum_ifmt, lum_fmt);
    if (!tex->pt)
      {
-        memset(tex, 0x77, sizeof(Evas_GL_Texture)); // mark as freed
         free(tex);
         return NULL;
      }
@@ -686,8 +994,6 @@ evas_gl_common_texture_yuv_new(Evas_GL_Context *gc, DATA8 **rows, int w, int h)
    tex->ptu = _pool_tex_new(gc, (w / 2) + 1, (h / 2)  + 1, lum_ifmt, lum_fmt);
    if (!tex->ptu)
      {
-        pt_unref(tex->pt);
-        memset(tex, 0x88, sizeof(Evas_GL_Texture)); // mark as freed
         free(tex);
         return NULL;
      }
@@ -700,7 +1006,6 @@ evas_gl_common_texture_yuv_new(Evas_GL_Context *gc, DATA8 **rows, int w, int h)
      {
         pt_unref(tex->pt);
         pt_unref(tex->ptu);
-        memset(tex, 0x99, sizeof(Evas_GL_Texture)); // mark as freed
         free(tex);
         return NULL;
      }
@@ -723,10 +1028,8 @@ evas_gl_common_texture_yuv_new(Evas_GL_Context *gc, DATA8 **rows, int w, int h)
 }
 
 void
-evas_gl_common_texture_yuv_update(Evas_GL_Texture *tex, DATA8 **rows, int w, int h)
+evas_gl_common_texture_yuv_update(Evas_GL_Texture *tex, DATA8 **rows, unsigned int w, unsigned int h)
 {
-   int y;
-
    if (!tex->pt) return;
    // FIXME: works on lowest size 4 pixel high buffers. must also be multiple of 2
 #ifdef GL_UNPACK_ROW_LENGTH
@@ -748,6 +1051,8 @@ evas_gl_common_texture_yuv_update(Evas_GL_Texture *tex, DATA8 **rows, int w, int
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    _tex_sub_2d(0, 0, w / 2, h / 2, tex->ptv->format, tex->ptv->dataformat, rows[h + (h / 2)]);
 #else
+   int y;
+
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    GLERR(__FUNCTION__, __FILE__, __LINE__, "");
    glBindTexture(GL_TEXTURE_2D, tex->pt->texture);
@@ -780,9 +1085,9 @@ evas_gl_common_texture_yuv_update(Evas_GL_Texture *tex, DATA8 **rows, int w, int
           _tex_sub_2d(0, y, w / 2, 1, tex->ptv->format, tex->ptv->dataformat, rows[h + (h / 2) + y]);
      }
 #endif   
-   if (tex->pt->texture != tex->gc->shader.cur_tex)
+   if (tex->pt->texture != tex->gc->pipe[0].shader.cur_tex)
      {
-        glBindTexture(GL_TEXTURE_2D, tex->gc->shader.cur_tex);
+        glBindTexture(GL_TEXTURE_2D, tex->gc->pipe[0].shader.cur_tex);
         GLERR(__FUNCTION__, __FILE__, __LINE__, "");
      }
 }
