@@ -3,14 +3,10 @@
 "#endif\n"
 "attribute vec4 vertex;\n"
 "attribute vec4 color;\n"
-"attribute vec2 tex_coord;\n"
-"attribute vec2 tex_coord2;\n"
-"attribute vec2 tex_coord3;\n"
+"attribute vec2 tex_coord, tex_coord2, tex_coord3;\n"
 "uniform mat4 mvp;\n"
 "varying vec4 col;\n"
-"varying vec2 tex_c;\n"
-"varying vec2 tex_c2;\n"
-"varying vec2 tex_c3;\n"
+"varying vec2 tex_c, tex_c2, tex_c3;\n"
 "void main()\n"
 "{\n"
 "   gl_Position = mvp * vertex;\n"
