@@ -13,7 +13,7 @@
 #   include <X11/Xatom.h>
 #   include <X11/Xutil.h>
 #   include <X11/extensions/Xrender.h>
-# include <X11/Xresource.h> // xres - dpi
+#   include <X11/Xresource.h> // xres - dpi
 #  elif defined(GLES_VARIETY_SGX)
 #   define SUPPORT_X11 1
 #   include <EGL/egl.h>
@@ -108,7 +108,7 @@ struct _Evas_GL_X11_Window
       unsigned int loose_binding : 1;
    } detected;
 #endif
-
+   int             surf : 1;
 };
 
 Evas_GL_X11_Window *eng_window_new(Display *disp, Window win, int screen,
@@ -117,6 +117,9 @@ Evas_GL_X11_Window *eng_window_new(Display *disp, Window win, int screen,
                                    int alpha, int rot);
 void      eng_window_free(Evas_GL_X11_Window *gw);
 void      eng_window_use(Evas_GL_X11_Window *gw);
+void      eng_window_unsurf(Evas_GL_X11_Window *gw);
+void      eng_window_resurf(Evas_GL_X11_Window *gw);
+
 Visual   *eng_best_visual_get(Evas_Engine_Info_GL_X11 *einfo);
 Colormap  eng_best_colormap_get(Evas_Engine_Info_GL_X11 *einfo);
 int       eng_best_depth_get(Evas_Engine_Info_GL_X11 *einfo);
