@@ -5,13 +5,12 @@
 #include <time.h>
 #include <SDL/SDL.h>
 
-#include "evas_common.h"
+#include "evas_common.h"/* Also includes international specific stuff */
 #include "evas_engine.h"
 int _evas_engine_soft16_sdl_log_dom = -1;
 
 /* function tables - filled in later (func and parent func) */
-static Evas_Func        func = {};
-static Evas_Func        pfunc = {};
+static Evas_Func func, pfunc;
 
 static Engine_Image_Entry       *_sdl16_image_alloc       (void);
 static void                      _sdl16_image_delete      (Engine_Image_Entry *eim);
@@ -19,7 +18,7 @@ static void                      _sdl16_image_delete      (Engine_Image_Entry *e
 static int                       _sdl16_image_constructor (Engine_Image_Entry *ie, void* data);
 static void                      _sdl16_image_destructor  (Engine_Image_Entry *eim);
 
-static void                      _sdl16_image_dirty_region(Engine_Image_Entry *eim, int x, int y, int w, int h);
+static void                      _sdl16_image_dirty_region(Engine_Image_Entry *eim, unsigned int x, unsigned int y, unsigned int w, unsigned int h);
 
 static int                       _sdl16_image_dirty       (Engine_Image_Entry *dst, const Engine_Image_Entry *src);
 
@@ -84,12 +83,12 @@ static void
 _tmp_out_alloc(Render_Engine *re)
 {
    Tilebuf_Rect *r;
-   int w = 0, h = 0;
+   unsigned int w = 0, h = 0;
 
    EINA_INLIST_FOREACH(re->rects, r)
      {
-	if (r->w > w) w = r->w;
-	if (r->h > h) h = r->h;
+	if (r->w > (int)w) w = r->w;
+	if (r->h > (int)h) h = r->h;
      }
 
    if (re->tmp_out)
@@ -129,7 +128,6 @@ _sdl16_output_setup(int w, int h, int rotation, int fullscreen, int noframe, int
    evas_common_convert_init();
    evas_common_scale_init();
    evas_common_rectangle_init();
-   evas_common_gradient_init();
    evas_common_polygon_init();
    evas_common_line_init();
    evas_common_font_init();
@@ -466,14 +464,12 @@ static void
 _tmp_out_process(Render_Engine *re, int out_x, int out_y, int w, int h)
 {
    Soft16_Image *d, *s;
-   DATA16 *dp, *sp;
-   int y, x, d_dir;
 
    d = (Soft16_Image *) re->soft16_engine_image->cache_entry.src;
    s = re->tmp_out;
 
-   if ((w < 1) || (h < 1)
-       || (out_x >= d->cache_entry.w) || (out_y >= d->cache_entry.h))
+   if ((w < 1) || (h < 1) ||
+       (out_x >= (int)d->cache_entry.w) || (out_y >= (int)d->cache_entry.h))
      return;
 
    if (re->rot == 90)
@@ -687,7 +683,7 @@ evas_engine_sdl16_image_data_get(void *data __UNUSED__, void *image,
      eim = (SDL_Engine_Image_Entry *) evas_cache_engine_image_alone(&eim->cache_entry,
                                                                     NULL);
 
-   /* FIXME: Handle colorspace convertion correctly. */
+   /* FIXME: Handle colorspace conversion correctly. */
    if (image_data) *image_data = (DATA32 *) im->pixels;
 
    return eim;
@@ -704,7 +700,7 @@ evas_engine_sdl16_image_data_put(void *data, void *image, DATA32* image_data)
    if (!eim) return NULL;
    im = (Soft16_Image *) eim->cache_entry.src;
 
-   /* FIXME: Handle colorspace convertion correctly. */
+   /* FIXME: Handle colorspace conversion correctly. */
    if ((DATA16 *) image_data == im->pixels) return eim;
 
    eim_new = (SDL_Engine_Image_Entry *) evas_cache_engine_image_data(re->cache,
@@ -805,7 +801,7 @@ evas_engine_sdl16_image_draw(void *data __UNUSED__, void *context, void *surface
 	_SDL_UPDATE_PIXELS(eim);
      }
 
-   soft16_image_draw((Soft16_Image *) eim->cache_entry.src,
+   evas_common_soft16_image_draw((Soft16_Image *) eim->cache_entry.src,
                      (Soft16_Image *) dst->cache_entry.src,
                      context,
                      src_region_x, src_region_y, src_region_w, src_region_h,
@@ -820,17 +816,17 @@ evas_engine_sdl16_image_draw(void *data __UNUSED__, void *context, void *surface
 }
 
 static void
-evas_engine_sdl16_image_map4_draw(void *data __UNUSED__, void *context, void *surface, void *image, RGBA_Map_Point *p, int smooth, int level)
+evas_engine_sdl16_image_map4_draw(void *data __UNUSED__, void *context __UNUSED__, void *surface __UNUSED__, void *image __UNUSED__, RGBA_Map_Point *p __UNUSED__, int smooth __UNUSED__, int level __UNUSED__)
 {
 }
 
 static void
-evas_engine_sdl16_image_scale_hint_set(void *data __UNUSED__, void *image, int hint)
+evas_engine_sdl16_image_scale_hint_set(void *data __UNUSED__, void *image __UNUSED__, int hint __UNUSED__)
 {
 }
 
 static int
-evas_engine_sdl16_image_scale_hint_get(void *data __UNUSED__, void *image)
+evas_engine_sdl16_image_scale_hint_get(void *data __UNUSED__, void *image __UNUSED__)
 {
    return EVAS_IMAGE_SCALE_HINT_NONE;
 }
@@ -877,7 +873,7 @@ evas_engine_sdl16_image_format_get(void *data __UNUSED__, void *image __UNUSED__
 }
 
 static void
-evas_engine_sdl16_font_draw(void *data __UNUSED__, void *context, void *surface, void *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const char *text)
+evas_engine_sdl16_font_draw(void *data __UNUSED__, void *context, void *surface, void *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props)
 {
    static RGBA_Image            *im = NULL;
    SDL_Engine_Image_Entry       *eim = surface;
@@ -896,10 +892,10 @@ evas_engine_sdl16_font_draw(void *data __UNUSED__, void *context, void *surface,
      }
    evas_common_draw_context_font_ext_set(context,
                                          dst,
-                                         soft16_font_glyph_new,
-                                         soft16_font_glyph_free,
-                                         soft16_font_glyph_draw);
-   evas_common_font_draw(im, context, font, x, y, text);
+                                         evas_common_soft16_font_glyph_new,
+                                         evas_common_soft16_font_glyph_free,
+                                         evas_common_soft16_font_glyph_draw);
+   evas_common_font_draw(im, context, font, x, y, text, intl_props);
    evas_common_draw_context_font_ext_set(context,
                                          NULL,
                                          NULL,
@@ -923,7 +919,7 @@ evas_engine_sdl16_line_draw(void *data __UNUSED__, void *context, void *surface,
 	_SDL_UPDATE_PIXELS(eim);
      }
 
-   soft16_line_draw((Soft16_Image *) eim->cache_entry.src,
+   evas_common_soft16_line_draw((Soft16_Image *) eim->cache_entry.src,
                     context,
                     x1, y1, x2, y2);
 
@@ -956,7 +952,7 @@ evas_engine_sdl16_rectangle_draw(void *data __UNUSED__, void *context, void *sur
 
              im = (Soft16_Image *) eim->cache_entry.src;
 
-             soft16_rectangle_draw(im, context, x, y, w, h);
+             evas_common_soft16_rectangle_draw(im, context, x, y, w, h);
 
              if (mustlock_im)
                SDL_UnlockSurface(eim->surface);
@@ -1005,7 +1001,7 @@ evas_engine_sdl16_polygon_draw(void *data __UNUSED__, void *context, void *surfa
 	_SDL_UPDATE_PIXELS(eim);
      }
 
-   soft16_polygon_draw((Soft16_Image *) eim->cache_entry.src, context, polygon, x, y);
+   evas_common_soft16_polygon_draw((Soft16_Image *) eim->cache_entry.src, context, polygon, x, y);
 
    if (mustlock_im)
      SDL_UnlockSurface(eim->surface);
@@ -1022,7 +1018,7 @@ evas_engine_sdl16_image_stride_get(void *data __UNUSED__, void *image, int *stri
 }
 
 static Eina_Bool
-evas_engine_sdl16_canvas_alpha_get(void *data, void *context)
+evas_engine_sdl16_canvas_alpha_get(void *data __UNUSED__, void *context __UNUSED__)
 {
    return EINA_FALSE;
 }
@@ -1034,11 +1030,12 @@ module_open(Evas_Module *em)
    if (!em) return 0;
    /* get whatever engine module we inherit from */
    if (!_evas_module_engine_inherit(&pfunc, "software_16")) return 0;
-   _evas_engine_soft16_sdl_log_dom = eina_log_domain_register("EvasSoft16SDL",EVAS_DEFAULT_LOG_COLOR);
-   if(_evas_engine_soft16_sdl_log_dom < 0)
+   _evas_engine_soft16_sdl_log_dom = eina_log_domain_register
+     ("evas-software_16_sdl", EVAS_DEFAULT_LOG_COLOR);
+   if (_evas_engine_soft16_sdl_log_dom < 0)
      {
-       EINA_LOG_ERR("Impossible to create a log domain for the Soft16_SDL engine.\n");
-       return 0;
+        EINA_LOG_ERR("Can not create a module log domain.");
+        return 0;
      }
 
    /* store it for later use */
@@ -1098,7 +1095,7 @@ module_open(Evas_Module *em)
 }
 
 static void
-module_close(Evas_Module *em)
+module_close(Evas_Module *em __UNUSED__)
 {
   eina_log_domain_unregister(_evas_engine_soft16_sdl_log_dom);
 }
@@ -1170,7 +1167,7 @@ _sdl16_image_destructor(Engine_Image_Entry *eim)
 }
 
 static void
-_sdl16_image_dirty_region(Engine_Image_Entry *eim, int x, int y, int w, int h)
+_sdl16_image_dirty_region(Engine_Image_Entry *eim, unsigned int x, unsigned int y, unsigned int w, unsigned int h)
 {
    SDL_Engine_Image_Entry       *dst;
    RGBA_Image *im;
