@@ -5,14 +5,13 @@
 #include <time.h>
 #include <SDL/SDL.h>
 
-#include "evas_common.h"
+#include "evas_common.h" /* Also includes international specific stuff */
 #include "evas_engine.h"
 
 int _evas_engine_soft_sdl_log_dom = -1;
 /* #define DEBUG_SDL */
 
-static Evas_Func	func = {};
-static Evas_Func	pfunc = {};
+static Evas_Func func, pfunc;
 
 static void*                     _sdl_output_setup	(int w, int h, int fullscreen, int noframe, int alpha, int hwsurface);
 
@@ -22,7 +21,7 @@ static void                      _sdl_image_delete      (Engine_Image_Entry *eim
 static int                       _sdl_image_constructor (Engine_Image_Entry*, void* data);
 static void                      _sdl_image_destructor  (Engine_Image_Entry *eim);
 
-static void                      _sdl_image_dirty_region(Engine_Image_Entry *eim, int x, int y, int w, int h);
+static void                      _sdl_image_dirty_region(Engine_Image_Entry *eim, unsigned int x, unsigned int y, unsigned int w, unsigned int h);
 
 static int                       _sdl_image_dirty       (Engine_Image_Entry *dst, const Engine_Image_Entry *src);
 
@@ -281,7 +280,7 @@ evas_engine_sdl_output_redraws_next_update_push	(void *data, void *surface __UNU
 }
 
 static void
-_sdl_image_dirty_region(Engine_Image_Entry *eim, int x, int y, int w, int h)
+_sdl_image_dirty_region(Engine_Image_Entry *eim, unsigned int x, unsigned int y, unsigned int w, unsigned int h)
 {
    SDL_Engine_Image_Entry       *dst;
    RGBA_Image *im;
@@ -739,7 +738,7 @@ evas_engine_sdl_image_format_get(void *data __UNUSED__, void *image __UNUSED__)
 }
 
 static void
-evas_engine_sdl_font_draw(void *data __UNUSED__, void *context, void *surface, void *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const char *text)
+evas_engine_sdl_font_draw(void *data __UNUSED__, void *context, void *surface, void *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props)
 {
    SDL_Engine_Image_Entry       *eim = surface;
    int                           mustlock_im = 0;
@@ -751,7 +750,7 @@ evas_engine_sdl_font_draw(void *data __UNUSED__, void *context, void *surface, v
 	_SDL_UPDATE_PIXELS(eim);
      }
 
-   evas_common_font_draw((RGBA_Image *) eim->cache_entry.src, context, font, x, y, text);
+   evas_common_font_draw((RGBA_Image *) eim->cache_entry.src, context, font, x, y, text, intl_props);
    evas_common_cpu_end_opt();
 
    if (mustlock_im)
@@ -857,36 +856,17 @@ evas_engine_sdl_polygon_draw(void *data __UNUSED__, void *context, void *surface
      SDL_UnlockSurface(eim->surface);
 }
 
-static void
-evas_engine_sdl_gradient_draw(void *data __UNUSED__, void *context, void *surface, void *gradient, int x, int y, int w, int h)
-{
-   SDL_Engine_Image_Entry       *eim = surface;
-   int                           mustlock_im = 0;
-
-   if (eim->surface && SDL_MUSTLOCK(eim->surface))
-     {
-        mustlock_im = 1;
-	SDL_LockSurface(eim->surface);
-	_SDL_UPDATE_PIXELS(eim);
-     }
-
-   evas_common_gradient_draw((RGBA_Image *) eim->cache_entry.src, context, x, y, w, h, gradient);
-   evas_common_cpu_end_opt();
-
-   if (mustlock_im)
-     SDL_UnlockSurface(eim->surface);
-}
-
 static int module_open(Evas_Module *em)
 {
    if (!em) return 0;
    /* get whatever engine module we inherit from */
    if (!_evas_module_engine_inherit(&pfunc, "software_generic")) return 0;
-   _evas_engine_soft_sdl_log_dom = eina_log_domain_register("EvasSoftSdl",EVAS_DEFAULT_LOG_COLOR);
-   if(_evas_engine_soft_sdl_log_dom < 0) 
+   _evas_engine_soft_sdl_log_dom = eina_log_domain_register
+     ("evas-software_sdl", EVAS_DEFAULT_LOG_COLOR);
+   if (_evas_engine_soft_sdl_log_dom < 0)
      {
-       EINA_LOG_ERR("Impossible to create a log domain for the SoftSdl engine.\n");
-       return 0;
+        EINA_LOG_ERR("Can not create a module log domain.");
+        return 0;
      }
    /* store it for later use */
    func = pfunc;
@@ -935,17 +915,16 @@ static int module_open(Evas_Module *em)
    ORD(line_draw);
    ORD(rectangle_draw);
    ORD(polygon_draw);
-   ORD(gradient_draw);
-   
+
    ORD(image_scale_hint_set);
    ORD(image_scale_hint_get);
-   
+
    /* now advertise out own api */
    em->functions = (void *)(&func);
    return 1;
 }
 
-static void module_close(Evas_Module *em)
+static void module_close(Evas_Module *em __UNUSED__)
 {
   eina_log_domain_unregister(_evas_engine_soft_sdl_log_dom);
 }
@@ -982,7 +961,6 @@ _sdl_output_setup		(int w, int h, int fullscreen, int noframe, int alpha, int hw
    evas_common_convert_init();
    evas_common_scale_init();
    evas_common_rectangle_init();
-   evas_common_gradient_init();
    evas_common_polygon_init();
    evas_common_line_init();
    evas_common_font_init();
@@ -1210,14 +1188,14 @@ _sdl_image_debug(const char* context, Engine_Image_Entry* eie)
    DBG("*** %s image (%p) ***", context, eim);
    if (eim)
      {
-        DBG ("* W: %i\n* H: %i\n* R: %i", eim->cache_entry.w, eim->cache_entry.h, eim->cache_entry.references);
+        DBG("W: %i, H: %i, R: %i", eim->cache_entry.w, eim->cache_entry.h, eim->cache_entry.references);
         if (eim->cache_entry.src)
-          DBG ("* Pixels: %p\n* SDL Surface: %p",((RGBA_Image*) eim->cache_entry.src)->image.data, eim->surface);
+          DBG("Pixels: %p, SDL Surface: %p",((RGBA_Image*) eim->cache_entry.src)->image.data, eim->surface);
         if (eim->surface)
-          DBG ("* Surface->pixels: %p", eim->surface->pixels);
-	DBG ("* Key: %s", eim->cache_entry.cache_key);
-        DBG ("* Reference: %i", eim->cache_entry.references);
+          DBG("Surface->pixels: %p", eim->surface->pixels);
+	DBG("Key: %s", eim->cache_entry.cache_key);
+        DBG("Reference: %i", eim->cache_entry.references);
      }
-   DBG ("*** ***");
+   DBG("*** ***");
 }
 #endif
