@@ -475,7 +475,6 @@ eng_setup(Evas *e, void *in)
         evas_common_convert_init();
         evas_common_scale_init();
         evas_common_rectangle_init();
-        evas_common_gradient_init();
         evas_common_polygon_init();
         evas_common_line_init();
         evas_common_font_init();
@@ -875,11 +874,12 @@ module_open(Evas_Module *em)
 
    /* get whatever engine module we inherit from */
    if (!_evas_module_engine_inherit(&pfunc, "software_generic")) return 0;
-   _evas_engine_soft_x11_log_dom = eina_log_domain_register("EvasSoftX11", EVAS_DEFAULT_LOG_COLOR);
-   if(_evas_engine_soft_x11_log_dom < 0)
+   _evas_engine_soft_x11_log_dom = eina_log_domain_register
+     ("evas-software_x11", EVAS_DEFAULT_LOG_COLOR);
+   if (_evas_engine_soft_x11_log_dom < 0)
      {
-       EINA_LOG_ERR("Impossible to create a log domain for the SoftX11 engine.\n");
-       return 0;
+        EINA_LOG_ERR("Can not create a module log domain.");
+        return 0;
      }
    /* store it for later use */
    func = pfunc;
