@@ -67,12 +67,16 @@ __xre_xcb_image_find(char *fkey)
 }
 
 XR_Image *
-_xre_xcb_image_load(Ximage_Info *xinf, const char *file, const char *key, Evas_Image_Load_Opts *lo)
+_xre_xcb_image_load(Ximage_Info *xinf, const char *file, const char *key, Evas_Image_Load_Opts *lo, int *error)
 {
    char      buf[4096];
    XR_Image *im;
 
-   if (!file) return NULL;
+   if (!file)
+     {
+	*error = EVAS_LOAD_ERROR_GENERIC;
+	return NULL;
+     }
    if (!lo)
      {
 	if (key)
@@ -90,12 +94,17 @@ _xre_xcb_image_load(Ximage_Info *xinf, const char *file, const char *key, Evas_I
    im = __xre_xcb_image_find(buf);
    if (im)
      {
+	*error = EVAS_LOAD_ERROR_NONE;
 	return im;
      }
 
    im = calloc(1, sizeof(XR_Image));
-   if (!im) return NULL;
-   im->im = evas_common_load_image_from_file(file, key, lo);
+   if (!im)
+     {
+	*error = EVAS_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED;
+	return NULL;
+     }
+   im->im = evas_common_load_image_from_file(file, key, lo, error);
    if (!im->im)
      {
 	free(im);
@@ -111,6 +120,7 @@ _xre_xcb_image_load(Ximage_Info *xinf, const char *file, const char *key, Evas_I
    im->h = im->im->cache_entry.h;
    im->references = 1;
    if (lo) im->load_opts = *lo;
+   im->load_error = error; /* points to object's load_error */
    if (im->im->info.comment) im->comment = (char *)eina_stringshare_add(im->im->info.comment);
 /*    if (im->im->info.format == 1) im->format = eina_stringshare_add("png"); */
    if (im->im->cache_entry.flags.alpha) im->alpha = 1;
@@ -300,7 +310,7 @@ _xre_xcb_image_copy(XR_Image *im)
    else
      {
 	if (!im->im)
-          im->im = evas_common_load_image_from_file(im->file, im->key, &(im->load_opts));
+          im->im = evas_common_load_image_from_file(im->file, im->key, &(im->load_opts), im->load_error);
 	if (im->im)
 	  {
              evas_cache_image_load_data(&im->im->cache_entry);
@@ -391,7 +401,7 @@ _xre_xcb_image_data_get(XR_Image *im)
    else if (im->cs.data) data = im->cs.data;
    else
      {
-	if (!im->im) im->im = evas_common_load_image_from_file(im->file, im->key, &(im->load_opts));
+	if (!im->im) im->im = evas_common_load_image_from_file(im->file, im->key, &(im->load_opts), im->load_error);
 	if (im->im)
 	  {
              evas_cache_image_load_data(&im->im->cache_entry);
@@ -560,7 +570,7 @@ _xre_xcb_image_surface_gen(XR_Image *im)
    if (im->data) data = im->data;
    else
      {
-	if (!im->im) im->im = evas_common_load_image_from_file(im->file, im->key, &(im->load_opts));
+	if (!im->im) im->im = evas_common_load_image_from_file(im->file, im->key, &(im->load_opts), im->load_error);
 	if (im->im)
 	  {
              evas_cache_image_load_data(&im->im->cache_entry);
