@@ -1,6 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
 #include "evas_common.h"
 //#include "evas_macros.h"
 #include "evas_private.h"
@@ -396,7 +393,7 @@ _xr_xlib_render_surface_composite(Xrender_Surface *srs, Xrender_Surface *drs, RG
 
    if ((dc) && (dc->mul.use))
      {
-	int r, g, b, a;
+	unsigned int r, g, b, a;
 
 	if ((op == PictOpOver) && (!dc->mul.col)) return;
 	a = dc->mul.col >> 24;
