@@ -1,12 +1,7 @@
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
 #ifdef HAVE_EVIL
 # include <Evil.h>
 #endif
@@ -181,6 +176,8 @@ pmaps_buffer_open(Pmaps_Buffer *b, const char *filename, int *error)
    if (len < 3)
      {
 	*error = EVAS_LOAD_ERROR_CORRUPT_FILE;
+        fclose(b->file);
+        b->file = NULL;
 	return EINA_FALSE;
      }
 
@@ -557,7 +554,7 @@ module_open(Evas_Module *em)
 }
 
 static void
-module_close(Evas_Module *em)
+module_close(Evas_Module *em __UNUSED__)
 {
 }
 
