@@ -1,12 +1,7 @@
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
 #include <stdio.h>
 #include <png.h>
 #include <setjmp.h>
@@ -18,7 +13,7 @@
 #ifdef _WIN32_WCE
 # define E_FOPEN(file, mode) evil_fopen_native((file), (mode))
 # define E_FREAD(buffer, size, count, stream) evil_fread_native(buffer, size, count, stream)
-# define/ E_FCLOSE(stream) evil_fclose_native(stream)
+# define E_FCLOSE(stream) evil_fclose_native(stream)
 #else
 # define E_FOPEN(file, mode) fopen((file), (mode))
 # define E_FREAD(buffer, size, count, stream) fread(buffer, size, count, stream)
@@ -151,7 +146,11 @@ evas_image_load_file_data_png(Image_Entry *ie, const char *file, const char *key
      }
 
    /* if we havent read the header before, set the header data */
-   E_FREAD(buf, 1, PNG_BYTES_TO_CHECK, f);
+   if (E_FREAD(buf, PNG_BYTES_TO_CHECK, 1, f) != 1)
+     {
+	*error = EVAS_LOAD_ERROR_CORRUPT_FILE;
+        goto close_file;
+     }
    if (png_sig_cmp(buf, 0, PNG_BYTES_TO_CHECK))
      {
 	*error = EVAS_LOAD_ERROR_CORRUPT_FILE;
@@ -257,7 +256,7 @@ module_open(Evas_Module *em)
 }
 
 static void
-module_close(Evas_Module *em)
+module_close(Evas_Module *em __UNUSED__)
 {
 }
 
