@@ -104,14 +104,17 @@ evas_image_load_file_head_svg(Image_Entry *ie, const char *file, const char *key
    else if ((ie->load_opts.w > 0) &&
 	    (ie->load_opts.h > 0))
      {
-	int w2, h2;
+	unsigned int w2, h2;
 	
 	w2 = ie->load_opts.w;
 	h2 = (ie->load_opts.w * h) / w;
 	if (h2 > ie->load_opts.h)
 	  {
+	     unsigned int w3;
 	     h2 = ie->load_opts.h;
-	     w2 = (ie->load_opts.h * w) / h;
+	     w3 = (ie->load_opts.h * w) / h;
+	     if (w3 > w2)
+	        w2 = w3;
 	  }
 	w = w2;
 	h = h2;
@@ -181,7 +184,7 @@ evas_image_load_file_data_svg(Image_Entry *ie, const char *file, const char *key
    else if ((ie->load_opts.w > 0) &&
 	    (ie->load_opts.h > 0))
      {
-	int w2, h2;
+	unsigned int w2, h2;
 	
 	w2 = ie->load_opts.w;
 	h2 = (ie->load_opts.w * h) / w;
@@ -249,7 +252,7 @@ module_open(Evas_Module *em)
 }
 
 static void
-module_close(Evas_Module *em)
+module_close(Evas_Module *em __UNUSED__)
 {
    if (!rsvg_initialized) return;
    //rsvg_term();
